implementation module Array where

import Ix
from StdMisc import abort
from StdInt import qualified instance + Int,instance one Int
import _SystemArray

infixl 9 !,/&/

data Array a b
	= Array !(!a,!a) !{b}

array :: (Ix a) => !(!a,!a) -> ![(a,b)] -> Array a b
array b ivs
	= Array b (fill_array ivs b (createArray (rangeSize b) undefined_array_element))

array`Int :: !(!Int,!Int) -> ![(Int,b)] -> Array Int b
array`Int b@(l,u) ivs
	| l==0
		= Array b (fill_array_0 ivs u (createArray (u+1) undefined_array_element))
		= Array b (fill_array ivs l u (createArray (u-l+1) undefined_array_element))
  where
	fill_array_0 :: ![(Int,b)] -> !Int -> !*{b} -> {b}
	fill_array_0 ((i,v):ivs) u a
		| unsigned_less_or_equal i u
			# a = {a & [i]=v}
			= fill_array_0 ivs u a
			= array_error
	fill_array_0 [] u a
		= a

	fill_array :: ![(Int,b)] -> !Int -> !Int -> !*{b} -> {b}
	fill_array ((i,v):ivs) l u a
		| i>=l && i<=u
			# a = {a & [i-l]=v}
			= fill_array ivs l u a
			= array_error
	fill_array [] l u a
		= a

array_error
	= abort `"Array.array: out-of-range array association"

listArray :: (Ix a) => !(!a,!a) -> ![b] -> Array a b
listArray b l
	= Array b (fill_array l 0 (createArray (rangeSize b) undefined_array_element))
  where
	fill_array :: ![a] -> !Int -> !*{a} -> *{a}
	fill_array (e:l) i a
		| i<size a
			# a = {a & [i]=e}
			= fill_array l (i+1) a
			= abort `"Array.listArray: out-of-range array association"
	fill_array [] i a
		= a

undefined_array_element :: a
undefined_array_element
	= abort `"(Array.!): undefined array element"

(!) :: (Ix a) => !(Array a b) -> a -> b
(Array b a) ! i
--	= a~[index b i]; inefficient code for a=Int
	# i=index b i
	= a~[i];

subscript`Int :: !(Array Int b) -> !Int -> b
subscript`Int (Array (l,u) a) i
	| i>=l && i<=u
		= a~[i-l]
		= index_out_of_range

index_out_of_range
	= abort `"Ix.index: Index out of range."

bounds :: (Ix a) => !(Array a b) -> (!a,!a)
bounds (Array b _) = b

indices :: (Ix a) => !(Array a b) -> [a]
indices (Array b _) = range b

elems :: (Ix a) => !(Array a b) -> [b]
elems (Array _ es)
	= array_to_list 0 es
	where
		array_to_list i a
			| i<size a
				#! (e,a) = a?[i]
				#! i=i+1
				= e : array_to_list i a
				= []

assocs :: (Ix a) => !(Array a b) -> [(a,b)]
assocs (Array b a)
	= assocs 0 a (range b)
  where
	assocs :: !Int -> !{b} -> ![a] -> [(a,b)]
	assocs i a (j:js)
		| i<size a
			#! (e,a) = a?[i]
			#! i=i+1
			= (j,e) : assocs i a js
	assocs i a _
		= []

assocs`Int :: !(Array Int b) -> [(Int,b)]
assocs`Int (Array (l,u) a)
	| l==0
		= assocs_0 0 a
		= assocs 0 a l
  where
	assocs_0 :: !Int -> !{a} -> [(Int,a)]
	assocs_0 i a
		| i<size a
			#! (e,a) = a?[i]
			#! i=i+1
			= (i,e) : assocs_0 i a
			= []

	assocs :: !Int -> !{a} -> !Int -> [(Int,a)]
	assocs i a l
		| i<size a
			#! (e,a) = a?[i]
			#! i1=i+1
			= (i,e) : assocs i1 a l
			= []


(/&/) :: (Ix a) => !(Array a b) -> ![(a,b)] -> Array a b
(Array b a) /&/ ivs
	= Array b (fill_array ivs b (copy_array a))

updateArray`Int :: !(Array Int b) -> ![(Int,b)] -> Array Int b
updateArray`Int (Array b@(l,u) a) ivs
	# a = copy_array a
	| l==0
		= Array b (update_array_0 ivs u a)
		= Array b (update_array ivs l u a)
  where
	update_array_0 :: ![(Int,b)] -> !Int -> !*{b} -> {b}
	update_array_0 ((i,v):ivs) u a
		| unsigned_less_or_equal i u
			# a = {a & [i]=v}
			= update_array_0 ivs u a
			= array_update_error
	update_array_0 [] u a
		= a

	update_array :: ![(Int,b)] -> !Int -> !Int -> !*{b} -> {b}
	update_array ((i,v):ivs) l u a
		| i>=l && i<=u
			# a = {a & [i-l]=v}
			= update_array ivs l u a
			= array_update_error
	update_array [] l u a
		= a

array_update_error
	= abort `"Array.//: out-of-range array association"

accum :: (Ix a) => (b -> c -> b) -> !(Array a b) -> ![(a,c)] -> Array a b
accum f (Array b a) jvs
	= Array b (accum_array jvs (copy_array a) b f)

accum`Int :: (b -> c -> b) -> !(Array Int b) -> ![(Int,c)] -> Array Int b
accum`Int f (Array b@(l,u) a) jvs
	# a = copy_array a
	| l==0
		= Array b (accum_array_int_0 jvs a u f)
		= Array b (accum_array_int jvs a l u f)

accumArray :: (Ix a) => (b -> c -> b) -> b -> !(!a,!a) -> ![(a,c)] -> Array a b
accumArray f z b jvs
	= Array b (accum_array jvs (createArray (rangeSize b) z) b f)

accumArray`Int :: (b -> c -> b) -> b -> !(!Int,!Int) -> ![(Int,c)] -> Array Int b
accumArray`Int f z b@(l,u) jvs
	# a = createArray (rangeSize b) z
	| l==0
		= Array b (accum_array_int_0 jvs a u f)
		= Array b (accum_array_int jvs a l u f)

copy_array a
	= {e ;; e<-:a}

fill_array :: (Ix a) => ![(a,.b)] -> (a,a) -> !*{.b} -> {.b}
fill_array ((i,v):ivs) b a
	# i = index b i
	  a = {a & [i]=v}
	= fill_array ivs b a
fill_array [] b a
	= a

accum_array ((j,n):jvs) a b f
	#! i = index b j
	#! (e,a) = a?[i]
	#! a = {a & [i] = f e n}
	= accum_array jvs a b f
accum_array [] a b f
	= a

accum_array_int :: ![(Int,b)] -> !*{a} -> !Int -> !Int -> (a->b->a) -> *{a}
accum_array_int ((j,n):jvs) a l u f
	| j>=l && j<=u
		# i = j-l
		# (e,a) = a?[i]
		#! a = {a & [i] = f e n}
		= accum_array_int jvs a l u f
		= accum_error
accum_array_int [] a l u f
	= a

accum_array_int_0 :: ![(Int,b)] -> !*{a} -> !Int -> (a->b->a) -> *{a}
accum_array_int_0 ((i,n):jvs) a u f
	| unsigned_less_or_equal i u
		# (e,a) = a?[i]
		#! a = {a & [i] = f e n}
		= accum_array_int_0 jvs a u f
		= accum_error
accum_array_int_0 [] a u f
	= a

accum_error
	=> abort `"Array.accum: out-of-range array association"

ixmap :: (Ix a, Ix b) => !(!a,!a) -> (a -> b) -> Array b c -> Array a c
ixmap b f a0
	# a1 = createArray (rangeSize b) undefined_array_element
	= Array b (fill_ixmap_array (range b) 0 a1 a0 (g f))
  where
  	g f a0 j = a0 ! f j

	fill_ixmap_array :: ![a] -> !Int -> !*{b} -> c -> (c -> a -> b) -> *{b}
	fill_ixmap_array (j:js) i a1 a0 h
  		# a1 = {a1 & [i] = h a0 j}
  		= fill_ixmap_array js (i+1) a1 a0 h
	fill_ixmap_array [] i a1 a0 h
		= a1

ixmap`IntInt :: !(!Int,!Int) -> (Int -> Int) -> Array Int c -> Array Int c
ixmap`IntInt b@(l,u) f a0
	| l==0
		# a1 = createArray (u+1) undefined_array_element
		= Array b (fill_ixmap_array_0 0 a1 a0 (g f))
		# a1 = createArray (u-l+1) undefined_array_element
		= Array b (fill_ixmap_array 0 l a1 a0 (g f))
  where
  	g f a0 j = subscript`Int a0 (f j)

	fill_ixmap_array_0 i a1 a0 h
		| i<size a1
	  		# a1 = {a1 & [i] = h a0 i}
  			= fill_ixmap_array_0 (i+1) a1 a0 h
			= a1

	fill_ixmap_array i l a1 a0 h
		| i<size a1
	  		# a1 = {a1 & [i] = h a0 (i-l)}
  			= fill_ixmap_array (i+1) l a1 a0 h
			= a1

instance (Ix a) => Functor (Array a) where
	fmap :: (Ix i) => (a -> b) -> !(Array i a) -> Array i b
	fmap f (Array b a)
		= Array b {f e ;; e<-:a}

instance (Ix a, Eq b) => Eq (Array a b) where
	a0 == a1 = assocs a0 == assocs a1
	a0 /= a1 = assocs a0 /= assocs a1

instance  (Ix a, Ord b) => Ord (Array a b)  where
    compare a b = compare (assocs a) (assocs b)

instance  (Ix a, Show a, Show b) => Show (Array a b)  where
    showsPrec p a
    	= showParen (p > 9)
    		(showString "array " . shows (bounds a) . showChar ' ' . shows (assocs a))

instance (Ix a, Read a, Read b) => Read (Array a b) where
	readsPrec p
		= readParen (p > 9)
			(\r -> [(array b as, u) |	("array",s) <- lex r,
										(b,t) <- readsPrec 10 s,
										(as,u) <- readsPrec 10 t])

unsigned_less_or_equal :: !Int -> !Int -> Bool
unsigned_less_or_equal x y
	= `code inline {
			gtU
			notB
		}
