implementation module Complex where

infix 6 :+

data {- (RealFloat a) => -}
	Complex a = !a :+ !a

instance (Eq a) => Eq (Complex a) where
	(==) ::  (Eq a) => !(Complex a) -> !(Complex a) -> Bool
	(r1 :+ i1) == (r2 :+ i2) = r1==r2 && i1==i2
	(/=) ::  (Eq a) => !(Complex a) -> !(Complex a) -> Bool
	(r1 :+ i1) /= (r2 :+ i2) = r1/=r2 || i1/=i2

instance (Show a) => Show (Complex a) where
	show :: (Show a) => !(Complex a) -> [Char]
	show (r:+i)
		= showsPrec 7 i [' :+ ' ; showsPrec 7 r ""]

	showsPrec :: (Show a) => !Int -> !(Complex a) -> ([Char]->[Char])
	showsPrec p (r:+i)
		| p>6
			= (\t -> '(': (\t -> showsPrecComplex r i (')' : t)) t)
			= showsPrecComplex r i
		where
			showsPrecComplex r i t
				= showsPrec 7 i [' :+ ' ; showsPrec 7 r t]

instance (Read a) => Read (Complex a) where
	readsPrec :: (Read a) => !Int -> ![Char] -> [(Complex a,[Char])]
	readsPrec p s
		= readsPrecR_Complex p s (readsOpenParensPrec p s)
	readsOpenParensPrec p s
		= readsOpenParensPrecR_Complex p s (readsOpenParensPrec p s) 

readsPrecR_Complex p s r0
	= case skip_spaces` s of
		'(':s
			# (_:rt0) = r0
			-> add_non_left_rec_infixs_Complex p (read_close` (readsPrecR_Complex 0 s rt0)) r0
		_
			-> read_infix_0 p r0

readsOpenParensPrecR_Complex p s r0
	= case skip_spaces` s of
		'(':s
			# (_:rt0) = r0
			  rl@(r:_) = readsOpenParensPrecR_Complex 0 s rt0
			-> add_non_left_rec_infixs_Complex p (read_close` r) r0 : rl 
		_
			-> [read_infix_0 p r0] 

add_non_left_rec_infixs_Complex p r r0
	= read_infix_0 p r0 ++ r

read_infix_0 p a0
	| p>`6
		= []
		# (a0:_) = a0
		= c_Complex_0 (skip_spaces`` a0)
where
	c_Complex_0 ((a0,':':'+':s) : t)
		| not_begins_with_id_char` s
			= c_Complex_1 t a0 (readsPrec `7 s) 
	c_Complex_0 (_ : t)
		= c_Complex_0 t
	c_Complex_0 _
		= []
	
	c_Complex_1 t a0 ((a1,s) : r)
		= (a0 :+ a1 ,s) : c_Complex_1 t a0 r
	c_Complex_1 t a0 []
		= c_Complex_0 t

conjugate :: (RealFloat a) => !(Complex a) -> Complex a
conjugate (x:+y) = x :+ (-y)

mkPolar :: (RealFloat a) => a -> a -> Complex a
mkPolar r theta = r * cos theta :+ r * sin theta

mkPolar`Double :: !Double -> !Double -> Complex Double
mkPolar`Double r theta = r * cos theta :+ r * sin theta

mkPolar`Float :: !Float -> !Float -> Complex Float
mkPolar`Float r theta = r * cos theta :+ r * sin theta

cis :: (RealFloat a) => a -> Complex a
cis theta = cos theta :+ sin theta

cis`Double :: !Double -> Complex Double
cis`Double theta = cos theta :+ sin theta

cis`Float :: !Float -> Complex Float
cis`Float theta = cos theta :+ sin theta

polar :: (RealFloat a) => Complex a -> (a,a)
polar z = (magnitude z, phase z)

polar`Double :: Complex Double -> (Double,Double)
polar`Double z = (magnitude`Double z, phase z)

polar`Float :: Complex Float -> (Float,Float)
polar`Float z = (magnitude`Float z, phase z)

magnitude :: (RealFloat a) => !Complex a -> a
magnitude (x:+y)
	= scaleFloat k (sqrt ((scaleFloat mk x)^(2::Int) + (scaleFloat mk y)^(2::Int)))
	where
		k = max (exponent x) (exponent y)
		mk = - k

magnitude`Double :: !Complex Double -> Double
magnitude`Double (x:+y)
	= scaleFloat k (sqrt (xs*xs + ys*ys))
	where
		k = max (exponent x) (exponent y)
		mk = - k
		xs = scaleFloat mk x
		ys = scaleFloat mk y

magnitude`Float :: !Complex Float -> Float
magnitude`Float (x:+y)
	= scaleFloat k (sqrt (xs*xs + ys*ys))
	where
		k = max (exponent x) (exponent y)
		mk = - k
		xs = scaleFloat mk x
		ys = scaleFloat mk y

phase :: (RealFloat a) => !Complex a -> a
phase (0 :+ 0) = 0
phase (x :+ y) = atan2 y x

instance (RealFloat a) => Num (Complex a) where
	(+) :: (RealFloat a) => (Complex a) -> !(Complex a) -> Complex a
	(x:+y) + (x':+y') = (x+x') :+ (y+y')
	(-) :: (RealFloat a) => (Complex a) -> !(Complex a) -> Complex a
	(x:+y) - (x':+y') = (x-x') :+ (y-y')
	(*) :: (RealFloat a) => (Complex a) -> !(Complex a) -> Complex a
	(x:+y) * (x':+y') = (x*x'-y*y') :+ (x*y'+y*x')
	negate :: (RealFloat a) => !(Complex a) -> Complex a
	negate (x:+y) = negate x :+ negate y
	abs :: (RealFloat a) => !(Complex a) -> Complex a
	abs z = magnitude z :+ 0
	signum :: (RealFloat a) => !(Complex a) -> Complex a
--	signum 0 = 0
	signum (0:+0) = 0
	signum z@(x:+y) = x/r :+ y/r where r = magnitude z
	fromInteger :: (RealFloat a) => !Integer -> Complex a
	fromInteger n = fromInteger n :+ 0

abs`ComplexDouble :: !(Complex Double) -> Complex Double
abs`ComplexDouble z = magnitude`Double z :+ 0

abs`ComplexFloat :: !(Complex Float) -> Complex Float
abs`ComplexFloat z = magnitude`Float z :+ 0

signum`ComplexDouble :: !(Complex Double) -> Complex Double
signum`ComplexDouble (0:+0) = 0
signum`ComplexDouble z@(x:+y) = x/r :+ y/r where r = magnitude`Double z

signum`ComplexFloat :: !(Complex Float) -> Complex Float
signum`ComplexFloat (0:+0) = 0
signum`ComplexFloat z@(x:+y) = x/r :+ y/r where r = magnitude`Float z

instance (RealFloat a) => Fractional (Complex a) where
	(/) :: (RealFloat a) => !(Complex a) -> !(Complex a) -> Complex a	
	(x:+y) / (x':+y') = (x*x''+y*y'') / d :+ (y*x''-x*y'') / d
	where
		x'' = scaleFloat k x'
		y'' = scaleFloat k y'
		k = - max (exponent x') (exponent y')
		d = x'*x'' + y'*y''
	fromRational :: (RealFloat a) => !Rational -> Complex a	
	fromRational a = fromRational a :+ 0

instance (RealFloat a) => Floating (Complex a) where
	pi :: (RealFloat a) => Complex a
	pi = pi :+ 0
	exp :: (RealFloat a) => !(Complex a) -> Complex a
	exp (x:+y) = cexp (exp x) y
	  where
		cexp expx y = expx * cos y :+ expx * sin y
	log :: (RealFloat a) => !(Complex a) -> Complex a
	log z = log (magnitude z) :+ phase z
	sqrt :: (RealFloat a) => !(Complex a) -> Complex a
--	sqrt 0 = 0
	sqrt (0:+0) = 0
	sqrt z@(x:+y) = u :+ (if y < 0 then -v else v)
	  where
	  	(u,v) = if x < 0 then (v',u') else (u',v')
		v' = abs y / (u'*2)
		u' = sqrt ((magnitude z + abs x) / 2)
	sin :: (RealFloat a) => !(Complex a) -> Complex a
	sin (x:+y) = sin x * cosh y :+ cos x * sinh y
	cos :: (RealFloat a) => !(Complex a) -> Complex a
	cos (x:+y) = cos x * cosh y :+ (- sin x * sinh y)
	tan :: (RealFloat a) => !(Complex a) -> Complex a
	tan (x:+y) = (sinx*coshy:+cosx*sinhy)/(cosx*coshy:+(-sinx*sinhy))
	  where
	  	sinx = sin x
		cosx = cos x
		sinhy = sinh y
		coshy = cosh y
	sinh :: (RealFloat a) => !(Complex a) -> Complex a
	sinh (x:+y) = cos y * sinh x :+ sin y * cosh x
	cosh :: (RealFloat a) => !(Complex a) -> Complex a
	cosh (x:+y) = cos y * cosh x :+ sin y * sinh x
	tanh :: (RealFloat a) => !(Complex a) -> Complex a
	tanh (x:+y) = (cosy*sinhx:+siny*coshx)/(cosy*coshx:+siny*sinhx)
	  where
		siny = sin y
		cosy = cos y
		sinhx = sinh x
		coshx = cosh x
	asin :: (RealFloat a) => !(Complex a) -> Complex a
	asin z@(x:+y) = y':+(-x')
		where (x':+y') = log (((-y):+x) + sqrt (1 - z*z))
	acos :: (RealFloat a) => !(Complex a) -> Complex a
	acos z@(x:+y) = y'':+(-x'')
	  where
		(x'':+y'') = log (z + ((-y'):+x'))
		(x':+y') = sqrt (1 - z*z)
	atan :: (RealFloat a) => !(Complex a) -> Complex a
	atan z@(x:+y) = y':+(-x')
		where (x':+y') = log (((1-y):+x) / sqrt (1+z*z))
	asinh :: (RealFloat a) => !(Complex a) -> Complex a
	asinh z = log (z + sqrt (1+z*z))
	acosh :: (RealFloat a) => !(Complex a) -> Complex a
	acosh z = log (z + (z+1) * sqrt ((z-1)/(z+1)))
	atanh :: (RealFloat a) => !(Complex a) -> Complex a
	atanh z = log ((1+z) / sqrt (1-z*z))

sin`ComplexDouble :: !(Complex Double) -> Complex Double
sin`ComplexDouble (x:+y) = sin_double_float x (sincosh y)

sin`ComplexFloat :: !(Complex Float) -> Complex Float
sin`ComplexFloat (x:+y) = sin_double_float x (sincosh y)

sin_double_float x (sinhy,coshy) = sin x * coshy :+ cos x * sinhy

cos`ComplexDouble :: !(Complex Double) -> Complex Double
cos`ComplexDouble (x:+y) = cos_double_float x (sincosh y)

cos`ComplexFloat :: !(Complex Float) -> Complex Float
cos`ComplexFloat (x:+y) = cos_double_float x (sincosh y)

cos_double_float x (sinhy,coshy) = cos x * coshy :+ (- sin x * sinhy)

tan`ComplexDouble :: !(Complex Double) -> Complex Double
tan`ComplexDouble (x:+y) = tan_double_float x (sincosh y)

tan`ComplexFloat :: !(Complex Float) -> Complex Float
tan`ComplexFloat (x:+y) = tan_double_float x (sincosh y)

tan_double_float x (sinhy,coshy)
	= (sinx*coshy:+cosx*sinhy)/(cosx*coshy:+(-sinx*sinhy))
  where
  	sinx = sin x
	cosx = cos x

sinh`ComplexDouble :: !(Complex Double) -> Complex Double
sinh`ComplexDouble (x:+y) = sinh_double_float y (sincosh x)

sinh`ComplexFloat :: !(Complex Float) -> Complex Float
sinh`ComplexFloat (x:+y) = sinh_double_float y (sincosh x)

sinh_double_float y (sinhx,coshx) = cos y * sinhx :+ sin y * coshx

cosh`ComplexDouble :: !(Complex Double) -> Complex Double
cosh`ComplexDouble (x:+y) = cosh_double_float y (sincosh x)

cosh`ComplexFloat :: !(Complex Float) -> Complex Float
cosh`ComplexFloat (x:+y) = cosh_double_float y (sincosh x)

cosh_double_float y (sinhx,coshx) = cos y * coshx :+ sin y * sinhx

tanh`ComplexDouble :: !(Complex Double) -> Complex Double
tanh`ComplexDouble (x:+y) = tanh_double_float y (sincosh x)

tanh`ComplexFloat :: !(Complex Float) -> Complex Float
tanh`ComplexFloat (x:+y) = tanh_double_float y (sincosh x)

tanh_double_float y (sinhx,coshx)
	= (cosy*sinhx:+siny*coshx)/(cosy*coshx:+siny*sinhx)
	  where
		siny = sin y
		cosy = cos y

sincosh x
	| abs x <= 3.0E-6
		# sinh = x+x*x*x/6.0
		  exp_x = exp x
		  cosh = 0.5*exp_x + 0.5/exp_x
		= (sinh,cosh)
		# exp_x = exp x
		  exp_xd = exp_x - 1.0
		  sinh = 0.5 * (exp_xd + exp_xd / exp_x)
		  cosh = 0.5*exp_x + 0.5/exp_x
		= (sinh,cosh)

log`ComplexDouble :: !(Complex Double) -> Complex Double
log`ComplexDouble z = log (magnitude`Double z) :+ phase z

log`ComplexFloat :: !(Complex Float) -> Complex Float
log`ComplexFloat z = log (magnitude`Float z) :+ phase z

sqrt`ComplexDouble :: !(Complex Double) -> Complex Double
sqrt`ComplexDouble (0:+0) = 0
sqrt`ComplexDouble z@(x:+y) = u :+ (if y < 0 then -v else v)
  where
  	(u,v) = if x < 0 then (v',u') else (u',v')
	v' = abs y / (u'*2)
	u' = sqrt ((magnitude`Double z + abs x) * 0.5)

sqrt`ComplexFloat :: !(Complex Float) -> Complex Float
sqrt`ComplexFloat (0:+0) = 0
sqrt`ComplexFloat z@(x:+y) = u :+ (if y < 0 then -v else v)
  where
  	(u,v) = if x < 0 then (v',u') else (u',v')
	v' = abs y / (u'*2)
	u' = sqrt ((magnitude`Float z + abs x) * 0.5)

asin`ComplexDouble :: !(Complex Double) -> Complex Double
asin`ComplexDouble z@(x:+y) = y':+(-x')
	where (x':+y') = log`ComplexDouble (((-y):+x) + sqrt`ComplexDouble (1 - z*z))

asin`ComplexFloat :: !(Complex Float) -> Complex Float
asin`ComplexFloat z@(x:+y) = y':+(-x')
	where (x':+y') = log`ComplexFloat (((-y):+x) + sqrt`ComplexFloat (1 - z*z))

acos`ComplexDouble :: !(Complex Double) -> Complex Double
acos`ComplexDouble z@(x:+y) = y'':+(-x'')
  where
	(x'':+y'') = log`ComplexDouble (z + ((-y'):+x'))
	(x':+y') = sqrt`ComplexDouble (1 - z*z)

acos`ComplexFloat :: !(Complex Float) -> Complex Float
acos`ComplexFloat z@(x:+y) = y'':+(-x'')
  where
	(x'':+y'') = log`ComplexFloat (z + ((-y'):+x'))
	(x':+y') = sqrt`ComplexFloat (1 - z*z)

atan`ComplexDouble :: !(Complex Double) -> Complex Double
atan`ComplexDouble z@(x:+y) = y':+(-x')
	where (x':+y') = log`ComplexDouble (((1-y):+x) / sqrt`ComplexDouble (1+z*z))

atan`ComplexFloat :: !(Complex Float) -> Complex Float
atan`ComplexFloat z@(x:+y) = y':+(-x')
	where (x':+y') = log`ComplexFloat (((1-y):+x) / sqrt`ComplexFloat (1+z*z))

asinh`ComplexDouble :: !(Complex Double) -> Complex Double
asinh`ComplexDouble z = log`ComplexDouble (z + sqrt`ComplexDouble (1+z*z))

asinh`ComplexFloat :: !(Complex Float) -> Complex Float
asinh`ComplexFloat z = log`ComplexFloat (z + sqrt`ComplexFloat (1+z*z))

acosh`ComplexDouble :: !(Complex Double) -> Complex Double
acosh`ComplexDouble z = log`ComplexDouble (z + (z+1) * sqrt`ComplexDouble ((z-1)/(z+1)))

acosh`ComplexFloat :: !(Complex Float) -> Complex Float
acosh`ComplexFloat z = log`ComplexFloat (z + (z+1) * sqrt`ComplexFloat ((z-1)/(z+1)))

atanh`ComplexDouble :: !(Complex Double) -> Complex Double
atanh`ComplexDouble z = log`ComplexDouble ((1+z) / sqrt`ComplexDouble (1-z*z))

atanh`ComplexFloat :: !(Complex Float) -> Complex Float
atanh`ComplexFloat z = log`ComplexFloat ((1+z) / sqrt`ComplexFloat (1-z*z))
