implementation module Control.Monad where

liftM2  :: (Monad m) => (a1 -> a2 -> r) -> m a1 -> m a2 -> m r
liftM2 f m1 m2
	= do
		x1 <- m1
		x2 <- m2
		return (f x1 x2)

liftM3  :: (Monad m) => (a1 -> a2 -> a3 -> r) -> m a1 -> m a2 -> m a3 -> m r
liftM3 f m1 m2 m3
	= do
		x1 <- m1
		x2 <- m2
		x3 <- m3
		return (f x1 x2 x3)

liftM4  :: (Monad m) => (a1 -> a2 -> a3 -> a4 -> r) -> m a1 -> m a2 -> m a3 -> m a4 -> m r
liftM4 f m1 m2 m3 m4
	= do
		x1 <- m1
		x2 <- m2
		x3 <- m3
		x4 <- m4
		return (f x1 x2 x3 x4)

zipWithM_ :: (Monad m) => (a -> b -> m c) -> [a] -> [b] -> m ()
zipWithM_ f xs ys = sequence_ (zipWith f xs ys)

--foldM :: (Monad m) => (a -> b -> m a) -> a -> [b] -> m a
foldM f r l :== foldM r l
 where
	foldM r (a:as) = f r a >>= \fra -> foldM fra as
	foldM r [] =  return r

replicateM :: (Monad m) => Int -> m a -> m [a]
replicateM n x = sequence (replicate n x)

--when :: (Monad m) => Bool -> m () -> m ()
when p s :== if p then s else return ()

--unless :: (Monad m) => Bool -> m () -> m ()
unless p s :== if p then return () else s
