implementation module Data.Int where

import PreludeInternal
from StdOverloaded import class toInt (..),class toReal(..),class toString(..)
from StdInt import instance toInt Double,bitxor,bitor,IF_INT_64_OR_32
from StdReal import entier,instance toReal Int
from StdString import instance toString Int
import StdArray

import qualified PreludeInteger
import qualified PreludeFloat

:: Int8 :==Int
:: Int16:==Int
:: Int32:==Int

shiftr -: !Int !Int -> Int;
shiftr a b = `code inline {
	shiftr%
}

ushiftr -: !Int !Int -> Int;
ushiftr a b = `code inline {
	shiftrU
}

negate_int :: !Int -> Int
negate_int a = `code {
		negI
	}

instance Eq Int32 where
	(==) :: !Int32 -> !Int32 -> Bool
	a == b = `code inline {
				eqI
		}

	(/=) :: !Int32 -> !Int32 -> Bool
	a /= b = `code inline {
				eqI
				notB
		}

instance Ord Int32 where
	compare :: !Int32 -> !Int32 -> Ordering
	compare a b
		= if a==b
		  then EQ
		  else  if a<b
				then LT
				else GT

	(<=) infix 4 :: !Int32 -> !Int32 -> Bool
	x <= y = `code inline {
				gtI
				notB
			}

	(<) infix  4 :: !Int32 -> !Int32 -> Bool
	x < y  = `code inline {
				ltI
			}

	(>=) infix 4 :: !Int32 -> !Int32 -> Bool
	x >= y = `code inline {
				ltI
				notB
			}

	(>) infix  4 :: !Int32 -> !Int32 -> Bool
	x > y  = `code inline {
				gtI
			} 

	min :: !Int32 -> !Int32 -> Int32
	min x y  = `code inline {
				.newlocallabel _INT32_min
				push_b 1
				push_b 1
				ltI
				jmp_false _INT32_min
				update_b 0 1
			:_INT32_min
				pop_b 1
			}

	max :: !Int32 -> !Int32 -> Int32
	max x y = `code inline {
				.newlocallabel _INT32_max
				push_b 1
				push_b 1
				gtI
				jmp_false _INT32_max
				update_b 0 1
			:_INT32_max
				pop_b 1
			}

instance Integral Int32 where
	toInteger :: !Int32 -> Integer
	toInteger i = `code inline {
		pushI 0
		create_array_ INT 0 1
	}

	quot :: !Int32 -> !Int32 -> Int32
	quot a b = `code inline {
		divI
	}

	rem :: !Int32 -> !Int32 -> Int32
	rem a b = `code inline {
		remI
	}

	div :: !Int32 -> !Int32 -> Int32
	div a b = `code inline {
		floordivI
	}

	mod :: !Int32 -> !Int32 -> Int32
	mod a b = `code inline {
		modI
	}

	quotRem :: !Int32 -> !Int32 -> (!Int32,!Int32)
	quotRem a b = `code inline {
		push_b 1
		push_b 1
		divI
		push_b 2
		push_b 1
		mulI
		push_b 2
		subI
		update_b 0 3
		update_b 1 2
		pop_b 2
	}

	divMod :: !Int32 -> !Int32 -> (!Int32,!Int32)
	divMod a b = `code inline {
		push_b 1
		push_b 1
		floordivI
		push_b 2
		push_b 1
		mulI
		push_b 2
		subI
		update_b 0 3
		update_b 1 2
		pop_b 2
	}

instance Num Int32 where
	(+) :: !Int32 -> !Int32 -> Int32
	a + b
		= `code inline {
			addI
		}
	(-) :: !Int32 -> !Int32 -> Int32
	a - b
		= `code inline {
			subI
		}
	(*) :: !Int32 -> !Int32 -> Int32
	a * b
		= `code inline {
			mulI
		}
	abs :: !Int32 -> Int32
	abs a
	 	| a<0
	 		= negate a
	 		= a
	signum :: !Int32 -> Int32
	signum a
		= (shiftr a (IF_INT_64_OR_32 63 31)) bitor (ushiftr (negate_int a) (IF_INT_64_OR_32 63 31))
	negate :: !Int32 -> Int32
	negate a
		= `code inline {
			negI
		}
	fromInteger :: !Integer -> Int32
	fromInteger {integer_s,integer_a}
		| size integer_a==`0
			= integer_s;
			= (integer_a~[0] bitxor integer_s) - integer_s;

instance Enum Int32 where
	toEnum :: !Int -> Int32
	toEnum i = i

	fromEnum :: !Int32 -> Int
	fromEnum i = i

instance Real Int32 where
	toRational :: !Int32 -> Rational
	toRational i = {integer_s=i,integer_a={}} :% 1

instance Show Int32 where
	show :: !Int32 -> [Char]
	show i = string_to_char_list (toString i)
	showsPrec _ i = string_to_char_list_append (toString i)
