module Directory (
  createDirectory,removeDirectory,removeFile,renameDirectory,renameFile,
  getDirectoryContents,getCurrentDirectory,setCurrentDirectory,
  doesFileExist,doesDirectoryExist,getModificationTime,
  getPermissions,setPermissions,Permissions(..)
  ) where

import PreludeInternal (char_list_to_string,char_list_to_zstring)
import PreludeInternalIOError
import PreludeInternal (string_to_char_list,char_list_to_string)

from StdInt import bitand,bitor,bitnot,<<,class toInt(..),instance toInt {#Char}
from StdString import class +++(+++),instance +++ {#Char},class %(%),instance % {#Char}

import Time (ClockTime(..))

--import `code from library "advapi_library"

data Permissions
  = Permissions { readable, writable, executable, searchable :: Bool } deriving (Eq, Ord, Read, Show)

createDirectory :: FilePath -> IO ()
createDirectory dir_path = IO (createDirectoryWorld dir_path)

createDirectoryWorld :: !FilePath -> !*World -> (!(),!*World)
createDirectoryWorld dir_path w
	# r = CreateDirectory (char_list_to_zstring dir_path) 0
	| r/=0
		= ((),w)
	# e = GetLastError
	| e==ERROR_ALREADY_EXISTS
		= ioErrorUnitWorld AlreadyExistsError w
	| e==ERROR_PATH_NOT_FOUND
		= ioErrorUnitWorld DoesNotExistError w
		= ioErrorUnitWorld PermissionError w

removeDirectory :: FilePath -> IO ()
removeDirectory dir_path = IO (removeDirectoryWorld dir_path)

removeDirectoryWorld :: !FilePath -> !*World -> (!(),!*World)
removeDirectoryWorld dir_path w
	# r = RemoveDirectory (char_list_to_zstring dir_path)
	| r/=0
		= ((),w)
	# e = GetLastError
	| e==ERROR_PATH_NOT_FOUND
		= ioErrorUnitWorld DoesNotExistError w
		= ioErrorUnitWorld PermissionError w

removeFile :: FilePath -> IO ()
removeFile dir_path = IO (removeFileWorld dir_path)

removeFileWorld :: !FilePath -> !*World -> (!(),!*World)
removeFileWorld dir_path w
	# r = DeleteFile (char_list_to_zstring dir_path)
	| r/=0
		= ((),w)
	# e = GetLastError
	| e==ERROR_PATH_NOT_FOUND || e==ERROR_FILE_NOT_FOUND
		= ioErrorUnitWorld DoesNotExistError w
		= ioErrorUnitWorld PermissionError w

renameDirectory :: FilePath -> FilePath -> IO ()
renameDirectory old_dir_path new_dir_path = IO (renameDirectoryWorld old_dir_path new_dir_path)

renameDirectoryWorld :: !FilePath -> !FilePath -> !*World -> (!(),!*World)
renameDirectoryWorld old_dir_path new_dir_path w
	# old_dir_path_zstring = char_list_to_zstring old_dir_path
	# new_dir_path_zstring = char_list_to_zstring new_dir_path
	# file_attributes = GetFileAttributes old_dir_path_zstring
	| file_attributes==INVALID_FILE_ATTRIBUTES
		= ioErrorUnitWorld DoesNotExistError w
	| file_attributes bitand FILE_ATTRIBUTE_DIRECTORY==0
		= ioErrorUnitWorld DoesNotExistError w
	# r = MoveFileEx old_dir_path_zstring new_dir_path_zstring MOVEFILE_COPY_ALLOWED		
	| r/=0
		= ((),w)
	# e=GetLastError
	| e==ERROR_ALREADY_EXISTS
		# r = RemoveDirectory new_dir_path_zstring
		| r/=0
			# r = MoveFileEx old_dir_path_zstring new_dir_path_zstring MOVEFILE_COPY_ALLOWED		
			| r/=0
				= ((),w)
				= ioErrorUnitWorld PermissionError w
			= ioErrorUnitWorld PermissionError w
		= ioErrorUnitWorld PermissionError w

renameFile :: FilePath -> FilePath -> IO ()
renameFile old_file_path new_file_path = IO (renameFileWorld old_file_path new_file_path)

renameFileWorld :: !FilePath -> !FilePath -> !*World -> (!(),!*World)
renameFileWorld old_file_path new_file_path w
	# old_file_path_zstring = char_list_to_zstring old_file_path
	# new_file_path_zstring = char_list_to_zstring new_file_path
	# file_attributes = GetFileAttributes old_file_path_zstring
	| file_attributes==INVALID_FILE_ATTRIBUTES
		= ioErrorUnitWorld DoesNotExistError w
	| file_attributes bitand FILE_ATTRIBUTE_DIRECTORY/=0
		= ioErrorUnitWorld DoesNotExistError w
	# r = MoveFileEx old_file_path_zstring new_file_path_zstring MOVEFILE_REPLACE_EXISTING_COPY_ALLOWED
	| r/=0
		= ((),w)
		= ioErrorUnitWorld PermissionError w

getDirectoryContents :: FilePath -> IO [FilePath]
getDirectoryContents dir_path = IO (getDirectoryContentsWorld dir_path)

getDirectoryContentsWorld :: FilePath -> !*World -> (![FilePath],!*World)
getDirectoryContentsWorld dir_path w
	# dir_path_string = char_list_to_string dir_path
	| size dir_path_string>0 && dir_path_string~[size dir_path_string-1]=='\\'
		# (contents,w) = find_files (dir_path_string+++`"*\0") w
		= (strings_to_char_lists contents,w)
		# (contents,w) = find_files (dir_path_string+++`"\\*\0") w
		= (strings_to_char_lists contents,w)
where
	find_files :: !{#Char} -> !*World -> (![{#Char}],!*World)
	find_files file_name w
		# (handle,find_file_data) = FindFirstFile file_name
		| handle==0 || handle== -1
			# e = GetLastError
			| e==ERROR_PATH_NOT_FOUND || e==ERROR_DIRECTORY
				# (_,w) = ioErrorUnitWorld DoesNotExistError w
				= ([],w)
				# (_,w) = ioErrorUnitWorld PermissionError w
				= ([],w)
			# file_name=get_WIN32_FIND_DATA_cFileName find_file_data
			# (file_names,w) = find_next_files_and_close handle w
			= (file_name:file_names,w)
	
	find_next_files_and_close :: !Int -> !*World -> (![{#Char}],!*World)
	find_next_files_and_close handle w
		# (ok,find_file_data) = FindNextFile handle
		| ok/=1
			# r=FindClose handle
			| r/=1
				= ([],w) //error
				= ([],w)
			# file_name=get_WIN32_FIND_DATA_cFileName find_file_data
			# (file_names,w) = find_next_files_and_close handle w
			= (file_name:file_names,w)

	strings_to_char_lists sl
		= [string_to_char_list s | s<-sl]

getCurrentDirectory :: IO FilePath
getCurrentDirectory = IO getCurrentDirectoryWorld

getCurrentDirectoryWorld :: !*World -> (!FilePath,!*World)
getCurrentDirectoryWorld w
	# s = createArray 256 '\0'
	# r = GetCurrentDirectory 256 s
	| r>0
		| r<256
			= (string_to_char_list (s % (0,r-1)),w)
			# n_chars = r
			  s = createArray n_chars '\0'
			  r = GetCurrentDirectory n_chars s
			| r>0
				= (string_to_char_list (s % (0,r-1)),w)
				= getCurrentDirectoryWorldError w
		= getCurrentDirectoryWorldError w
	where
		getCurrentDirectoryWorldError w
			# e = GetLastError
			| e==ERROR_PATH_NOT_FOUND
				# (_,w) = ioErrorUnitWorld DoesNotExistError w
				= ("",w)
				# (_,w) = ioErrorUnitWorld PermissionError w
				= ("",w)

setCurrentDirectory :: FilePath -> IO ()
setCurrentDirectory dir_path = IO (setCurrentDirectoryWorld dir_path)

setCurrentDirectoryWorld :: !FilePath -> !*World -> (!!(),!*World)
setCurrentDirectoryWorld dir_path w
	# r = SetCurrentDirectory (char_list_to_zstring dir_path)
	| r/=0
		= ((),w)
	# e = GetLastError
	| e==ERROR_PATH_NOT_FOUND || e==ERROR_FILE_NOT_FOUND
		= ioErrorUnitWorld DoesNotExistError w
		= ioErrorUnitWorld PermissionError w

doesDirectoryExist :: FilePath -> IO Bool
doesDirectoryExist file_path = IO (doesDirectoryExistWorld file_path)

doesDirectoryExistWorld :: !FilePath -> !*World -> (!Bool,!*World)
doesDirectoryExistWorld file_path w
	# file_attributes = GetFileAttributes (char_list_to_zstring file_path)
	| file_attributes==INVALID_FILE_ATTRIBUTES
		= (False,w)
	| file_attributes bitand FILE_ATTRIBUTE_DIRECTORY/=0
		= (True,w)
		= (False,w)

doesFileExist :: FilePath -> IO Bool
doesFileExist file_path = IO (doesFileExistWorld file_path)

doesFileExistWorld :: !FilePath -> !*World -> (!Bool,!*World)
doesFileExistWorld file_path w
	# file_attributes = GetFileAttributes (char_list_to_zstring file_path)
	| file_attributes==INVALID_FILE_ATTRIBUTES
		= (False,w)
	| file_attributes bitand FILE_ATTRIBUTE_DIRECTORY/=0
		= (False,w)
		= (True,w)

getModificationTime :: FilePath -> IO ClockTime
getModificationTime file_path = IO (getModificationTimeWorld file_path)

getModificationTimeWorld :: FilePath -> !*World -> (!ClockTime,!*World)
getModificationTimeWorld file_path w
	// use GetFileAttributesEx instead ?
	# file_path_zstring = char_list_to_zstring file_path
	| GetFileAttributes file_path_zstring==INVALID_FILE_ATTRIBUTES
		# e = GetLastError
		| e==ERROR_PATH_NOT_FOUND || e==ERROR_FILE_NOT_FOUND || e==ERROR_DIRECTORY			
			= error_time DoesNotExistError w
			= error_time PermissionError w
	# (handle,find_file_data) = FindFirstFile file_path_zstring
	| handle==0 || handle== -1
		# e = GetLastError
		| e==ERROR_PATH_NOT_FOUND || e==ERROR_FILE_NOT_FOUND || e==ERROR_DIRECTORY			
			= error_time DoesNotExistError w
			= error_time PermissionError w
		# (file_time_l,file_time_h) = get_WIN32_FIND_DATA_ftLastWriteTime find_file_data
		  (rh,qh) = divdu 0 file_time_h 10000
		  (rl,ql) = divdu rh file_time_l 10000
		  clock_time = {ct_milli_sec_l=ql,ct_milli_sec_h=qh,ct_pico_sec=rl*100000}
		  w = close_find handle w
		= (clock_time,w)
  where	
	close_find :: !Int -> !*World -> *World
	close_find handle w
		# r=FindClose handle
		| r/=1
			= w //error
			= w

	error_time :: !IOError -> !*World -> (!ClockTime,!*World)
	error_time e w
		# (_,w) = ioErrorUnitWorld e w
		= ({ct_milli_sec_l=0,ct_milli_sec_h=0,ct_pico_sec=0},w)

getPermissions :: FilePath -> IO Permissions
getPermissions file_path = IO (getPermissionsWorld file_path)

getPermissionsWorld :: !FilePath -> !*World -> (!Permissions,!*World)
getPermissionsWorld file_path w
	# file_path_zstring = char_list_to_zstring file_path
	  requested_information = OWNER_SECURITY_INFORMATION bitor GROUP_SECURITY_INFORMATION bitor DACL_SECURITY_INFORMATION
	  (r,sd_length) = GetFileSecurity_sd_length file_path_zstring requested_information 0 0
	#! e=if (r==0) GetLastError 0
	| r==0 && e/=ERROR_INSUFFICIENT_BUFFER
		| e==ERROR_FILE_NOT_FOUND || e==ERROR_PATH_NOT_FOUND
			# (_,w) = ioErrorUnitWorld DoesNotExistError w
			= (Permissions False False False False,w)
			= error w		
	# sd = LocalAlloc LMEM_FIXED sd_length
	| sd==0
		= free_sd_and_error sd w
	# (r,sd_length) = GetFileSecurity file_path_zstring requested_information sd sd_length
	| r==0
		= free_sd_and_error sd w
	# r = ImpersonateSelf SecurityImpersonation
	| r==0
		= free_sd_and_error sd w
	# (r,thread_token) = OpenThreadToken GetCurrentThread TOKEN_QUERY 0
	| r==0
		= revert_to_self_free_sd_and_error sd w
	# (check_access_ok,maximum_access_mask) = check_access sd thread_token MAXIMUM_ALLOWED
	| not check_access_ok
		= close_handle_revert_to_self_free_sd_and_error thread_token sd w
	# r = CloseHandle thread_token
	| r==0
		= revert_to_self_free_sd_and_error sd w
	# r = RevertToSelf
	| r==0
		= free_sd_and_error sd w
	# r = LocalFree sd
	| r/=0
		= error w
	#! read_access = (maximum_access_mask bitand FILE_GENERIC_READ)==FILE_GENERIC_READ
	#! write_access = (maximum_access_mask bitand FILE_GENERIC_WRITE)==FILE_GENERIC_WRITE
	#! execute_access = (maximum_access_mask bitand FILE_GENERIC_EXECUTE)==FILE_GENERIC_EXECUTE
	# file_attributes = GetFileAttributes file_path_zstring
	#! is_directory = file_attributes/=INVALID_FILE_ATTRIBUTES && file_attributes bitand FILE_ATTRIBUTE_DIRECTORY/=0
	= (Permissions {readable=read_access,writable=write_access,executable=execute_access,searchable=is_directory},w)
where
	error w
		# (_,w) = ioErrorUnitWorld PermissionError w
		= (Permissions False False False False,w)

	free_sd_and_error sd w
		= seq (LocalFree sd) (error w)

	revert_to_self_free_sd_and_error sd w
		= seq RevertToSelf (free_sd_and_error sd w)
		
	close_handle_revert_to_self_free_sd_and_error thread_token sd w
		= seq (CloseHandle thread_token) (revert_to_self_free_sd_and_error sd w)

setPermissions :: FilePath -> Permissions -> IO ()
setPermissions file_path permissions = IO (setPermissionsWorld file_path permissions)

setPermissionsWorld :: !FilePath -> !Permissions -> !*World -> (!(),!*World)
setPermissionsWorld file_path (Permissions {readable,writable,executable,searchable}) w
	# file_path_zstring = char_list_to_zstring file_path
	  requested_information = OWNER_SECURITY_INFORMATION bitor GROUP_SECURITY_INFORMATION bitor DACL_SECURITY_INFORMATION
	# (r,sd_length) = GetFileSecurity_sd_length file_path_zstring requested_information 0 0
	#! e=if (r==0) GetLastError 0
	| r==0 && e/=ERROR_INSUFFICIENT_BUFFER
		| e==ERROR_FILE_NOT_FOUND || e==ERROR_PATH_NOT_FOUND
			= ioErrorUnitWorld DoesNotExistError w
			= error w
	# sd = LocalAlloc LMEM_FIXED sd_length
	| sd==0
		= free_sd_and_error sd w
	# (r,sd_length) = GetFileSecurity file_path_zstring requested_information sd sd_length
	| r==0
		= free_sd_and_error sd w
	# file_attributes = GetFileAttributes file_path_zstring
	#! is_directory = file_attributes/=INVALID_FILE_ATTRIBUTES && file_attributes bitand FILE_ATTRIBUTE_DIRECTORY/=0
	| is_directory/=searchable
		= seq (LocalFree sd) (ioErrorUnitWorld DoesNotExistError w)
	# r = ImpersonateSelf SecurityImpersonation
	| r==0
		= free_sd_and_error sd w
	# (r,thread_token) = OpenThreadToken GetCurrentThread TOKEN_QUERY 0
	| r==0
		= revert_to_self_free_sd_and_error sd w
	# (check_access_ok,maximum_access_mask) = check_access sd thread_token MAXIMUM_ALLOWED
	| not check_access_ok
		= close_handle_revert_to_self_free_sd_and_error thread_token sd w
	#! read_access = (maximum_access_mask bitand FILE_GENERIC_READ)==FILE_GENERIC_READ
	#! write_access = (maximum_access_mask bitand FILE_GENERIC_WRITE)==FILE_GENERIC_WRITE
	#! execute_access = (maximum_access_mask bitand FILE_GENERIC_EXECUTE)==FILE_GENERIC_EXECUTE
	| readable==read_access && writable==write_access && executable==execute_access
		= close_handle_revert_to_self_and_free_sd thread_token sd w
	# (new_allow_permissions,new_deny_permissions)
		= determine_new_permissions readable writable executable maximum_access_mask
	| new_allow_permissions==0 && new_deny_permissions==0
		= close_handle_revert_to_self_free_sd_and_error thread_token sd w
	# (r,dacl_present,dacl,dacl_defaulted) = GetSecurityDescriptorDacl sd
	| r==0
		= close_handle_revert_to_self_free_sd_and_error thread_token sd w
	# (e,user_p) = get_token_user thread_token
	| e/=0 || user_p==0
		| user_p/=0
			= free_user_p_close_handle_revert_to_self_free_sd_and_error user_p thread_token sd w
			= close_handle_revert_to_self_free_sd_and_error thread_token sd w
	#! user_sid = load_int user_p
	# (r,new_dacl) = add_entries_in_acl new_allow_permissions new_deny_permissions user_sid dacl
	| r/=0
		= free_user_p_close_handle_revert_to_self_free_sd_and_error user_p thread_token sd w
	# r = SetNamedSecurityInfo file_path_zstring SE_FILE_OBJECT DACL_SECURITY_INFORMATION 0 0 new_dacl 0
	| r/=0
		= free_new_dacl_free_user_p_close_handle_revert_to_self_free_sd_and_error new_dacl user_p thread_token sd w
	| LocalFree new_dacl/=0
		= free_user_p_close_handle_revert_to_self_free_sd_and_error user_p thread_token sd w
	| LocalFree sd/=0
		= close_handle_revert_to_self_free_sd_and_error thread_token sd w
	= close_handle_revert_to_self_and_free_sd thread_token sd w
  where
	error w
		= ioErrorUnitWorld PermissionError w

	free_sd_and_error sd w
		= seq (LocalFree sd) (error w)

	revert_to_self_free_sd_and_error sd w
		= seq RevertToSelf (free_sd_and_error sd w)

	close_handle_revert_to_self_free_sd_and_error thread_token sd w
		= seq (CloseHandle thread_token) (revert_to_self_free_sd_and_error sd w)

	free_user_p_close_handle_revert_to_self_free_sd_and_error user_p thread_token sd w
		= seq (LocalFree user_p) (close_handle_revert_to_self_free_sd_and_error thread_token sd w)

	free_new_dacl_free_user_p_close_handle_revert_to_self_free_sd_and_error new_dacl user_p thread_token sd w
		= seq (LocalFree new_dacl) (free_user_p_close_handle_revert_to_self_free_sd_and_error user_p thread_token sd w)

	close_handle_revert_to_self_and_free_sd thread_token sd w
		# r = CloseHandle thread_token
		| r==0
			= revert_to_self_free_sd_and_error sd w
		# r = RevertToSelf
		| r==0
			= free_sd_and_error sd w
		| LocalFree sd/=0
			= error w
		= ((),w)

	determine_new_permissions :: !Bool -> !Bool -> !Bool -> !Int -> (!Int,!Int)
	determine_new_permissions readable writable executable current_permissions
		# allow_permissions = if readable FILE_GENERIC_READ 0
		  allow_permissions = if writable (allow_permissions bitor FILE_GENERIC_WRITE) allow_permissions
		  allow_permissions = if executable (allow_permissions bitor FILE_GENERIC_EXECUTE) allow_permissions

		# deny_permissions = if readable 0 FILE_GENERIC_READ
		  deny_permissions = if writable deny_permissions (deny_permissions bitor FILE_GENERIC_WRITE)
		  deny_permissions = if executable deny_permissions (deny_permissions bitor FILE_GENERIC_EXECUTE)
		
		# deny_permissions = deny_permissions bitand (bitnot allow_permissions)
		  new_allow_permissions = allow_permissions bitand (bitnot current_permissions)
		  new_deny_permissions = deny_permissions bitand current_permissions bitand (bitnot READ_CONTROL)

		= (new_allow_permissions,new_deny_permissions)

	add_entries_in_acl :: !Int -> !Int -> !Int -> !Int -> (!Int,!Int)
	add_entries_in_acl new_allow_permissions new_deny_permissions user_sid dacl
		| new_deny_permissions==0
			# explicit_entries = {
				new_allow_permissions,GRANT_ACCESS,NO_INHERITANCE,
				0,0,0,0,user_sid
			   }
			= SetEntriesInAcl 1 explicit_entries dacl
		| new_allow_permissions==0
			# explicit_entries = {
				new_deny_permissions,DENY_ACCESS,NO_INHERITANCE,
				0,0,0,0,user_sid
			   }
			= SetEntriesInAcl 1 explicit_entries dacl		
			# explicit_entries = {
				new_allow_permissions,GRANT_ACCESS,NO_INHERITANCE,
				0,0,0,0,user_sid,
				new_deny_permissions,DENY_ACCESS,NO_INHERITANCE,
				0,0,0,0,user_sid
			   }
			= SetEntriesInAcl 2 explicit_entries dacl

check_access :: !Int -> !Int -> !Int -> (!Bool,!Int)
check_access sd thread_token desired_access
	# access_mask_a = {desired_access}
	# generic_mapping = {FILE_GENERIC_READ,FILE_GENERIC_WRITE,FILE_GENERIC_EXECUTE,FILE_ALL_ACCESS}
	| MapGenericMask access_mask_a generic_mapping 0/=0
		= (False,0)
	# privilege_set_size = 32
	# privilege_set = createArray privilege_set_size '\0'
	# privilege_set_size_a = {privilege_set_size}
	# (r,grantedAccess,accessStatus) = AccessCheck sd thread_token access_mask_a~[0] generic_mapping privilege_set privilege_set_size_a
	| r/=0			
		= (True,grantedAccess)
	# e = if (r==0) GetLastError 0
	| e==ERROR_INSUFFICIENT_BUFFER
		# privilege_set_size = privilege_set_size_a~[0]
		# privilege_set = createArray privilege_set_size '\0'
		# (r,grantedAccess,accessStatus) = AccessCheck sd thread_token access_mask_a~[0] generic_mapping privilege_set privilege_set_size_a
		| r/=0
			= (True,grantedAccess)
			= (False,0)
		= (False,0)

get_token_user :: !Int -> (!Int,!Int)
get_token_user thread_token
	# (r,length) = GetTokenInformation thread_token TokenUser 0 0
	#! e = if (r==0) GetLastError 0 
	| r==0 && e/=ERROR_INSUFFICIENT_BUFFER
		= (e,0)
	# user_p = LocalAlloc LMEM_FIXED length
	| user_p==0
		= (0,0)
	# (r,length) = GetTokenInformation thread_token TokenUser user_p length
	| r/=0
		= (0,user_p)
		= (GetLastError,user_p)

NO_INHERITANCE:==`0

ioErrorUnitWorld :: !IOError -> !*World -> (!(),!*World)
ioErrorUnitWorld e w
	# (IO io) = ioError e
	= io w

ERROR_FILE_NOT_FOUND:==`2
ERROR_PATH_NOT_FOUND:==`3
ERROR_ACCESS_DENIED:==`5
ERROR_INSUFFICIENT_BUFFER:==`122
ERROR_INVALID_NAME:==`123
ERROR_ALREADY_EXISTS:==`183
ERROR_DIRECTORY:==`267

CreateDirectory :: !{#Char} -> !Int -> Int
CreateDirectory pathName securityAttributes = `code {
	ccall CreateDirectoryA@8 "PsI:I"
}

RemoveDirectory :: !{#Char} -> Int
RemoveDirectory pathName = `code {
	ccall RemoveDirectoryA@4 "Ps:I"
}

DeleteFile :: !{#Char} -> Int
DeleteFile fileName = `code {
	ccall DeleteFileA@4 "Ps:I"
}

MOVEFILE_REPLACE_EXISTING:==`1
MOVEFILE_COPY_ALLOWED:==`2

MOVEFILE_REPLACE_EXISTING_COPY_ALLOWED:==`3

MoveFileEx :: !{#Char} -> !{#Char} -> !Int -> Int
MoveFileEx existingFileName newFileName flags = `code {
	ccall MoveFileExA@12 "PssI:I"
}

INVALID_FILE_ATTRIBUTES:==`-1

FILE_ATTRIBUTE_DIRECTORY:==`0x10

GetFileAttributes :: !{#Char} -> Int
GetFileAttributes fileName = `code {
	ccall GetFileAttributesA@4 "Ps:I"
}

:: WIN32_FIND_DATA:=={#Char}

WIN32_FIND_DATA_SIZE:==318

WIN32_FIND_DATA_ftLastWriteTime_offset:==20
WIN32_FIND_DATA_cFileName_offset:==44

MAX_PATH:==260

FindFirstFile :: !{#Char} -> (!Int,!WIN32_FIND_DATA)
FindFirstFile file_name
	# find_file_data=createArray WIN32_FIND_DATA_SIZE '\0'
	#! r=find_first_file file_name find_file_data
	= (r,find_file_data)

find_first_file :: !{#Char} -> !WIN32_FIND_DATA -> Int
find_first_file lpFileName lpFindFileData = `code {
	ccall FindFirstFileA@8 "Pss:I"
}

FindNextFile :: !Int -> (!Int,!WIN32_FIND_DATA)
FindNextFile hFindFile
	# find_file_data=createArray WIN32_FIND_DATA_SIZE '\0'
	#! r=find_next_file hFindFile find_file_data
	= (r,find_file_data)

find_next_file :: !Int -> !WIN32_FIND_DATA -> Int
find_next_file hFindFile lpFindFileData = `code {
	ccall FindNextFileA@8 "PIs:I"
}

FindClose :: !Int -> Int
FindClose hFindFile = `code {
	ccall FindClose@4 "PI:I"
}

(read_DWORD) a i = (toInt a~[i]+(toInt a~[i+1]<<8)+(toInt a~[i+2]<<16)+(toInt a~[i+3]<<24));

get_WIN32_FIND_DATA_cFileName :: !WIN32_FIND_DATA -> {#Char}
get_WIN32_FIND_DATA_cFileName find_file_data
	# begin_index = WIN32_FIND_DATA_cFileName_offset
	# end_index = find_null_char_index begin_index (begin_index+MAX_PATH) find_file_data
	= find_file_data % (begin_index,end_index-1)
  where
	find_null_char_index i max_i s
	    | s~[i]=='\0' || i==max_i
		= i
		= find_null_char_index (i+1) max_i s

get_WIN32_FIND_DATA_ftLastWriteTime :: WIN32_FIND_DATA -> (!Int,!Int)
get_WIN32_FIND_DATA_ftLastWriteTime find_file_data
	= (	find_file_data read_DWORD WIN32_FIND_DATA_ftLastWriteTime_offset,
		find_file_data read_DWORD (WIN32_FIND_DATA_ftLastWriteTime_offset+4)
	  )

GetCurrentDirectory :: !Int ->  !{#Char} -> Int
GetCurrentDirectory bufferLength buffer = `code {
	ccall GetCurrentDirectoryA@8 "PIs:I"
}

SetCurrentDirectory :: !{#Char} -> Int
SetCurrentDirectory pathName = `code {
	ccall SetCurrentDirectoryA@4 "Ps:I"
}

FS_PERSISTENT_ACLS:==`8  

GetVolumeInformation_file_system_flags :: !{#Char} -> !Int -> !Int -> !Int -> !Int -> !{#Int} -> !Int -> !Int -> Int
GetVolumeInformation_file_system_flags rootPathName volumeNameBuffer volumeNameSize volumeSerialNumber maximumComponentLength fileSystemFlags fileSystemNameBuffer fileSystemNameSize
  = `code {
	ccall GetVolumeInformationA@32 "PsIIIIAII:I"
}

GetVolumeInformation_p_file_system_flags :: !Int -> !Int -> !Int -> !Int -> !Int -> !{#Int} -> !Int -> !Int -> Int
GetVolumeInformation_p_file_system_flags rootPathName volumeNameBuffer volumeNameSize volumeSerialNumber maximumComponentLength fileSystemFlags fileSystemNameBuffer fileSystemNameSize
  = `code {
	ccall GetVolumeInformationA@32 "PIIIIIAII:I"
}

OWNER_SECURITY_INFORMATION:==`1
GROUP_SECURITY_INFORMATION:==`2
DACL_SECURITY_INFORMATION:==`4
SACL_SECURITY_INFORMATION:==`8

GetFileSecurity_sd_length :: !{#Char} -> !Int -> !Int -> !Int -> (!Int,!Int)
GetFileSecurity_sd_length fileName requestedInformation securityDescriptor length = `code {
	ccall GetFileSecurityA@20 "PsIII:II"
}

GetFileSecurity :: !{#Char} -> !Int -> !Int -> !Int -> (!Int,!Int)
GetFileSecurity fileName requestedInformation securityDescriptor length = `code {
	ccall GetFileSecurityA@20 "PsIII:II"
}

SecurityImpersonation:==`2

ImpersonateSelf :: !Int -> Int
ImpersonateSelf impersonationLevel = `code {
	ccall ImpersonateSelf@4 "PI:I"
}

GetCurrentThread :: Int
GetCurrentThread = `code {
	ccall GetCurrentThread@0 "P:I"
}

TOKEN_QUERY:==`8

OpenThreadToken :: !Int -> !Int -> !Int -> (!Int,!Int)
OpenThreadToken threadHandle desiredAccess openAsSelf = `code {
	ccall OpenThreadToken@16 "PIII:II"
}

TokenUser:==`1
TokenGroups:==`2
TokenGroupsAndPrivileges:==13

GetTokenInformation :: !Int -> !Int -> !Int -> !Int -> (!Int,!Int)
GetTokenInformation tokenHandle tokenInformationClass tokenInformation tokenInformationLength = `code {
	ccall GetTokenInformation@20 "PIIII:II"
}

GetSecurityDescriptorDacl :: !Int -> (!Int,!Int,!Int,!Int)
GetSecurityDescriptorDacl securityDescriptor = `code {
	ccall GetSecurityDescriptorDacl@20 "PI:IIII"
}

MAXIMUM_ALLOWED:==`0x02000000

READ_CONTROL:==`0x00020000
FILE_GENERIC_READ:==`0x00120089
FILE_GENERIC_WRITE:==`0x00120116
FILE_GENERIC_EXECUTE:==`0x001200A0
FILE_ALL_ACCESS:==`0x001F01FF

MapGenericMask :: !{#Int} -> !{#Int} -> !Int -> Int
MapGenericMask accessMask genericMapping result = `code {
	ccall MapGenericMask@8 "PAA:V:I"
};

AccessCheck :: !Int -> !Int -> !Int -> !{#Int} -> !{#Char} -> !{#Int} -> (!Int,!Int,!Int)
AccessCheck securityDescriptor clientToken desiredAccess genericMapping privilegeSet privilegeSetLength = `code {
	ccall AccessCheck@32 "PIIIAsA:III"
}

NO_MULTIPLE_TRUSTEE:==`0
TRUSTEE_IS_SID:==`0
TRUSTEE_IS_UNKNOWN:==`0

TRUSTEE_Int_size:==5

NOT_USED_ACCESS:==`0
GRANT_ACCESS:==`1
SET_ACCESS:==`2
DENY_ACCESS:==`3
REVOKE_ACCESS:==`4
SET_AUDIT_SUCCESS:==`5
SET_AUDIT_FAILURE:==`6

SetEntriesInAcl :: !Int -> !{#Int} -> !Int -> (!Int,!Int)
SetEntriesInAcl countOfExplicitEntries listOfExplicitEntries oldAcl = `code {
	ccall SetEntriesInAclA@16 "PIAI:II"
};

SE_FILE_OBJECT:==`1

SetNamedSecurityInfo :: !{#Char} -> !Int -> !Int -> !Int -> !Int -> !Int -> !Int -> Int
SetNamedSecurityInfo objectName objectType securityInfo sidOwner sidGroup dacl sacl = `code {
	ccall SetNamedSecurityInfoA@28 "PsIIIIII:I"
};

CloseHandle :: !Int -> Int
CloseHandle handle = `code {
	ccall CloseHandle@4 "PI:I"
}

RevertToSelf :: Int;
RevertToSelf = `code {
	ccall RevertToSelf@0 "P:I"
}

LMEM_FIXED:==`0

LocalAlloc :: !Int -> !Int -> Int
LocalAlloc flags bytes = `code {
	ccall LocalAlloc@8 "PII:I"
}

LocalFree :: !Int -> Int
LocalFree handle = `code {
	ccall LocalFree@4 "PI:I"
}

GetLastError :: Int
GetLastError = `code {
	ccall GetLastError@0 "P:I"
}

load_int :: !Int -> Int
load_int p = `code {
	load_i 0
}

divdu :: !Int -> !Int -> !Int -> (!Int,!Int)
divdu a b c = `code inline {
	divLU
}
