implementation module Ix where

from StdMisc import abort
from StdInt import qualified instance + Int,instance one Int

instance Ix Char where
	range :: !(Char,Char) -> [Char]
	range (m,n) = [m..n]
	index :: !(!Char,!Char) -> !Char -> Int
	index (c,c') ci
		| c <= ci && ci <= c'
			= fromEnum ci - fromEnum c
			= index_out_of_range
	inRange :: !(!Char,Char) -> !Char -> Bool
	inRange (c,c') i = c <= i && i <= c'
	rangeSize :: !(!Char,!Char) -> Int
	rangeSize (m,n)
		| n>=m
			= fromEnum n-fromEnum m+1
			= 0

instance Ix Int where
	range :: !(Int,Int) -> [Int]
	range (m,n) = [m..n]
	index :: !(!Int,!Int) -> !Int -> Int
	index b@(m,n) i
		| m <= i && i <= n
			= i - m
			= index_out_of_range
	inRange :: !(!Int,!Int) -> !Int -> Bool
	inRange (m,n) i = m <= i && i <= n
	rangeSize :: !(!Int,!Int) -> Int
	rangeSize (m,n)
		| n>=m
			= n-m+1
			= 0

instance Ix Integer where
	range :: !(Integer,Integer) -> [Integer]
	range (m,n) = [m..n]
	index :: !(!Integer,!Integer) -> !Integer -> Int
	index b@(m,n) i
		| inRange b i
			= fromInteger (i - m)
			= index_out_of_range
	inRange :: !(!Integer,Integer) -> !Integer -> Bool
	inRange (m,n) i = m <= i && i <= n
	rangeSize :: !(!Integer,!Integer) -> Int
	rangeSize (m,n)
		| n>=m
			= fromInteger (n-m+1)
			= 0

instance Ix Bool where
	range :: !(Bool,Bool) -> [Bool]
	range (m,n) = [m..n]
	index :: !(!Bool,!Bool) -> !Bool -> Int
	index (c,c') ci
		| c <= ci && ci <= c'
			= fromEnum ci - fromEnum c
			= index_out_of_range
	inRange :: !(!Bool,Bool) -> !Bool -> Bool
	inRange (c,c') i = c <= i && i <= c'
	rangeSize :: !(!Bool,!Bool) -> Int
	rangeSize (m,n)
		| n>=m
			= fromEnum n-fromEnum m+1
			= 0

instance Ix () where
	range :: !(!(),!()) -> [()]
    range ((),())  = [()]
	inRange :: !(!(),!()) -> !() -> Bool
    inRange ((),()) () = True
	index :: !(!(),!()) -> !() -> Int
    index ((),()) () = 0
	rangeSize :: !(!(),!()) -> Int
	rangeSize ((),()) = 1

instance Ix Ordering where
	range :: !(Ordering,Ordering) -> [Ordering]
	range (m,n) = [m..n]
	index :: !(!Ordering,!Ordering) -> !Ordering -> Int
	index (c,c') ci
		| c <= ci && ci <= c'
			= fromEnum ci - fromEnum c
			= index_out_of_range
	inRange :: !(!Ordering,Ordering) -> !Ordering -> Bool
	inRange (c,c') i = c <= i && i <= c'
	rangeSize :: !(!Ordering,!Ordering) -> Int
	rangeSize (m,n)
		| n>=m
			= fromEnum n-fromEnum m+1
			= 0

instance (Ix e0,Ix e1) => Ix (e0,e1) where	
	range :: (Ix e0,Ix e1) => !(!(e0,e1),!(e0,e1)) -> [(e0,e1)]
    range ((l1,l2),(u1,u2))
    	= [(i1,i2) | i1<-range (l1,u1), i2<-range (l2,u2)]
	inRange :: (Ix e0,Ix e1) => !(!(e0,e1),!(e0,e1)) -> !(e0,e1) -> Bool
    inRange ((l1,l2),(u1,u2)) (i1,i2)
    	= inRange (l1,u1) i1 && inRange (l2,u2) i2
	index :: (Ix e0,Ix e1) => !(!(e0,e1),!(e0,e1)) -> !(e0,e1) -> Int
	index ((l1,l2),(u1,u2)) (i1,i2)
		| inRange (l1,u1) i1 && inRange (l2,u2) i2
			= index (l1,u1) i1 * rangeSize (l2,u2) + index (l2,u2) i2
			= index_out_of_range
	rangeSize :: (Ix e0,Ix e1) => !(!(e0,e1),!(e0,e1)) -> Int
    rangeSize ((l1,l2),(u1,u2))
		| null (range (l1,u1)) || null (range (l2,u2))
			= 0
			= index (l1,u1) u1 * rangeSize (l2,u2) + index (l2,u2) u2 + 1

range`2tuple_int_int :: !(!(!Int,Int),!(!Int,Int)) -> [(Int,Int)]
range`2tuple_int_int ((l1,l2),(u1,u2))
   	= [(i1,i2) | i1 <- [l1..u1], i2 <- [l2..u2]]

inRange`2tuple_int_int :: !(!(!Int,!Int),!(!Int,!Int)) -> !(!Int,!Int) -> Bool
inRange`2tuple_int_int ((l1,l2),(u1,u2)) (i1,i2)
	= i1>=l1 && i1<=u1 && i2>=l2 && i2<=u2

index`2tuple_int_int :: !(!(!Int,!Int),!(!Int,!Int)) -> !(!Int,!Int) -> Int
index`2tuple_int_int ((l1,l2),(u1,u2)) (i1,i2)
	| i1>=l1 && i1<=u1 && i2>=l2 && i2<=u2
		= (i1-l1) * (u2-l2+1) + (i2-l2)
		= index_out_of_range

rangeSize`2tuple_int_int :: !(!(!Int,!Int),!(!Int,!Int)) -> Int
rangeSize`2tuple_int_int ((l1,l2),(u1,u2))
	| l1<=u1 && l2<=u2
		= (u1-l1+1) * (u2-l2+1)
		= 0

instance (Ix e0,Ix e1,Ix e2) => Ix (e0,e1,e2) where	
	range :: (Ix e0,Ix e1,Ix e2) => !(!(e0,e1,e2),!(e0,e1,e2)) -> [(e0,e1,e2)]
    range ((l1,l2,l3),(u1,u2,u3))
		= [(i1,i2,i3) | i1<-range (l1,u1), i2<-range (l2,u2), i3<-range (l3,u3)]
	inRange :: (Ix e0,Ix e1,Ix e2) => !(!(e0,e1,e2),!(e0,e1,e2)) -> !(e0,e1,e2) -> Bool
	inRange ((l1,l2,l3),(u1,u2,u3)) (i1,i2,i3)
		= inRange (l1,u1) i1 && inRange (l2,u2) i2 && inRange (l3,u3) i3
	index :: (Ix e0,Ix e1,Ix e2) => !(!(e0,e1,e2),!(e0,e1,e2)) -> !(e0,e1,e2) -> Int
	index ((l1,l2,l3),(u1,u2,u3)) (i1,i2,i3)
		| inRange (l1,u1) i1 && inRange (l2,u2) i2 && inRange (l3,u3) i3
			# i = index (l1,u1) i1
			  i = i * rangeSize (l2,u2) + index (l2,u2) i2
			=     i * rangeSize (l3,u3) + index (l3,u3) i3
			= index_out_of_range
	rangeSize :: (Ix e0,Ix e1,Ix e2) => !(!(e0,e1,e2),!(e0,e1,e2)) -> Int
	rangeSize ((l1,l2,l3),(u1,u2,u3))
		| null (range (l1,u1)) || null (range (l2,u2)) || null (range (l3,u3))
			= 0
			# i = index (l1,u1) u1
			  i = i * rangeSize (l2,u2) + index (l2,u2) u2
			=     i * rangeSize (l3,u3) + index (l3,u3) u3 + 1

range`3tuple_int_int :: !(!(!Int,Int,Int),!(!Int,Int,Int)) -> [(Int,Int,Int)]
range`3tuple_int_int ((l1,l2,l3),(u1,u2,u3))
   	= [(i1,i2,i3) | i1 <- [l1..u1], i2 <- [l2..u2], i3 <- [l3..u3]]

inRange`3tuple_int_int :: !(!(!Int,!Int,!Int),!(!Int,!Int,!Int)) -> !(!Int,!Int,!Int) -> Bool
inRange`3tuple_int_int ((l1,l2,l3),(u1,u2,u3)) (i1,i2,i3)
	= i1>=l1 && i1<=u1 && i2>=l2 && i2<=u2 && i3>=l3 && i3<=u3

index`3tuple_int_int :: !(!(!Int,!Int,!Int),!(!Int,!Int,!Int)) -> !(!Int,!Int,!Int) -> Int
index`3tuple_int_int ((l1,l2,l3),(u1,u2,u3)) (i1,i2,i3)
	| i1>=l1 && i1<=u1 && i2>=l2 && i2<=u2 && i3>=l3 && i3<=u3
		# i = i1-l1
		  i = i * (u2-l2+1) + (i2-l2)
		=     i * (u3-l3+1) + (i3-l3)
		= index_out_of_range

rangeSize`3tuple_int_int :: !(!(!Int,!Int,!Int),!(!Int,!Int,!Int)) -> Int
rangeSize`3tuple_int_int ((l1,l2,l3),(u1,u2,u3))
	| l1<=u1 && l2<=u2 && l3<=u3
		= (u1-l1+1) * (u2-l2+1) * (u3-l3+1)
		= 0

instance (Ix e0,Ix e1,Ix e2,Ix e3) => Ix (e0,e1,e2,e3) where
	range :: (Ix e0,Ix e1,Ix e2,Ix e3) => !(!(e0,e1,e2,e3),!(e0,e1,e2,e3)) -> [(e0,e1,e2,e3)]
	range ((l1,l2,l3,l4),(u1,u2,u3,u4))
		= [(i1,i2,i3,i4) | i1<-range (l1,u1), i2<-range (l2,u2), i3<-range (l3,u3), i4<-range (l4,u4)]
	inRange :: (Ix e0,Ix e1,Ix e2,Ix e3) => !(!(e0,e1,e2,e3),!(e0,e1,e2,e3)) -> !(e0,e1,e2,e3) -> Bool
    inRange ((l1,l2,l3,l4),(u1,u2,u3,u4)) (i1,i2,i3,i4)
    	= inRange (l1,u1) i1 && inRange (l2,u2) i2 && inRange (l3,u3) i3 && inRange (l4,u4) i4
	index :: (Ix e0,Ix e1,Ix e2,Ix e3) => !(!(e0,e1,e2,e3),!(e0,e1,e2,e3)) -> !(e0,e1,e2,e3) -> Int
	index b@((l1,l2,l3,l4),(u1,u2,u3,u4)) i@(i1,i2,i3,i4)
		| inRange b i
			# i = index (l1,u1) i1
			  i = i * rangeSize (l2,u2) + index (l2,u2) i2
			  i = i * rangeSize (l3,u3) + index (l3,u3) i3
			=     i * rangeSize (l4,u4) + index (l4,u4) i4
			= index_out_of_range
	rangeSize :: (Ix e0,Ix e1,Ix e2,Ix e3) => !(!(e0,e1,e2,e3),!(e0,e1,e2,e3)) -> Int
    rangeSize b=:((l1,l2,l3,l4),(u1,u2,u3,u4))
		| null (range (l1,u1)) || null (range (l2,u2)) || null (range (l3,u3)) || null (range (l4,u4))
			= 0
			# i = index (l1,u1) u1
			  i = i * rangeSize (l2,u2) + index (l2,u2) u2
			  i = i * rangeSize (l3,u3) + index (l3,u3) u3
			=     i * rangeSize (l4,u4) + index (l4,u4) u4 + 1

instance (Ix e0,Ix e1,Ix e2,Ix e3,Ix e4) => Ix (e0,e1,e2,e3,e4) where
	range :: (Ix e0,Ix e1,Ix e2,Ix e3,Ix e4) => !(!(e0,e1,e2,e3,e4),!(e0,e1,e2,e3,e4)) -> [(e0,e1,e2,e3,e4)]
	range ((l0,l1,l2,l3,l4),(u0,u1,u2,u3,u4))
		= [(i0,i1,i2,i3,i4) | i0<-range (l0,u0), i1<-range (l1,u1), i2<-range (l2,u2), i3<-range (l3,u3), i4<-range (l4,u4)]
	inRange :: (Ix e0,Ix e1,Ix e2,Ix e3,Ix e4) => !(!(e0,e1,e2,e3,e4),!(e0,e1,e2,e3,e4)) -> !(e0,e1,e2,e3,e4) -> Bool
    inRange ((l0,l1,l2,l3,l4),(u0,u1,u2,u3,u4)) (i0,i1,i2,i3,i4)
    	= inRange (l0,u0) i0 && inRange (l1,u1) i1 && inRange (l2,u2) i2 && inRange (l3,u3) i3 && inRange (l4,u4) i4
	index :: (Ix e0,Ix e1,Ix e2,Ix e3,Ix e4) => !(!(e0,e1,e2,e3,e4),!(e0,e1,e2,e3,e4)) -> !(e0,e1,e2,e3,e4) -> Int
	index b@((l0,l1,l2,l3,l4),(u0,u1,u2,u3,u4)) i@(i0,i1,i2,i3,i4)
		| inRange b i
			# i = index (l0,u0) i0
			  i = i * rangeSize (l1,u1) + index (l1,u1) i1
			  i = i * rangeSize (l2,u2) + index (l2,u2) i2
			  i = i * rangeSize (l3,u3) + index (l3,u3) i3
			=     i * rangeSize (l4,u4) + index (l4,u4) i4
			= index_out_of_range
	rangeSize :: (Ix e0,Ix e1,Ix e2,Ix e3,Ix e4) => !(!(e0,e1,e2,e3,e4),!(e0,e1,e2,e3,e4)) -> Int
    rangeSize b=:((l0,l1,l2,l3,l4),(u0,u1,u2,u3,u4))
		| null (range (l0,u0)) || null (range (l1,u1)) || null (range (l2,u2)) || null (range (l3,u3)) || null (range (l4,u4))
			= 0
			# i = index (l0,u0) u0
			  i = i * rangeSize (l1,u1) + index (l1,u1) u1
			  i = i * rangeSize (l2,u2) + index (l2,u2) u2
			  i = i * rangeSize (l3,u3) + index (l3,u3) u3
			=     i * rangeSize (l4,u4) + index (l4,u4) u4 + 1

index_out_of_range
	= abort `"Ix.index: Index out of range."
