implementation module PreludeFloat where

import Prelude

import qualified StdEnv
import _SystemArray
from StdEnv import qualified class toReal(..),class toInt(..),class toString(..),class +++(..)
from StdReal import qualified instance toReal Int
from StdInt import instance toInt Double,instance toInt Char
from StdFloat import qualified class toFloat(..),instance toFloat Double,instance toReal Float

from StdInt import IF_INT_64_OR_32

import qualified StdDebug,qualified StdMisc
from StdString import instance toString Int,instance toString {#Char},instance +++ {#Char}

ExpMask :== 0x7ff00000

double_to_ints :: !Double -> (!Int,!Int)
double_to_ints r = `code inline {
	update_b 1 1
	update_b 0 0
}

ints_to_double :: !Int -> !Int -> Double
ints_to_double l h = `code inline {
	update_b 1 1
	update_b 0 0
}

ushiftr -: !Int !Int -> Int;
ushiftr a b = `code inline {
	shiftrU
}

(+>>) infix 7;
(+>>) a b :== ushiftr a b;

truncate_real_int :: !Double -> Int
truncate_real_int r = `code inline {
	truncateR
}

ceiling_real_int :: !Double -> Int
ceiling_real_int r = `code inline {
	ceilingR
}

round_real64 :: !Double -> !*{#Int} -> *{#Int}
round_real64 r a = `code inline {
	.d 1 2 r
		jsr	round_real64
	.o 1 0
}

truncate_real64 :: !Double -> !*{#Int} -> *{#Int}
truncate_real64 r a = `code inline {
	.d 1 2 r
		jsr	truncate_real64
	.o 1 0
}

entier_real64 :: !Double -> !*{#Int} -> *{#Int}
entier_real64 r a = `code inline {
	.d 1 2 r
		jsr	entier_real64
	.o 1 0
}

ceiling_real64 :: !Double -> !*{#Int} -> *{#Int}
ceiling_real64 r a = `code inline {
	.d 1 2 r
		jsr	ceiling_real64
	.o 1 0
}

int64a_to_real :: !{#Int} -> Double
int64a_to_real a = `code inline {
	.d 1 0
		jsr	int64a_to_real
	.o 0 2 r
}

create_unintialized_int_array :: !Int -> *{#Int};
create_unintialized_int_array size = `code inline {	
	create_array_ INT 0 1
}

instance Real Double where
	toRational :: !Double -> Rational
	toRational r
		| r>=4503599627370496.0 /* 1<<52 */
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= 1 :% 0
				= (large_real_to_integer_p exp h l :% 1)
		| r<= -4503599627370496.0 /* - 1<<52 */
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= -1 :% 0
				= (large_real_to_integer_n exp h l :% 1)
		# (l,h) = double_to_ints r
		  sign = h StdEnv.>>IF_INT_64_OR_32 63 31
		  sig_h = h StdEnv.bitand 0xfffff
		  exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
		| l==0
			| sig_h==0
				| exp==0
					= 0 :% 1
					# exp = exp - 1023
					| exp>=0
						= power_2_s_integer sign exp :% 1
						= if (h<0) (-1) 1 :% power_2_integer (-exp)
				# n_trailing_0s = count_trailing_zeros sig_h
				  exp = exp - 1023
				  exp_t = exp-(20-n_trailing_0s)
				  sig_h = sig_h StdEnv.bitor 0x100000
				| exp_t>=0
					| exp<32
						| exp<20
							# n = sig_h StdEnv.>> (20-exp)
							# n=(n StdEnv.bitxor sign)-sign
							= {integer_s=n,integer_a={}} :% 1
						| exp<31
							# n = sig_h StdEnv.<< (exp-20)
							# n=(n StdEnv.bitxor sign)-sign
							= {integer_s=n,integer_a={}} :% 1
						//exp==31
							# n = sig_h StdEnv.<< 11
							= {integer_s=sign,integer_a={n}} :% 1
						| exp<52
							# h = sig_h StdEnv.>> (52-exp)
							# l = sig_h StdEnv.<< (exp-20)
							= {integer_s=sign,integer_a={l,h}} :% 1
							// not possible
							= error_in_toRational_real
					# n=sig_h StdEnv.>>n_trailing_0s
					# n=(n StdEnv.bitxor sign)-sign
					= {integer_s=n,integer_a={}} :% power_2_integer (-exp_t)
			# n_trailing_0s = count_trailing_zeros l
			  exp = exp - 1023
			  exp_t = exp-(52-n_trailing_0s)
			  sig_h = sig_h StdEnv.bitor 0x100000
			| exp_t>=0
				| exp<32
					| exp<31
						| exp<20
							// not possible
							= error_in_toRational_real
						# n = (sig_h StdEnv.<< (exp-20)) + (ushiftr l (52-exp))
						# n=(n StdEnv.bitxor sign)-sign
						= {integer_s=n,integer_a={}} :% 1
					//exp==31
						# n = (sig_h StdEnv.<< 11) + (ushiftr l 21)
						= {integer_s=sign,integer_a={n}} :% 1
				| exp<52
					# shift_r = 52-exp
					# h = ushiftr sig_h shift_r
					# l = (ushiftr l shift_r) + (sig_h StdEnv.<<(exp-20))
					= {integer_s=sign,integer_a={l,h}} :% 1
					// not possible
					= error_in_toRational_real
			| n_trailing_0s<20
				| n_trailing_0s==0
					= {integer_s=sign,integer_a={l,sig_h}} :% power_2_integer (-exp_t)
					# h = sig_h StdEnv.>> n_trailing_0s
					# l = (ushiftr l n_trailing_0s) + (sig_h StdEnv.<< (32-n_trailing_0s))
					= {integer_s=sign,integer_a={l,h}} :% power_2_integer (-exp_t)
				# n = (ushiftr l n_trailing_0s) + (sig_h StdEnv.<< (32-n_trailing_0s))
				# n=(n StdEnv.bitxor sign)-sign
				= {integer_s=n,integer_a={}} :% power_2_integer (-exp_t)
	where
		error_in_toRational_real
			= StdMisc.abort `"toRational Double"

instance Real Float where
	toRational :: !Float -> Rational
	toRational f = toRational (StdEnv.toReal f)

power_2_integer n
	| n<31
		= {integer_s=1 StdEnv.<<n,integer_a={}}
		# max_index = n StdEnv.>> 5
		  bit_n = n StdEnv.bitand 31
		= {integer_s=0,integer_a={createArray (max_index+1) 0 & [max_index]=1 StdEnv.<<bit_n}}

power_2_s_integer s n
	| n<31
		# n=1 StdEnv.<<n
		# n=(n StdEnv.bitxor s)-s
		= {integer_s=n,integer_a={}}
	| n==31
		| s>=0
			= {integer_s=0,integer_a={0x80000000}}
			= {integer_s=0x80000000,integer_a={}}
		# max_index = n StdEnv.>> 5
		  bit_n = n StdEnv.bitand 31
		= {integer_s=s,integer_a={createArray (max_index+1) 0 & [max_index]=1 StdEnv.<<bit_n}}

DeBruijnSequence:==IF_INT_64_OR_32 0x218a392cd3d5dbf 0x77cb531

count_trailing_zeros :: !Int -> Int
count_trailing_zeros i
	# p2=i StdEnv.bitand (-i)
	# s=IF_INT_64_OR_32
		`"\x00\x01\x02\x07\x03\x0d\x08\x13\x04\x19\x0e\x1c\x09\x22\x14\x28\x05\x11\x1a\x26\x0f\x2e\x1d\x30\x0a\x1f\x23\x36\x15\x32\x29\x39\x3f\x06\x0c\x12\x18\x1b\x21\x27\x10\x25\x2d\x2f\x1e\x35\x31\x38\x3e\x0b\x17\x20\x24\x2c\x34\x37\x3d\x16\x2b\x33\x3c\x2a\x3b\x3a"
		`"\x00\x01\x1c\x02\x1d\x0e\x18\x03\x1e\x16\x14\x0f\x19\x11\x04\x08\x1f\x1b\x0d\x17\x15\x13\x10\x07\x1a\x0c\x12\x06\x0b\x05\x0a\x09";
	= StdEnv.toInt (s~[ushiftr (p2 * DeBruijnSequence) (IF_INT_64_OR_32 58 27)])

round_real_to_integer :: !Double -> Integer
round_real_to_integer r
	| r>= -2147483648.5
		| r<2147483647.5
			# i=StdEnv.toInt r
			= {integer_s=i,integer_a={}}
			| r<4611686018427387904.0 /*1<<62*/
				# a = round_real64 r (create_unintialized_int_array 2)
				= {integer_s=0,integer_a=a}
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"round Double Infinity"
				= large_real_to_integer_p exp h l
		| r>= -4611686018427387904.0 /*- 1<<62*/
			# a = round_real64 (-r) (create_unintialized_int_array 2)
			= {integer_s= -1,integer_a=a}
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"round Double -Infinity"
				= large_real_to_integer_n exp h l

truncate_real_to_integer :: !Double -> Integer
truncate_real_to_integer r
	| r> -2147483649.0
		| r<2147483648.0
			# i=truncate_real_int r
			= {integer_s=i,integer_a={}}
		| r<4611686018427387904.0 /*1<<62*/
			# a = truncate_real64 r (create_unintialized_int_array 2)
			= {integer_s=0,integer_a=a}
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"truncate Double Infinity"
				= large_real_to_integer_p exp h l
		| r>= -4611686018427387904.0 /*- 1<<62*/
			# a = truncate_real64 (-r) (create_unintialized_int_array 2)
			= {integer_s= -1,integer_a=a}
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"truncate Double -Infinity"
				= large_real_to_integer_n exp h l

floor_real_to_integer :: !Double -> Integer
floor_real_to_integer r
	| r>= -2147483648.0
		| r<2147483648.0
			# i=StdEnv.entier r
			= {integer_s=i,integer_a={}}
		| r<4611686018427387904.0 /*1<<62*/
			# a = entier_real64 r (create_unintialized_int_array 2)
			= {integer_s=0,integer_a=a}
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"floor Double Infinity"
				= large_real_to_integer_p exp h l
		| r>= -4611686018427387904.0 /*- 1<<62*/
			# a = ceiling_real64 /* not entier_real64*/ (-r) (create_unintialized_int_array 2)
			= {integer_s= -1,integer_a=a}
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"floor Double -Infinity"
				= large_real_to_integer_n exp h l

ceiling_real_to_integer :: !Double -> Integer
ceiling_real_to_integer r
	| r> -2147483649.0
		| r<=2147483647.0
			# i=ceiling_real_int r
			= {integer_s=i,integer_a={}}
		| r<4611686018427387904.0 /*1<<62*/
			# a = ceiling_real64 r (create_unintialized_int_array 2)
			= {integer_s=0,integer_a=a}
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"ceiling Double Infinity"
				= large_real_to_integer_p exp h l
		| r>= -4611686018427387904.0 /*- 1<<62*/
			# a = entier_real64 /* not ceiling_real64*/ (-r) (create_unintialized_int_array 2)
			= {integer_s= -1,integer_a=a}
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"ceiling Double -Infinity"
				= large_real_to_integer_n exp h l

proper_integer_fraction_real :: !Double -> (!Integer,!Double)
proper_integer_fraction_real r
	| r> -2147483649.0
		| r<2147483648.0
			# i=truncate_real_int r
			# f=r-StdEnv.toReal i
			= ({integer_s=i,integer_a={}},f)
		| r<4611686018427387904.0 /*1<<62*/
			# a = truncate_real64 r (create_unintialized_int_array 2)
			# f = r - int64a_to_real a
			= ({integer_s=0,integer_a=a},f)
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"properFraction Double Infinity"
				= (large_real_to_integer_p exp h l,0.0)
		| r>= -4611686018427387904.0 /*- 1<<62*/
			# a = truncate_real64 (-r) (create_unintialized_int_array 2)
			# f = r + int64a_to_real a
			= ({integer_s= -1,integer_a=a},f)
			# (l,h) = double_to_ints r
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			| exp==0x7ff
				= StdEnv.abort `"properFraction Double -Infinity"
				= (large_real_to_integer_n exp h l,0.0)

large_real_to_integer_p :: !Int -> !Int -> !Int -> Integer
large_real_to_integer_p exp h l
	# max_bit_n=exp-1023
	# max_i=max_bit_n StdEnv.>>5
	# h = (h StdEnv.bitand 0xfffff) + 0x100000
	# a = createArray (max_i+1) 0
	# shift=((max_bit_n-52) StdEnv.bitand 31)
	| shift<=11
		| shift==0
			# a = {a & [max_i-1]=l,[max_i]=h}
			= {integer_s=0,integer_a=a}
			# h=(h StdEnv.<<shift) StdEnv.bitor (l+>>(32-shift))
			# l=l StdEnv.<<shift
			# a = {a & [max_i-1]=l,[max_i]=h}
			= {integer_s=0,integer_a=a}
		# shift_r=32-shift
		# m=(h StdEnv.<<shift) StdEnv.bitor (l+>>shift_r)
		# l=l StdEnv.<<shift
		# h=h+>>shift_r
		# a = {a & [max_i-2]=l,[max_i-1]=m,[max_i]=h}
		= {integer_s=0,integer_a=a}

large_real_to_integer_n :: !Int -> !Int -> !Int -> Integer
large_real_to_integer_n exp h l
	# max_bit_n=exp-1023
	# max_i=max_bit_n StdEnv.>>5
	# h = (h StdEnv.bitand 0xfffff) + 0x100000
	# a = createArray (max_i+1) 0
	# shift=((max_bit_n-52) StdEnv.bitand 31)
	| shift<=11
		| shift==0
			# a = {a & [max_i-1]=l,[max_i]=h}
			= {integer_s= -1,integer_a=a}
			# h=(h StdEnv.<<shift) StdEnv.bitor (l+>>(32-shift))
			# l=l StdEnv.<<shift
			# a = {a & [max_i-1]=l,[max_i]=h}
			= {integer_s= -1,integer_a=a}
		# shift_r=32-shift
		# m=(h StdEnv.<<shift) StdEnv.bitor (l+>>shift_r)
		# l=l StdEnv.<<shift
		# h=h+>>shift_r
		# a = {a & [max_i-2]=l,[max_i-1]=m,[max_i]=h}
		= {integer_s= -1,integer_a=a}

pow2 exp :== ints_to_double 0 (exp StdEnv.<< 20)

instance RealFloat Double where

	floatRadix _ = 2

	floatDigits _ = 53

	floatRange :: Double -> (!Int,!Int)
	floatRange _ = (-1021,1024)

	decodeFloat :: !Double -> (!Integer,!Int)
	decodeFloat r
		# (l,h) = double_to_ints r
		# sign = h StdEnv.>> 31
		# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
		# h = h StdEnv.bitand 0xfffff
		| exp/=0
			// what to do whith Infinity or NAN ?
			# exp=exp-1075
			# n={integer_s=sign,integer_a={l,h StdEnv.bitor 0x100000}}
			= (n,exp)
		| h+h==0 && l==0
			// what to do with 0 or -0 ?
			= (0,0)
			// denormalized number
			# (l,h) = double_to_ints (r*4503599627370496.0 /* 1<<52 */)
			# sign = h StdEnv.>> 31
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			# h = h StdEnv.bitand 0xfffff
			# exp=exp-1127
			# n={integer_s=sign,integer_a={l,h StdEnv.bitor 0x100000}}
			= (n,exp)

	encodeFloat :: !Integer -> !Int -> Double
	encodeFloat {integer_s,integer_a} exp
		# sa=size integer_a
		| sa==0
			| integer_s==0
				= 0.0
				= scale (StdEnv.toReal integer_a~[0]) exp
		# l=integer_a~[0]
		#! rl = (StdEnv.toReal (l-2147483648))+2147483648.0
		| sa==1
			| integer_s>=0
				= scale rl exp
				= scale (-rl) exp
		| sa==2
			# h=integer_a~[1]
			| h StdEnv.bitand 0xffe00000==0
				#! rh = StdEnv.toReal h
				| integer_s>=0
					= scale (rh * 4294967296.0 /*1<<32*/+rl) exp
					= scale (rh * -4294967296.0 /*1<<32*/-rl) exp
				= StdEnv.abort `"encodeFloat: invalid significand"
		where
			scale r exp
				| exp > -1022
					| exp <= 1023
						= r * pow2 (exp+1023)
						= r / 0.0 // infinity
					= scale (r*(pow2 1)) (exp+1022)

	exponent :: !Double -> Int
	exponent r
		# (l,h) = double_to_ints r
		# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
		| exp/=0
			// what to do with Infinity or NAN ?
			= exp-1022
		| h+h==0 && l==0
			= 0
			// denormalized number
			# (l,h) = double_to_ints (r*4503599627370496.0 /* 1<<52 */)
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			= exp-1074

	significand :: !Double -> Double
	significand r
		# (l,h) = double_to_ints r
		# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
		| exp/=0
			# h = (h StdEnv.bitand 0x800fffff) StdEnv.bitor 0x3fe00000
			= ints_to_double l h
		| h+h==0 && l==0
			= 0.0
			// denormalized number
			# (l,h) = double_to_ints (r*4503599627370496.0 /* 1<<52 */)
			# h = (h StdEnv.bitand 0x800fffff) StdEnv.bitor 0x3fe00000
			= ints_to_double l h

	scaleFloat :: !Int -> !Double -> Double
	scaleFloat a r
		| a > -1022
			| a <= 1023
				= r * pow2 (a+1023)
			| a <= 2100
				# r=r*ints_to_double 0 0x7FE00000
				# a=a-1023
				= scaleFloat a r
				= r * (1.0/0.0)
			| a >= -2100
				# r=r*ints_to_double 0 0x00100000
				# a=a+1022
				= scaleFloat a r
				= r / (1.0/0.0)

	isNaN :: !Double -> Bool
	isNaN r
		# (l,h) = double_to_ints r
		= (h StdEnv.bitand ExpMask)==ExpMask && not ((h StdEnv.bitand 0xfffff)==0 && l==0)

	isInfinite :: !Double -> Bool
	isInfinite r
		# (l,h) = double_to_ints r
		= h+h==0xffe00000 && l==0

	isDenormalized :: !Double -> Bool
	isDenormalized r
		# (l,h) = double_to_ints r
		= (h StdEnv.bitand ExpMask)==0 && not (h+h==0 && l==0)
		
	isNegativeZero :: !Double -> Bool
	isNegativeZero r
		# (l,h) = double_to_ints r
		= h==0x80000000 && l==0

	isIEEE _ = True

	atan2 :: !Double -> !Double -> Double
	atan2 y x
		| x>0 = atan (y/x)
		| x==0 && y>0 = pi/2
		| x<0 && y>0 = pi + atan (y/x)
		| (x<=0 && y<0) ||
		  (x<0 && DoubleIsNegativeZero y) ||
		  (DoubleIsNegativeZero x && DoubleIsNegativeZero y)
			= -atan2 (-y) x
		| y==0 && (x<0 || DoubleIsNegativeZero x)
			= pi -- must be after the previous test on zero y
		| x==0 && y==0 = y -- must be after the other double zero tests
		| otherwise = x + y -- x or y is a NaN, return a NaN (via +)

DoubleIsNegativeZero r :== let (l,h) = double_to_ints r in h==0x80000000 && l==0

instance RealFloat Float where

	floatRadix _ = 2

	floatDigits _ = 53

	floatRange :: Float -> (!Int,!Int)
	floatRange _ = (-1021,1024)

	decodeFloat :: !Float -> (!Integer,!Int)
	decodeFloat f
		= decodeFloat (StdEnv.toReal f)

	encodeFloat :: !Integer -> !Int -> Float
	encodeFloat integer exp
		= StdFloat.toFloat (encodeFloat integer exp)

	exponent :: !Float -> Int
	exponent f
		# r = StdEnv.toReal f
		# (l,h) = double_to_ints r
		# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
		| exp/=0
			// what to do whith Infinity or NAN ?
			= exp-1022
		| h+h==0 && l==0
			= 0
			// denormalized number
			# (l,h) = double_to_ints (r*4503599627370496.0 /* 1<<52 */)
			# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
			= exp-1074

	significand :: !Float -> Float
	significand f
		# r = StdEnv.toReal f
		# (l,h) = double_to_ints r
		# exp = (h StdEnv.bitand ExpMask)StdEnv.>>20
		| exp/=0
			# h = (h StdEnv.bitand 0x800fffff) StdEnv.bitor 0x3fe00000
			= StdFloat.toFloat (ints_to_double l h)
		| h+h==0 && l==0
			= 0.0
			// denormalized number
			# (l,h) = double_to_ints (r*4503599627370496.0 /* 1<<52 */)
			# h = (h StdEnv.bitand 0x800fffff) StdEnv.bitor 0x3fe00000
			= StdFloat.toFloat (ints_to_double l h)

	scaleFloat :: !Int -> !Float -> Float
	scaleFloat a f
		# r = StdEnv.toReal f
		| a > -1022
			| a <= 1023
				= StdFloat.toFloat (r * pow2 (a+1023))
			| a <= 2100
				# r=r*ints_to_double 0 0x7FE00000
				# a=a-1023
				= scaleFloat a (StdFloat.toFloat r)
				= StdFloat.toFloat (r * (1.0/0.0))
			| a >= -2100
				# r=r*ints_to_double 0 0x00100000
				# a=a+1022
				= scaleFloat a (StdFloat.toFloat r)
				= StdFloat.toFloat (r / (1.0/0.0))

	isNaN :: !Float -> Bool
	isNaN f
		# (l,h) = double_to_ints (StdEnv.toReal f)
		= (h StdEnv.bitand ExpMask)==ExpMask && not ((h StdEnv.bitand 0xfffff)==0 && l==0)

	isInfinite :: !Float -> Bool
	isInfinite f
		# (l,h) = double_to_ints (StdEnv.toReal f)
		= h+h==0xffe00000 && l==0

	isDenormalized :: !Float -> Bool
	isDenormalized f
		# (l,h) = double_to_ints (StdEnv.toReal f)
		= (h StdEnv.bitand ExpMask)==0 && not (h+h==0 && l==0)
		
	isNegativeZero :: !Float -> Bool
	isNegativeZero f
		# (l,h) = double_to_ints (StdEnv.toReal f)
		= h==0x80000000 && l==0

	isIEEE _ = True

	atan2 :: !Float -> !Float -> Float
	atan2 y x = StdFloat.toFloat (atan2 (StdEnv.toReal y) (StdEnv.toReal x))
