system module PreludeInline where

import PreludeTypes

instance Integral Int where
	toInteger :: !Int -> Integer
	quot :: !Int -> !Int -> Int
	rem :: !Int -> !Int -> Int
	div :: !Int -> !Int -> Int
	mod :: !Int -> !Int -> Int
	quotRem :: !Int -> !Int -> (!Int,!Int)
	divMod :: !Int -> !Int -> (!Int,!Int)

instance Fractional Double where
	(/) :: !Double -> !Double -> Double
	recip :: !Double -> Double
	fromRational :: !Rational -> Double

instance Fractional Float where
	(/) :: !Float -> !Float -> Float
	recip :: !Float -> Float
	fromRational :: !Rational -> Float

instance Floating Double where
	sqrt :: !Double -> Double
	log :: !Double -> Double
	exp :: !Double -> Double
	sin :: !Double -> Double
	cos :: !Double -> Double
	tan :: !Double -> Double
	asin :: !Double -> Double
	acos :: !Double -> Double
	atan :: !Double -> Double
	pi :: Double
	sinh :: !Double -> Double
	cosh :: !Double -> Double
	tanh :: !Double -> Double
	asinh :: !Double -> Double
	acosh :: !Double -> Double
	atanh :: !Double -> Double
	(**) :: !Double -> !Double -> Double
	logBase :: !Double -> !Double -> Double

instance Floating Float where
	sqrt :: !Float -> Float
	log :: !Float -> Float
	exp :: !Float -> Float
	sin :: !Float -> Float
	cos :: !Float -> Float
	tan :: !Float -> Float
	asin :: !Float -> Float
	acos :: !Float -> Float
	atan :: !Float -> Float
	pi :: Float
	sinh :: !Float -> Float
	cosh :: !Float -> Float
	tanh :: !Float -> Float
	asinh :: !Float -> Float
	acosh :: !Float -> Float
	atanh :: !Float -> Float
	(**) :: !Float -> !Float -> Float
	logBase :: !Float -> !Float -> Float

instance RealFrac Double where
	round    :: (Integral b) => !Double -> b special b=Int:==round`double_to_int    ; b=Integer
	truncate :: (Integral b) => !Double -> b special b=Int:==truncate`double_to_int ; b=Integer
	floor    :: (Integral b) => !Double -> b special b=Int:==floor`double_to_int    ; b=Integer
	ceiling  :: (Integral b) => !Double -> b special b=Int:==ceiling`double_to_int  ; b=Integer
	properFraction :: (Integral b) => !Double -> (b,!Double)

instance RealFrac Float where
	round    :: (Integral b) => !Float -> b special b=Int:==round_float_to_int    ; b=Integer
	truncate :: (Integral b) => !Float -> b special b=Int:==truncate_float_to_int ; b=Integer
	floor    :: (Integral b) => !Float -> b special b=Int:==floor_float_to_int    ; b=Integer
	ceiling  :: (Integral b) => !Float -> b special b=Int:==ceiling_float_to_int  ; b=Integer
	properFraction :: (Integral b) => !Float -> (b,!Float)

round`double_to_int :: !Double -> Int
truncate`double_to_int :: !Double -> Int
floor`double_to_int :: !Double -> Int
ceiling`double_to_int :: !Double -> Int

round_float_to_int :: !Float -> Int
truncate_float_to_int :: !Float -> Int
floor_float_to_int :: !Float -> Int
ceiling_float_to_int :: !Float -> Int

instance Enum Char where
	toEnum :: !Int -> Char
	fromEnum :: !Char -> Int

instance Eq Int where
	(==) :: !Int -> !Int -> Bool
	(/=) :: !Int -> !Int -> Bool

instance Eq Char where
	(==) :: !Char -> !Char -> Bool
	(/=) :: !Char -> !Char -> Bool

instance Eq Double where
	(==) :: !Double -> !Double -> Bool
	(/=) :: !Double -> !Double -> Bool

instance Eq Float where
	(==) :: !Float -> !Float -> Bool
	(/=) :: !Float -> !Float -> Bool

instance Eq Bool where
	(==) :: !Bool -> !Bool -> Bool
	(/=) :: !Bool -> !Bool -> Bool

instance Ord Char where
	(<) infix  4 :: !Char -> !Char -> Bool
	(<=) infix 4 :: !Char -> !Char -> Bool
	(>=) infix 4 :: !Char -> !Char -> Bool
	(>) infix  4 :: !Char -> !Char -> Bool
	compare :: !Char -> !Char -> Ordering
	min :: !Char -> !Char -> Char
	max :: !Char -> !Char -> Char

instance Ord Int where
	(<) infix  4 :: !Int -> !Int -> Bool
	(<=) infix 4 :: !Int -> !Int -> Bool
	(>=) infix 4 :: !Int -> !Int -> Bool
	(>) infix  4 :: !Int -> !Int -> Bool
	compare :: !Int -> !Int -> Ordering
	min :: !Int -> !Int -> Int
	max :: !Int -> !Int -> Int

instance Ord Double where
	(<) infix  4 :: !Double -> !Double -> Bool
	(<=) infix 4 :: !Double -> !Double -> Bool
	(>=) infix 4 :: !Double -> !Double -> Bool
	(>) infix  4 :: !Double -> !Double -> Bool
	compare :: !Double -> !Double -> Ordering
	min :: !Double -> !Double -> Double
	max :: !Double -> !Double -> Double

instance Ord Float where
	(<) infix  4 :: !Float -> !Float -> Bool
	(<=) infix 4 :: !Float -> !Float -> Bool
	(>=) infix 4 :: !Float -> !Float -> Bool
	(>) infix  4 :: !Float -> !Float -> Bool
	compare :: !Float -> !Float -> Ordering
	min :: !Float -> !Float -> Float
	max :: !Float -> !Float -> Float

instance Num Int where
	(+) :: !Int -> !Int -> Int
	(-) :: !Int -> !Int -> Int
	(*) :: !Int -> !Int -> Int
	abs :: !Int -> Int
	signum :: !Int -> Int
	negate :: !Int -> Int
	fromInteger :: !Integer -> Int

instance Num Double where
	(+) :: !Double -> !Double -> Double
	(-) :: !Double -> !Double -> Double
	(*) :: !Double -> !Double -> Double
	abs :: !Double -> Double
	signum :: !Double -> Double
	negate :: !Double -> Double
	fromInteger :: !Integer -> Double

instance Num Float where
	(+) :: !Float -> !Float -> Float
	(-) :: !Float -> !Float -> Float
	(*) :: !Float -> !Float -> Float
	abs :: !Float -> Float
	signum :: !Float -> Float
	negate :: !Float -> Float
	fromInteger :: !Integer -> Float

instance Eq Ordering where
	(==) :: !Ordering -> !Ordering -> Bool
	(/=) :: !Ordering -> !Ordering -> Bool

instance Enum Double where
	toEnum :: !Int -> Double
	fromEnum :: !Double -> Int
	succ :: !Double -> Double
	pred :: !Double -> Double
	enumFrom :: (Double -> [Double])
	enumFromThen :: (Double -> Double -> [Double])
	enumFromTo :: (Double -> Double -> [Double])
	enumFromThenTo :: (Double -> Double -> Double -> [Double])

instance Enum Float where
	toEnum :: !Int -> Float
	fromEnum :: !Float -> Int
	succ :: !Float -> Float
	pred :: !Float -> Float
	enumFrom :: (Float -> [Float])
	enumFromThen :: (Float -> Float -> [Float])
	enumFromTo :: (Float -> Float -> [Float])
	enumFromThenTo :: (Float -> Float -> Float -> [Float])

desc0_number' :: !a -> Int
get_desc' :: !a -> Int;
get_desc0_number' :: !Int -> Int;
equal_constructor :: !a -> !a -> Bool
less_constructor :: !a -> !a -> Bool
less_or_equal_constructor :: !a -> !a -> Bool;
greater_constructor :: !a -> !a -> Bool
greater_or_equal_constructor :: !a -> !a -> Bool
