implementation module PreludeInternal where

from StdChar import isUpper,isDigit,isHexDigit,isOctDigit
from StdInt import bitor,bitand,IF_INT_64_OR_32
from StdInt import qualified instance + Int,instance one Int
import StdArray

from PreludeLibraryInline import chr,ord

string_to_char_list :: !{#Char} -> [Char]
string_to_char_list s = string_to_char_list 0 s
where
	string_to_char_list i s
		| i<size s
			#! c=s~[i]
			= c:string_to_char_list (i+1) s
			= []

string_to_char_list_append :: !{#Char} -> [Char] -> [Char]
string_to_char_list_append s t = string_to_char_list_append 0 s t
where
	string_to_char_list_append i s t
		| i<size s
			#! c=s~[i]
			= c:string_to_char_list_append (i+1) s t
			= t

char_list_to_string :: ![Char] -> *{#Char}
char_list_to_string l = {c;;c<-l}

char_list_to_zstring :: [Char] -> *{#Char}
char_list_to_zstring cl
	= fill_string 0 cl (createArray (length cl+1) '\0')
  where
	fill_string :: !Int -> ![Char] -> !*{#Char} -> *{#Char}
	fill_string i (c:s) a
		= fill_string (i+1) s {a & [i]=c}
	fill_string i [] a
		= a

DigitToInt c :== ord c - 48

lexLitChar :: ![Char] -> [([Char],[Char])]
lexLitChar ('\\':s) = map (prefix '\\') (lexEsc s)
  where
	lexEsc (c:s)     | c `elem` "abfnrtv\\\"'" = [([c],s)]
	lexEsc ('^':c:s) | c >= '@' && c <= '_' = [(['^',c],s)]
	--Numeric escapes
	lexEsc ('o':s) = [prefix 'o' (span isOctDigit s)]
	lexEsc ('x':s) = [prefix 'x' (span isHexDigit s)]
	lexEsc s@(d:_) | isDigit d = [span isDigit s]
	--Very crude approximation to \XYZ.
	lexEsc s@(c:_) | isUpper c = [span isCharName s]
	lexEsc _ = []

	isCharName c = isUpper c || isDigit c
	prefix c (t,s) = (c:t, s)
lexLitChar (c:s) = [([c],s)]
lexLitChar "" = []

readEsc :: ![Char] -> [(Char,[Char])]
readEsc ('a':s) = [('\a',s)]
readEsc ('b':s) = [('\b',s)]
readEsc ('f':s) = [('\f',s)]
readEsc ('n':s) = [('\n',s)]
readEsc ('r':s) = [('\r',s)]
readEsc ('t':s) = [('\t',s)]
readEsc ('v':s) = [('\v',s)]
readEsc ('\\':s) = [('\\',s)]
readEsc ('"':s) = [('"',s)]
readEsc ('\'':s) = [('\'',s)]
readEsc ('^':c:s) | c >= '@' && c <= '_'
	= [(chr (ord c - ord '@'), s)]
readEsc (d:s)
	| isDigit d
		# (n,t) = read_rest_int s (DigitToInt d)
		= [(chr n, t)]
readEsc ('o':d:s)
	| isOctDigit d
		# (n,t) = read_rest_oct_int s (DigitToInt d)
		= [(chr n, t)]
		= []
readEsc ('x':d:s)
	| isDigit d
		# (n,t) = read_rest_hex_int s (DigitToInt d)
		= [(chr n, t)]
	# du=(ord d) bitor 0x20
	| du>=97 && du<=102
		# (n,t) = read_rest_hex_int s (du-87)
		= [(chr n, t)]
readEsc s@(c:t)
	| isUpper c
		= case (c,t) of
			('A',['CK';t]) -> lt2 '\ACK' t
			('B',['S';t]) -> lt2 '\BS' t
			('B',['EL';t]) -> lt2 '\BEL' t
			('C',['AN';t]) -> lt2 '\CAN' t
			('C',['R';t]) -> lt2 '\CR' t
			('D',['C1';t]) -> lt2 '\DC1' t
			('D',['C2';t]) -> lt2 '\DC2' t
			('D',['C3';t]) -> lt2 '\DC3' t
			('D',['C4';t]) -> lt2 '\DC4' t
			('D',['EL';t]) -> lt2 '\DEL' t
			('D',['LE';t]) -> lt2 '\DLE' t
			('E',['M';t]) -> lt2 '\EM' t
			('E',['NQ';t]) -> lt2 '\ENQ' t
			('E',['OT';t]) -> lt2 '\EOT' t
			('E',['SC';t]) -> lt2 '\ESC' t
			('E',['TB';t]) -> lt2 '\ETB' t
			('E',['TX';t]) -> lt2 '\ETX' t
			('F',['F';t]) -> lt2 '\FF' t
			('F',['S';t]) -> lt2 '\FS' t
			('G',['S';t]) -> lt2 '\GS' t
			('H',['T';t]) -> lt2 '\HT' t
			('L',['F';t]) -> lt2 '\LF' t
			('N',['AK';t]) -> lt2 '\NAK' t
			('N',['UL';t]) -> lt2 '\NUL' t
			('R',['S';t]) -> lt2 '\RS' t
			('S',['I';t]) -> lt2 '\SI' t
			('S',['OH';t]) -> lt2 '\SOH' t
			('S',['O';t]) -> lt2 '\SO' t
			('S',['P';t]) -> lt2 ' ' t
			('S',['TX';t]) -> lt2 '\STX' t
			('S',['UB';t]) -> lt2 '\SUB' t
			('S',['YN';t]) -> lt2 '\SYN' t
			('U',['S';t]) -> lt2 '\US' t
			('V',['T';t]) -> lt2 '\VT' t
			_ -> []
		where
			lt2 c l = [(c,l)]
readEsc _ = []

read_rest_int :: ![Char] -> !Int -> (!Int,![Char])
read_rest_int (c:s) v
	| isDigit c
		= read_rest_int s (v*10+DigitToInt c)
read_rest_int s v
	= (v,s)

read_rest_oct_int :: ![Char] -> !Int -> (!Int,![Char])
read_rest_oct_int (c:s) v
	| isOctDigit c
		= read_rest_oct_int s (v*8+DigitToInt c)
read_rest_oct_int s v
	= (v,s)

read_rest_hex_int :: ![Char] -> !Int -> (!Int,![Char])
read_rest_hex_int (c:s) v
	| isDigit c
		= read_rest_hex_int s (v*16+DigitToInt c)
	# cu=(ord c) bitor 0x20
	| cu>=97 && cu<=102
		= read_rest_hex_int s (v*16+(cu-87))
read_rest_hex_int s v
	= (v,s)

ShowChar c s :== c:s

showLitChar :: !Char -> ([Char] -> [Char])
showLitChar c
	| c >= '\DEL'
		| c == '\DEL'
			= showString "\\DEL"
			= ShowChar '\\' . ProtectEsc isDigit (shows (ord c))
showLitChar '\\' = \s -> ['\\\\';s]
showLitChar c | c >= ' ' = ShowChar c
showLitChar '\t' = show_bs_char 't'
showLitChar '\n' = show_bs_char 'n'
showLitChar '\r' = show_bs_char 'r'
showLitChar '\a' = show_bs_char 'a'
showLitChar '\b' = show_bs_char 'b'
showLitChar '\f' = show_bs_char 'f'
showLitChar '\v' = show_bs_char 'v'
showLitChar '\SO' = ProtectEsc (== 'H') (\s -> ['\\SO';s])
showLitChar c = \s -> show_ascii c s
	where
	show_ascii '\NUL' s = ['\\NUL';s]
	show_ascii '\SOH' s = ['\\SOH';s]
	show_ascii '\STX' s = ['\\STX';s]
	show_ascii '\ETX' s = ['\\ETX';s]
	show_ascii '\EOT' s = ['\\EOT';s]
	show_ascii '\ENQ' s = ['\\ENQ';s]
	show_ascii '\ACK' s = ['\\ACK';s]
	show_ascii '\SI' s = ['\\SI';s]
	show_ascii '\DLE' s = ['\\DLE';s]
	show_ascii '\DC1' s = ['\\DC1';s]
	show_ascii '\DC2' s = ['\\DC2';s]
	show_ascii '\DC3' s = ['\\DC3';s]
	show_ascii '\DC4' s = ['\\DC4';s]
	show_ascii '\NAK' s = ['\\NAK';s]
	show_ascii '\SYN' s = ['\\SYN';s]
	show_ascii '\ETB' s = ['\\ETB';s]
	show_ascii '\CAN' s = ['\\CAN';s]
	show_ascii '\EM' s = ['\\EM';s]
	show_ascii '\SUB' s = ['\\SUB';s]
	show_ascii '\ESC' s = ['\\ESC';s]
	show_ascii '\FS' s = ['\\FS';s]
	show_ascii '\GS' s = ['\\GS';s]
	show_ascii '\RS' s = ['\\RS';s]
	show_ascii '\US' s = ['\\US';s]

show_bs_char bs_char s = '\\' : bs_char : s

ProtectEsc p f
	:== \s -> f (cont s)
	where
		cont s@(c:_) | p c = '\\' : '&' : s
		cont s = s

infix 4 +<

(+<) :: !Int -> !Int -> Bool;
(+<) a b
	= `code inline {
		ltU
	}

infix 7 +>>
(+>>) :: !Int -> !Int -> Int;
(+>>) a b
	= `code inline {
		shiftrU
	}

n_leading_zeros_nibble n :== ((`0x000055af+>>(n+n)) bitand 3);

n_leading_zeros n :== IF_INT_64_OR_32 (n_leading_zeros_64 n) (n_leading_zeros_32 n);

n_leading_zeros_64 :: !Int -> Int
n_leading_zeros_64 n // n<>0
	| n+<0x100000000
		| n+<0x10000
			| n+<0x100
				| n+<0x10
					= 60 + n_leading_zeros_nibble n;
					= 56 + n_leading_zeros_nibble (n+>>4);
				| n+<0x1000
					= 52 + n_leading_zeros_nibble (n+>>8);
					= 48 + n_leading_zeros_nibble (n+>>12);
			| n+<0x1000000
				| n+<0x100000
					= 44 + n_leading_zeros_nibble (n+>>16);
					= 40 + n_leading_zeros_nibble (n+>>20);
				| n+<0x10000000
					= 36 + n_leading_zeros_nibble (n+>>24);
					= 32 + n_leading_zeros_nibble (n+>>28);
		| n+<0x1000000000000
			| n+<0x10000000000
				| n+<0x1000000000
					= 28 + n_leading_zeros_nibble (n+>>32);
					= 24 + n_leading_zeros_nibble (n+>>36);
				| n+<0x100000000000
					= 20 + n_leading_zeros_nibble (n+>>40);
					= 16 + n_leading_zeros_nibble (n+>>44);
			| n+<0x100000000000000
				| n+<0x10000000000000
					= 12 + n_leading_zeros_nibble (n+>>48);
					= 8 + n_leading_zeros_nibble (n+>>52);
				| n+<0x1000000000000000
					= 4 + n_leading_zeros_nibble (n+>>56);
					= n_leading_zeros_nibble (n+>>60);

n_leading_zeros_32 :: !Int -> Int
n_leading_zeros_32 n // n<>0
	| n+<0x10000
		| n+<0x100
			| n+<0x10
				= 28 + n_leading_zeros_nibble n;
				= 24 + n_leading_zeros_nibble (n+>>4);
			| n+<0x1000
				= 20 + n_leading_zeros_nibble (n+>>8);
				= 16 + n_leading_zeros_nibble (n+>>12);
		| n+<0x1000000
			| n+<0x100000
				= 12 + n_leading_zeros_nibble (n+>>16);
				= 8 + n_leading_zeros_nibble (n+>>20);
			| n+<0x10000000
				= 4 + n_leading_zeros_nibble (n+>>24);
				= n_leading_zeros_nibble (n+>>28);

int_array_to_unsigned_integer :: !*{#Int} -> *Integer;
int_array_to_unsigned_integer a
	#! s = size a;
	# n=a~[s-1];
	| n/=0
		| s/=1 || n<0
			= {integer_s=0, integer_a=a};
			= {integer_s=n, integer_a={}};
		= remove_zeros_and_resize_p (s-1) a;
  where
	remove_zeros_and_resize_p :: !Int -> !*{#Int} -> *Integer;
	remove_zeros_and_resize_p i a 
		| i==0
			= {integer_s=0,integer_a={}};
		#! n=a~[i-1];
		| n==0
			= remove_zeros_and_resize_p (i-1) a;
		| i/=1 || n<0
			= {integer_s=0, integer_a=resize_int_array i a};
			= {integer_s=n, integer_a={}};

resize_int_array :: !Int -> !*{#Int} -> *{#Int};
resize_int_array s a
	| size a>=s
		= unsafe_resize_int_array s a;

unsafe_resize_int_array :: !Int -> !*{#Int} -> *{#Int};
unsafe_resize_int_array s a
	= `code {
		fill1_r _ARAY_ 0 1 0 01
	}