definition module PreludeList where

from PreludeTypes import
	::String,::Integer,::Maybe,class Eq,class Ord,class Num,class Show
from StdMisc import qualified abort
from StdBool import not

from StdList import
	map,++,last,init,foldl,foldr,iterate,repeat,take,drop,reverse,and,or,all,any,unzip

//filter :: (a -> .Bool) -> !.[a] -> .[a]
filter f l :== filter l
  where
	filter (a:x) | f a	= a:filter x
						= filter x
	filter []			= []

concat :: ![[.a]] -> [.a]
concatMap :: (.a -> [.b]) -> ![.a] -> [.b]
head :: ![.a] -> .a
tail :: !u:[.a] -> u:[.a]
length :: ![a] -> Int
(!!) infixl 9 :: ![.a] -> !Int -> .a
foldl1 op (r:l) :== foldl op r l

//scanl :: (a -> b -> a) -> a -> [b] -> [a]
scanl f q xs :== scanl q xs
	where
		scanl q xs
			= q : (case xs of
					[] -> []
					x:xs -> scanl (f q x) xs)
//scanl1 :: (a -> a -> a) -> [a] -> [a]
scanl1 f l :==
		case l of
			(x:xs) -> scanl1 x xs
			[] -> []
	where
		scanl1 q xs
			= q : (case xs of
					[] -> []
					x:xs -> scanl1 (f q x) xs)

//foldr1 :: (a -> a -> a) -> [a] -> a
foldr1 op l :== foldr1 l
	where
		foldr1 [x]	= x
		foldr1 (a:x) = op a (foldr1 x)
		foldr1 [] = StdMisc.abort `"Prelude.foldr1: empty list"

//scanr :: (a -> b -> b) -> b -> [a] -> [b]
scanr f q0 l :== scanr l
	where
		scanr [] = [q0]
		scanr (x:xs) = f x q : qs
			where (qs@(q:_)) = scanr xs

//scanr1 :: (a -> a -> a) -> [a] -> [a]
scanr1 f xs :== scanr1 xs
where
	scanr1 [] = []
	scanr1 [x] = [x]
	scanr1 (x:xs) = f x q : qs
		where qs@(q:_) = scanr1 xs

replicate :: !Int -> a -> [a]
splitAt :: !Int -> u:[.a] -> ([.a],u:[.a])

//takeWhile :: (a -> Bool) -> [a] -> [a]
takeWhile p l :== takeWhile l
where
	takeWhile [] = []
	takeWhile (x:xs)
		| p x
			= x : takeWhile xs
			= []

//dropWhile :: (a -> Bool) -> [a] -> [a]
dropWhile p l :== dropWhile l
where
	dropWhile [] = []
	dropWhile xs@(x:xs`)
		| p x
			= dropWhile xs`
			= xs

//span :: (a -> Bool) -> [a] -> ([a],[a])
span p l :== span l
	where
		span [] = ([],[])
		span xs@(x:xs')
			| p x = (x:ys,zs)
			| otherwise = ([],xs)
		where (ys,zs) = span xs'

//break :: (a -> Bool) -> [a] -> ([a],[a])
break p l :== break l
	where
		break [] = ([],[])
		break xs@(x:xs')
			| not (p x)
				= (x:ys,zs)
				= ([],xs)
		where (ys,zs) = break xs'

lines :: !String -> [String]
unlines :: ([String] -> String)
words :: !String -> [String]
unwords :: ![String] -> String

elem infix 4 :: (Eq a) => a -> !.[a] -> Bool
									special a=Int
											a=Char
											a=Double
											a=Float

notElem infix 4 :: (Eq a) => a -> !.[a] -> Bool
									special a=Int
											a=Char
											a=Double
											a=Float

lookup :: (Eq a) => a -> ![(a,b)] -> Maybe b
									special a=Int
											a=Char
											a=Double
											a=Float

sum :: (Num a) => !.[a] -> a
									special	a=Int
											a=Double
											a=Float
											a=Integer

product :: (Num a) => !.[a] -> a
									special	a=Int
											a=Double
											a=Float
											a=Integer

maximum :: (Ord a) => ![a] -> a
									special	a=Int
											a=Double
											a=Float
											a=Integer
minimum :: (Ord a) => ![a] -> a
									special	a=Int
											a=Double
											a=Float
											a=Integer

zip :: ![.a] -> [.b] -> [(.a,.b)]
zip3 :: ![.a] -> [.b] -> [.c] -> [(.a,.b,.c)]

zipWith z l1 l2 :== zipWith l1 l2
	where
		zipWith (a:as) (b:bs) = z a b : zipWith as bs
		zipWith _ _ = []

zipWith3 z l1 l2 l3 :== zipWith3 l1 l2 l3
	where
		zipWith3 (a:as) (b:bs) (c:cs) = z a b c : zipWith3 as bs cs
		zipWith3 _ _ _ = []

unzip3 :: ![(.a,.b,.c)] -> ([.a],[.b],[.c])
