implementation module PreludeList where

from StdMisc import abort
from StdChar import isSpace

concat :: ![[.a]] -> [.a]
concat xss = foldr (++) [] xss

concatMap :: (.a -> [.b]) -> ![.a] -> [.b]
concatMap f (x:xs) = f x ++ concatMap f xs
concatMap f [] = []

head :: ![.a] -> .a
head (a:x)	= a
head []		= abort `"head of []"

tail :: !u:[.a] -> u:[.a]
tail (a:x)	= x
tail []		= abort `"tail of []"

length :: ![a] -> Int
length xs = acclen 0 xs
	where
		acclen n (x:xs) = acclen (n+1) xs
		acclen n []     = n

infixl 9 !!
(!!) :: ![.a] -> !Int -> .a
(!!) [] i
	| i>=0
		= index_too_large_error
		= negative_index_error
(!!) list i
	| i>=0
		= index list i
		= negative_index_error
	where
		index ::![.a] -> !Int -> .a
		index (hd:tl) 0
			= hd
		index (hd:tl) n
			= index tl (n - 1)
		index [] _
			= index_too_large_error

negative_index_error => abort `"Prelude.!!: negative index"
index_too_large_error => abort `"Prelude.!!: index too large"

replicate :: !Int -> a -> [a]
replicate n x
	| n<=0
		= []
		= replicate n x
	where
		replicate :: !Int -> !a -> [a]
		replicate n x
			| n<=1
				= [x]
				= x:replicate (n-1) x

splitAt :: !Int -> u:[.a] -> ([.a],u:[.a])
splitAt n xs
	| n<=0
		=	([],xs)
splitAt _ []
	=	([],[])
splitAt n (x:xs)
	= (x:xs',xs'') 
	where
		(xs',xs'') = splitAt (n-1) xs

-- lines breaks a string up into a list of strings at newline characters.
-- The resulting strings do not contain newlines. Similary, words
-- breaks a string up into a list of words, which were delimited by
-- white space. unlines and unwords are the inverse operations.
-- unlines joins lines with terminating newlines, and unwords joins
-- words with separating spaces.
lines :: !String -> [String]
lines "" = []
lines s = let (l, s') = break (== '\n') s
		in l : case s' of
			[] -> []
			(_:s'') -> lines s''

unlines :: ([String] -> String)
unlines
	= unlines 
	where
		unlines (x:xs) = x ++ ('\n' : unlines xs)
		unlines [] = []

words :: !String -> [String]
words s = case dropWhile isSpace s of
			"" -> []
			s1 -> w : words s2
			where
				(w, s2) = break isSpace s1

unwords :: ![String] -> String
unwords [] = ""
unwords ws = foldr1 (\w s -> w ++ ' ':s) ws

infix 4 `elem`
elem :: (Eq a) => a -> !.[a] -> Bool
elem x (hd:tl) = hd==x || elem x tl
elem x []	= False

infix 4 `notElem`
notElem :: (Eq a) => a -> !.[a] -> Bool
notElem x (e : l) = e/=x && notElem x l
notElem x [] = True

lookup :: (Eq a) => a -> ![(a,b)] -> Maybe b
lookup key [] = Nothing
lookup key ((x,y):xys)
	| key == x = Just y
	| otherwise = lookup key xys

sum :: (Num a) => !.[a] -> a
sum xs = accsum xs 0
where
	accsum (x:xs) n = accsum xs (n + x)
	accsum []     n = n

product :: (Num a) => !.[a] -> a
product xs = accproduct xs 1
where
	accproduct (x:xs) n = accproduct xs (n * x)
	accproduct []     n = n

maximum :: (Ord a) => ![a] -> a
maximum [] = abort `"Prelude.maximum: empty list"
maximum xs = foldl1 max xs

minimum :: (Ord a) => ![a] -> a
minimum [] = abort `"Prelude.minimum: empty list"
minimum xs = foldl1 min xs

zip :: ![.a] -> [.b] -> [(.a,.b)]
zip (a:as) (b:bs)	= (a,b):zip as bs
zip as bs			= []

zip3 :: ![.a] -> [.b] -> [.c] -> [(.a,.b,.c)]
zip3 (a:as) (b:bs) (c:cs) = (a,b,c):zip3 as bs cs
zip3 as bs cs = []

unzip3 :: ![(.a,.b,.c)] -> ([.a],[.b],[.c])
unzip3 [] = ([], [], [])
unzip3 ((x,y,z) : xyzs) = (x:xs, y:ys, z:zs)
where
	(xs,ys,zs) = unzip3 xyzs
