implementation module PreludeRatio where

import Prelude;

from StdMisc import abort;

instance (Integral a) => Eq (Ratio a) where
	(==) :: (Integral a) => !(Ratio a) -> !(Ratio a) -> Bool
	(n0 :% d0) == (n1 :% d1) = n0==n1 && d0==d1
	(/=) :: (Integral a) => !(Ratio a) -> !(Ratio a) -> Bool
	(n0 :% d0) /= (n1 :% d1) = n0/=n1 || d0/=d1

instance (Integral a) => Ord (Ratio a) where
	(<=) :: (Integral a) => !(Ratio a) -> !(Ratio a) -> Bool;
	(x:%y) <= (x':%y') = x * y' <= x' * y
	(<) :: (Integral a) => !(Ratio a) -> !(Ratio a) -> Bool;
	(x:%y) < (x':%y') = x * y' < x' * y

instance (Integral a) => Num (Ratio a) where
	(x:%y) + (x':%y') = reduce (x*y' + x'*y) (y*y')
	(x:%y) * (x':%y') = reduce (x * x') (y * y')
	negate (x:%y) = (-x) :% y
	abs (x:%y) = abs x :% y
	signum (x:%y) = signum x :% 1
	fromInteger x = fromInteger x :% 1

reduce _ 0 = abort `"Ratio.% : zero denominator"
reduce x y = (x `quot` d) :% (y `quot` d)
	where
		d = gcd x y

instance (Integral a) => Real (Ratio a) where
	toRational (x:%y) = toInteger x :% toInteger y

instance (Integral a) => Fractional (Ratio a) where
	(x:%y) / (x':%y') = (x*y') % (y*x')
	recip (x:%y) = y % x
	fromRational (x:%y) = fromInteger x :% fromInteger y

x % y = reduce (x * signum y) (abs y)

instance (Integral a) => RealFrac (Ratio a) where
	properFraction (x:%y) = (fromIntegral q, r:%y)
							where (q,r) = quotRem x y

instance (Integral a) => Enum (Ratio a) where
	succ x = x+1
	pred x = x-1
	toEnum :: (Integral a) => (Int -> Ratio a)
	toEnum = fromIntegral
	fromEnum x = fromInteger (truncate x)	-- May overflow
	enumFrom x = numericEnumFrom x
	enumFromThen n m = numericEnumFromThen n m
	enumFromTo n m = numericEnumFromTo n m
	enumFromThenTo n n' m = numericEnumFromThenTo n n' m

numericEnumFrom :: (Fractional a) => a -> [a]
numericEnumFrom x = x : numericEnumFrom (x+1)

numericEnumFromThen :: (Fractional a) => a -> a -> [a]
numericEnumFromThen n m = numericEnumFromThen n (m-n) where
	numericEnumFromThen x d
		= x : numericEnumFromThen (x+d) d

numericEnumFromTo :: (Fractional a, Ord a) => a -> a -> [a]
numericEnumFromTo n m = numericEnumFromTo n (m+1/2)
	where
		numericEnumFromTo x u
			| x<=u
				= x : numericEnumFromTo (x+1) u
				= []

numericEnumFromThenTo :: (Fractional a, Ord a) => a -> a -> a -> [a]
numericEnumFromThenTo n n' m
	# d = n'-n
	| n'>=n
		= numericEnumFromThenToUp n d (m+d/2)
		= numericEnumFromThenToDown n d (m+d/2)

numericEnumFromThenToUp n d u
	| n<=u
		#! n`=n+d
		= n : numericEnumFromThenToUp n` d u
		= []

numericEnumFromThenToDown n d l
	| n>=l
		#! n`=n+d
		= n : numericEnumFromThenToDown n` d l
		= []

instance (Integral a) => Show (Ratio a) where
	showsPrec p (x :% y)
		= showParen (p > 7) (\s -> showsPrec 8 x [' % ';showsPrec 8 y s])

instance (Read a, Integral a) => Read (Ratio a) where
	readsPrec p
		= readParen (p > `7)
			(\r -> [(x%y,u) |	(x,s) <- readsPrec `8 r,
								("%",t) <- lex s,
								(y,u) <- readsPrec `8 t ])
