implementation module PreludeRead where

import _SystemArray
import StdMisc
from StdChar import isSpace,isDigit,isUpper,isAlpha,isAlphanum,isOctDigit,isHexDigit,digitToInt
from StdInt import bitor
from StdReal import class toReal(..),instance toReal {#Char}
from StdFloat import qualified class toFloat(..),instance toFloat Double

isAlphaNum c :== isAlphanum c

DigitToInt c :== ord c - 48

from PreludeStringToInteger import string_to_integer
from PreludeInternal import lexLitChar,read_rest_int,read_rest_oct_int,read_rest_hex_int

read :: (Read a) => String -> a
read s
	= case [x | (x,t) <- reads s, ("","") <- lex t] of
		[x] -> x
		[] -> abort `"Prelude.read: no parse"
		_ -> abort `"Prelude.read: ambiguous parse"

--lex :: ReadS String
lex :: !String -> [(String,String)]
lex "" = [("","")]
lex (c:s) | isSpace c = lex (dropWhile isSpace s)
lex ('\'':s) = [('\'':ch++"'", t) | (ch,'\'':t) <-lexLitChar s,ch /="'"]
lex ('"':s) = [('"':str, t) | (str,t) <-lexString s]
  where
	lexString ('"':s) = [("\"",s)]
	lexString s = [(ch++str, u)
				  | (ch,t) <-lexStrItem s,
				  (str,u) <-lexString t ]
	
	lexStrItem ('\\':'&':s) = [("\\&",s)]
	lexStrItem ('\\':c:s) | isSpace c = [("\\&",t) | '\\':t <- [dropWhile isSpace s]]
	lexStrItem s = lexLitChar s
lex (c:s)
	| isSingle c = [([c],s)]
	| isSym c = [(c:sym,t) | (sym,t) <-[span isSym s]]
	| isAlpha c = [(c:nam,t) | (nam,t) <-[span isIdChar s]]
	| isDigit c = [(c:ds++fe,t) | (ds,s) <-[span isDigit s], (fe,t) <-lexFracExp s ]
	| otherwise = [] --bad character
  where
	isSingle c = c `elem` ",;()[]{}_"
	isSym c = c `elem` "!@#$%&*+./<=>?\\^|:-"
	isIdChar c = isAlphaNum c || c `elem` "_'"

	lexFracExp ('.':c:cs)
		| isDigit c
			= [('.':ds++e,u) | (ds,t) <-lexDigits (c:cs),(e,u) <-lexExp t]
	lexFracExp s = lexExp s

	lexExp (e:s)
		| e `elem` "eE"
			= [(e:c:ds,u) | (c:t) <-[s], c=='+' || c=='-',
							(ds,u) <-lexDigits t] ++
							[(e:ds,t) | (ds,t) <-lexDigits s]
	lexExp s = [("",s)]

lexDigits :: ReadS String
lexDigits = nonnull isDigit

nonnull p s :== [(cs,t) | (cs@(_:_),t) <- [span p s]]

readParen :: !Bool -> ReadS a -> ReadS a
readParen b g = if b then mandatory else optional
where
	optional r = g r ++ mandatory r

	mandatory r = case skip_spaces` r of
					'(':s
						-> read_close` (optional s)
					_
						-> []

{-
readParen' :: !Bool -> (Bool -> ReadS a) -> String -> [(a,String)]
readParen' b g r = if b (mandatory r) (optional r)
where
	mandatory r  = case skip_spaces` r of
					'(':s -> read_close` (optional s)
					s     -> g False s

	optional r  = case skip_spaces` r of
					'(':s -> read_close` (optional s)
					s     -> g True s

readParen'' :: !Bool -> ReadS a -> String -> [(a,String)]
readParen'' b g r = if b then (mandatory r) else (optional r)
where
	optional r
		| no_open_p_after_spaces' r
			= g r
			= g r ++ mandatory r

	mandatory r = case skip_spaces` r of
					'(':s
						-> read_close` (optional s)
					_
						-> []

no_open_p_after_spaces' :: !String -> Bool
no_open_p_after_spaces' s
	= case skip_spaces` s of
		'(':_
			-> False
		_
			-> True
-}

skip_spaces`` :: ![(a,String)] -> [(a,String)]
skip_spaces`` r
	= [(e,skip_spaces` s) ;; (e,s) <-r]

read_close` :: ![(a,String)] -> [(a,String)]
read_close` ((x,t):xts)
	= case skip_spaces` t of
		')':u -> (x,u) : read_close` xts
		_     -> read_close` xts
read_close` []
	= []

not_begins_with_id_char` :: !String -> Bool
not_begins_with_id_char` (c:_)
	= not (isAlphanum c || c=='_' || c=='`')
not_begins_with_id_char` []
	= True

instance Read Int
	where
		readsPrec :: Int -> ![Char] -> [(Int,[Char])]
		readsPrec p s
			= case skip_spaces` s of
				'(':s
					-> read_close` (readsPrec p s)
				'-':s
					-> read_neg_int s
				s
					-> read_int s

--		readsOpenParensPrec :: Int -> String -> [[(Int,String)]]
		readsOpenParensPrec p s
			= readsOpenParensPrecInt s
		where
			readsOpenParensPrecInt s
				= case skip_spaces` s of
					'(':s
						-> read_close_p1 (readsOpenParensPrecInt s)
					'-':s
						-> [read_neg_int s]
					s
						-> [read_int s]

instance Read Integer
	where
		readsPrec :: Int -> ![Char] -> [(Integer,[Char])]
		readsPrec p s
			= case skip_spaces` s of
				'(':s
					-> read_close` (readsPrec p s)
				'-':s
					-> read_neg_integer s
				s
					-> read_integer s

--		readsOpenParensPrec :: Int -> String -> [[(Int,String)]]
		readsOpenParensPrec p s
			= readsOpenParensPrecInt s
		where
			readsOpenParensPrecInt s
				= case skip_spaces` s of
					'(':s
						-> read_close_p1 (readsOpenParensPrecInt s)
					'-':s
						-> [read_neg_integer s]
					s
						-> [read_integer s]

instance Read Double
	where
		readsPrec :: Int -> ![Char] -> [(Double,[Char])]
		readsPrec p s
			= case skip_spaces` s of
				'(':s
					-> read_close` (readsPrec p s)
				s
					-> read_double s

		readsOpenParensPrec p s
			= readsOpenParensPrecInt s
		where
			readsOpenParensPrecInt s
				= case skip_spaces` s of
					'(':s
						-> read_close_p1 (readsOpenParensPrecInt s)
					s
						-> [read_double s]

instance Read Float
	where
		readsPrec :: Int -> ![Char] -> [(Float,[Char])]
		readsPrec p s
			= case skip_spaces` s of
				'(':s
					-> read_close` (readsPrec p s)
				s
					-> read_float s

		readsOpenParensPrec p s
			= readsOpenParensPrecInt s
		where
			readsOpenParensPrecInt s
				= case skip_spaces` s of
					'(':s
						-> read_close_p1 (readsOpenParensPrecInt s)
					s
						-> [read_float s]

read_close_p1 l@([(i,s)]:_)
	= case skip_spaces` s of
		')':s
			-> [(i,s)]:l
		_
			-> []:l
read_close_p1 l
	= []:l

read_neg_int (c:s)
	| isDigit c
		# (i,s) = read_rest_int s (DigitToInt c)
		= [(-i,s)]
read_neg_int _
	= []

read_int (c:s)
	| isDigit c
		# (i,s) = read_rest_int s (DigitToInt c)
		= [(i,s)]
read_int _
	= []

read_neg_integer (c:s)
	| isDigit c
		# (i,s) = read_rest_integer s [c,'-'] 2
		= [(i,s)]
read_neg_integer _
	= []

read_integer (c:s)
	| isDigit c
		# (i,s) = read_rest_integer s [c] 1
		= [(i,s)]
read_integer _
	= []

read_rest_integer :: ![Char] -> ![Char] -> !Int -> (!Integer,![Char])
read_rest_integer (c:s) reversed_chars n_reversed_chars
	| isDigit c
		= read_rest_integer s (c:reversed_chars) (n_reversed_chars+1)
read_rest_integer s reversed_chars n_reversed_chars
	= (string_to_integer (reversed_chars_to_string reversed_chars n_reversed_chars),s)

reversed_chars_to_string reversed_chars n_reversed_chars
	= store_reversed_chars_in_string (n_reversed_chars-1) reversed_chars (createArray n_reversed_chars ' ')
where
	store_reversed_chars_in_string :: !Int -> ![Char] -> !*{#Char} -> *{#Char}
	store_reversed_chars_in_string i (c:cs) s
		= store_reversed_chars_in_string (i-1) cs {s & [i]=c}
	store_reversed_chars_in_string `-1 [] s
		= s;

read_double :: ![Char] -> [(Double,[Char])];
read_double ('-':c:s)
	| isDigit c
		# (reversed_chars,n_reversed_chars,s) = read_real s [c,'-'] 2;
		| n_reversed_chars==0
			= []
			#! r=toReal (reversed_chars_to_string reversed_chars n_reversed_chars)
			= [(r,s)]
read_double (c:s)
	| isDigit c
		# (reversed_chars,n_reversed_chars,s) = read_real s [c] 1;
		| n_reversed_chars==0
			= []
			#! r=toReal (reversed_chars_to_string reversed_chars n_reversed_chars)
			= [(r,s)]
read_double ['NaN';s]
	= [(0.0 / 0.0,s)];
read_double ['Infinity';s]
	= [(1.0 / 0.0,s)];
read_double ['-NaN';s]
	= [(- (0.0 / 0.0),s)];
read_double ['-Infinity';s]
	= [(-1.0 / 0.0,s)];
read_double _
	= []

read_float :: ![Char] -> [(Float,[Char])];
read_float ('-':c:s)
	| isDigit c
		# (reversed_chars,n_reversed_chars,s) = read_real s [c,'-'] 2;
		| n_reversed_chars==0
			= []
			#! r=StdFloat.toFloat (toReal (reversed_chars_to_string reversed_chars n_reversed_chars))
			= [(r,s)]
read_float (c:s)
	| isDigit c
		# (reversed_chars,n_reversed_chars,s) = read_real s [c] 1;
		| n_reversed_chars==0
			= []
			#! r=StdFloat.toFloat (toReal (reversed_chars_to_string reversed_chars n_reversed_chars))
			= [(r,s)]
read_float ['NaN';s]
	= [(StdFloat.toFloat (0.0 / 0.0),s)];
read_float ['Infinity';s]
	= [(StdFloat.toFloat (1.0 / 0.0),s)];
read_float ['-NaN';s]
	= [(StdFloat.toFloat (- (0.0 / 0.0)),s)];
read_float ['-Infinity';s]
	= [(StdFloat.toFloat (-1.0 / 0.0),s)];
read_float _
	= []

read_real :: ![Char] -> ![Char] -> !Int -> (![Char],!Int,![Char]);
read_real (c:s) reversed_chars n_reversed_chars
	| isDigit c
		= read_real s (c:reversed_chars) (n_reversed_chars+1)
read_real (c1@'.':cs@(c2:s)) reversed_chars n_reversed_chars
	| isDigit c2
		= read_real_frac s (c2:c1:reversed_chars) (n_reversed_chars+2)
		= ([],0,[]) // error
read_real cs reversed_chars n_reversed_chars
	= read_real_exp cs reversed_chars n_reversed_chars

read_real_frac :: ![Char] -> ![Char] -> !Int -> (![Char],!Int,![Char]);
read_real_frac (c:s) reversed_chars n_reversed_chars
	| isDigit c
		= read_real_frac s (c:reversed_chars) (n_reversed_chars+1)
read_real_frac cs reversed_chars n_reversed_chars
	= read_real_exp cs reversed_chars n_reversed_chars

read_real_exp :: ![Char] -> ![Char] -> !Int -> (![Char],!Int,![Char]);
read_real_exp (c1:c2:s) reversed_chars n_reversed_chars
	| c1=='e' || c1=='E'
		| isDigit c2
			= read_real_exp_digits s (c2:c1:reversed_chars) (n_reversed_chars+2)
		| c2=='-' || c2=='+'
			= case s of
				c3:s | isDigit c3
					-> read_real_exp_digits s (c3:c2:c1:reversed_chars) (n_reversed_chars+3)
				_
					-> ([],0,[]) // error
			= ([],0,[]) // error
read_real_exp cs reversed_chars n_reversed_chars
	= (reversed_chars,n_reversed_chars,cs)

read_real_exp_digits :: ![Char] -> ![Char] -> !Int -> (![Char],!Int,![Char]);
read_real_exp_digits (c:s) reversed_chars n_reversed_chars
	| isDigit c
		= read_real_exp_digits s (c:reversed_chars) (n_reversed_chars+1)
read_real_exp_digits cs reversed_chars n_reversed_chars
	= (reversed_chars,n_reversed_chars,cs)

/*
readFloat :: (RealFrac a) => ReadS a
readFloat r
	= [(fromRational ((n%1)*10^^(k-d)),t) | (n,d,s) <- readFix r,
		(k,t) <- readExp s] ++
	  [ (0/0, t) | ("NaN",t) <- lex r] ++
	  [ (1/0, t) | ("Infinity",t) <- lex r]
where
	readFix r
		= [(read (ds++ds'), length ds', t)
		  | (ds,d) <- lexDigits r,
			(ds',t) <- lexFrac d ]

	lexFrac ('.':ds) = lexDigits ds
	lexFrac s = [("",s)]

	readExp (e:s) | e `elem` "eE" = readExp' s
	readExp s = [(0,s)]

	readExp' ('-':s) = [(-k,t) | (k,t) <- readDec s]
	readExp' ('+':s) = readDec s
	readExp' s = readDec s
*/

instance Read Char
	where
		readsPrec :: Int -> ![Char] -> [(Char,[Char])]
		readsPrec p s
			= case skip_spaces` s of
				'\'':'\\':s
					-> case readEsc s of
						[(c,'\\':s)]
							-> [(c,s)]
						_
							-> []
				'\'':c:'\'':s
					-> [(c,s)]
				'(':s
					-> read_close` (readsPrec p s)
				_
					-> []

		readsOpenParensPrec p s
			= readsOpenParensPrecChar s
		where
			readsOpenParensPrecChar s
				= case skip_spaces` s of
					'\'':'\\':s
						-> case readEsc s of
							[(c,'\\':s)]
								-> [[(c,s)]]
							_
								-> [[]]
					'\'':c:'\'':s
						-> [[(c,s)]]
					'(':s
						-> read_close_p1 (readsOpenParensPrecChar s)
					_
						-> [[]]

		readList :: ![Char] -> [([Char],[Char])]
		readList s
			= case skip_spaces` s of
				'"':s
					# (ok,r,s) = read_string s
					| ok
						-> [(r,s)]
						-> []
				'(':s
					-> read_close` (readList s)
				_
					-> []
		where
			read_string :: ![Char] -> (!Bool,![Char],![Char])
			read_string ('"':s) = (True,"",s)
			read_string ('\\':'&':s) = read_string s
			read_string ('\\':s)
				= case readEsc s of
					[(c,t)]
						# (ok,r,s) = read_string t
						| ok
							-> (True,c:r,s)
							-> (False,[],[])
					_
						-> (False,[],[])
			read_string (c:t)
				#! (ok,r,s) = read_string t
				| ok
					= (True,c:r,s)
					= (False,[],[])
			read_string []
				= (False,[],[])

readEsc :: ReadS Char
readEsc ('a':s) = [('\a',s)]
readEsc ('b':s) = [('\b',s)]
readEsc ('f':s) = [('\f',s)]
readEsc ('n':s) = [('\n',s)]
readEsc ('r':s) = [('\r',s)]
readEsc ('t':s) = [('\t',s)]
readEsc ('v':s) = [('\v',s)]
readEsc ('\\':s) = [('\\',s)]
readEsc ('"':s) = [('"',s)]
readEsc ('\'':s) = [('\'',s)]
readEsc ('^':c:s) | c >= '@' && c <= '_'
	= [(chr (ord c - ord '@'), s)]
readEsc (d:s)
	| isDigit d
		# (n,t) = read_rest_int s (digitToInt d)
		= [(chr n, t)]
readEsc ('o':d:s)
	| isOctDigit d
		# (n,t) = read_rest_oct_int s (digitToInt d)
		= [(chr n, t)]
		= []
readEsc ('x':d:s)
	| isDigit d
		# (n,t) = read_rest_hex_int s (digitToInt d)
		= [(chr n, t)]
	# du=(ord d) bitor 0x20
	| du>=97 && du<=102
		# (n,t) = read_rest_hex_int s (du-87)
		= [(chr n, t)]
readEsc s@(c:t)
	| isUpper c
		= case (c,t) of
			('A',['CK';t]) -> lt2 '\ACK' t
			('B',['S';t]) -> lt2 '\BS' t
			('B',['EL';t]) -> lt2 '\BEL' t
			('C',['AN';t]) -> lt2 '\CAN' t
			('C',['R';t]) -> lt2 '\CR' t
			('D',['C1';t]) -> lt2 '\DC1' t
			('D',['C2';t]) -> lt2 '\DC2' t
			('D',['C3';t]) -> lt2 '\DC3' t
			('D',['C4';t]) -> lt2 '\DC4' t
			('D',['EL';t]) -> lt2 '\DEL' t
			('D',['LE';t]) -> lt2 '\DLE' t
			('E',['M';t]) -> lt2 '\EM' t
			('E',['NQ';t]) -> lt2 '\ENQ' t
			('E',['OT';t]) -> lt2 '\EOT' t
			('E',['SC';t]) -> lt2 '\ESC' t
			('E',['TB';t]) -> lt2 '\ETB' t
			('E',['TX';t]) -> lt2 '\ETX' t
			('F',['F';t]) -> lt2 '\FF' t
			('F',['S';t]) -> lt2 '\FS' t
			('G',['S';t]) -> lt2 '\GS' t
			('H',['T';t]) -> lt2 '\HT' t
			('L',['F';t]) -> lt2 '\LF' t
			('N',['AK';t]) -> lt2 '\NAK' t
			('N',['UL';t]) -> lt2 '\NUL' t
			('R',['S';t]) -> lt2 '\RS' t
			('S',['I';t]) -> lt2 '\SI' t
			('S',['OH';t]) -> lt2 '\SOH' t
			('S',['O';t]) -> lt2 '\SO' t
			('S',['P';t]) -> lt2 ' ' t
			('S',['TX';t]) -> lt2 '\STX' t
			('S',['UB';t]) -> lt2 '\SUB' t
			('S',['YN';t]) -> lt2 '\SYN' t
			('U',['S';t]) -> lt2 '\US' t
			('V',['T';t]) -> lt2 '\VT' t
			_ -> []
		where
			lt2 c l = [(c,l)]
readEsc _ = []

ord :: !Char -> Int
ord c
	= `code inline {
			CtoI
	}

chr :: !Int -> Char
chr i
	= `code inline {
			ItoC
	}

instance Read ()
	where
		readsPrec :: Int -> ![Char] -> [((),[Char])]
		readsPrec p s
			= case skip_spaces` s of
				'(':s
					->	read_unit_close s
				_
					-> []
		where
			read_unit_close s
				= case skip_spaces` s of
					')':s
						-> [((),s)]
					'(':s
						-> read_close` (read_unit_close s)
					_
						-> []

		readsOpenParensPrec p s
			= case skip_spaces` s of
				'(':s
					->	read_unit_close s
				_
					-> [[]]
		where
			read_unit_close s
				= case skip_spaces` s of
					')':s
						-> [[((),s)]]
					'(':s
						-> read_close_p1 (read_unit_close s)
					_
						-> [[]]

instance Read Bool where
	readsPrec :: Int -> ![Char] -> [(Bool,[Char])]
	readsPrec p s
		= case skip_spaces` s of
			'(':s
				-> read_close` (readsPrec 0 s)
			s
				-> read_Bool_constructors s

	readsOpenParensPrec p s
		= case skip_spaces` s of
			'(':s
				# rl@(r:_) = readsOpenParensPrec 0 s
				-> read_close` r : rl
			s
				-> [read_Bool_constructors s]

read_Bool_constructors ['False';s]
	| not_begins_with_id_char` s
		= [(False,s)];
read_Bool_constructors ['True';s]
	| not_begins_with_id_char` s
		= [(True,s)];
read_Bool_constructors _
	= [];

instance Read Ordering where
	readsPrec :: Int -> ![Char] -> [(Ordering,[Char])]
	readsPrec p s
		= case skip_spaces` s of
			'(':s
				-> read_close` (readsPrec 0 s)
			s
				-> read_Ordering_constructors s

	readsOpenParensPrec p s
		= case skip_spaces` s of
			'(':s
				# rl@(r:_) = readsOpenParensPrec 0 s
				-> read_close` r : rl
			s
				-> [read_Ordering_constructors s]

read_Ordering_constructors ['GT';s]
	| not_begins_with_id_char` s
		= [(GT,s)];
read_Ordering_constructors ['EQ';s]
	| not_begins_with_id_char` s
		= [(EQ,s)];
read_Ordering_constructors ['LT';s]
	| not_begins_with_id_char` s
		= [(LT,s)];
read_Ordering_constructors _
	= [];

instance (Read a) => Read (Maybe a) where
	readsPrec :: (Read a) => Int -> ![Char] -> [(Maybe a,[Char])]
	readsPrec p s
		= case skip_spaces` s of
			'(':s
				-> read_close` (readsPrec 0 s)
			s
				-> read_Maybe_constructors s

	readsOpenParensPrec p s
		= case skip_spaces` s of
			'(':s
				# rl@(r1:_) = readsOpenParensPrec 0 s 
				-> read_close` r1 : rl
			s
				-> [read_Maybe_constructors s]

read_Maybe_constructors ['Nothing';s]
	| not_begins_with_id_char` s
		= [(Nothing,s)];
read_Maybe_constructors ['Just';s]
	| not_begins_with_id_char` s
		= [(Just a,t) | (a,t) <- readsPrec 11 s];
read_Maybe_constructors _
	= [];

instance (Read a,Read b) => Read (Either a b) where
	readsPrec :: (Read a,Read b) => Int -> ![Char] -> [(Either a b,[Char])]
	readsPrec p s
		= case skip_spaces` s of
			'(':s
				-> read_close` (readsPrec 0 s)
			s
				-> read_Either_constructors s

	readsOpenParensPrec p s
		= case skip_spaces` s of
			'(':s
				# rl@(r1:_) = readsOpenParensPrec 0 s 
				-> read_close` r1 : rl
			s
				-> [read_Either_constructors s]

read_Either_constructors ['Left';s]
	| not_begins_with_id_char` s
		= [(Left a,t) | (a,t) <- readsPrec 11 s];
read_Either_constructors ['Right';s]
	| not_begins_with_id_char` s
		= [(Right a,t) | (a,t) <- readsPrec 11 s];
read_Either_constructors _
	= [];

{-
readSigned:: (Real a) => ReadS a -> ReadS a
readSigned readPos = readParen False read'
	     where read' r  = read'' r ++
				      [(-x,t) | ("-",s) <- lex r,
									(x,t)   <- read'' s]
			   read'' r = [(n,s) | (str,s) <- lex r,
								   (n,"")  <- readPos str]
-}

instance (Read a) => Read [a]
	where
		readsPrec p = readList

ReadsPrecTuple read_tuple_elements_close read_tuple p s
	:== case skip_spaces` s of
		'(':s
			-> case skip_spaces` s of
				'(':se
					# (e1:e1s) = readsOpenParensPrec 0 s
					-> read_tuple_elements_close e1++(read_close` (read_tuple se e1s))
				_
					-> read_tuple_elements_close (readsPrec 0 s)
		_
			-> []

ReadsOpenParensPrecTuple read_tuple_elements_close read_tuple p s
	:== case skip_spaces` s of
		'(':s -> read_p_tuple s (readsOpenParensPrec 0 s)
		_ -> []
  where
	read_p_tuple s e1r
		# (e1:e1s) = e1r
		= case skip_spaces` s of
			'(':se
				-> (read_tuple_elements_close e1++(read_close` (read_tuple se e1s)))
					: read_p_tuple se e1s
			_
				-> [read_tuple_elements_close e1]

instance (Read e0,Read e1) =>
		Read (e0,e1) where
		readsPrec :: (Read e0,Read e1) =>
					Int -> ![Char] -> [((e0,e1),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements2_close read_tuple2 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements2_close read_tuple2 p s

instance (Read e0,Read e1,Read e2)
		=> Read (e0,e1,e2) where
		readsPrec :: (Read e0,Read e1,Read e2) =>
					Int -> ![Char] -> [((e0,e1,e2),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements3_close read_tuple3 p s
	
		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements3_close read_tuple3 p s

instance (Read e0,Read e1,Read e2,Read e3)
		=> Read (e0,e1,e2,e3) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3) =>
					Int -> ![Char] -> [((e0,e1,e2,e3),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements4_close read_tuple4 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements4_close read_tuple4 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4)
		=> Read (e0,e1,e2,e3,e4) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements5_close read_tuple5 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements5_close read_tuple5 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5)
		=> Read (e0,e1,e2,e3,e4,e5) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements6_close read_tuple6 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements6_close read_tuple6 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6)
		=> Read (e0,e1,e2,e3,e4,e5,e6) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5,e6),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements7_close read_tuple7 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements7_close read_tuple7 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7)
		=> Read (e0,e1,e2,e3,e4,e5,e6,e7) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5,e6,e7),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements8_close read_tuple8 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements8_close read_tuple8 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8)
		=> Read (e0,e1,e2,e3,e4,e5,e6,e7,e8) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5,e6,e7,e8),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements9_close read_tuple9 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements9_close read_tuple9 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9)
		=> Read (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements10_close read_tuple10 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements10_close read_tuple10 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10)
		=> Read (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements11_close read_tuple11 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements11_close read_tuple11 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10,Read e11)
		=> Read (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10,Read e11) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements12_close read_tuple12 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements12_close read_tuple12 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10,Read e11,Read e12)
		=> Read (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10,Read e11,Read e12) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements13_close read_tuple13 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements13_close read_tuple13 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10,Read e11,Read e12,Read e13)
		=> Read (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10,Read e11,Read e12,Read e13) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements14_close read_tuple14 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements14_close read_tuple14 p s

instance (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10,Read e11,Read e12,Read e13,Read e14)
		=> Read (e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13,e14) where
		readsPrec :: (Read e0,Read e1,Read e2,Read e3,Read e4,Read e5,Read e6,Read e7,Read e8,Read e9,Read e10,Read e11,Read e12,Read e13,Read e14) =>
					Int -> ![Char] -> [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13,e14),[Char])]
		readsPrec p s
			= ReadsPrecTuple read_tuple_elements15_close read_tuple15 p s

		readsOpenParensPrec p s
			= ReadsOpenParensPrecTuple read_tuple_elements15_close read_tuple15 p s

read_tuple2 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements2_close e1++ (read_close` (read_tuple2 se e1s))
		_
			-> read_tuple_elements2_close e1

read_tuple3 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements3_close e1++ (read_close` (read_tuple3 se e1s))
		_
			-> read_tuple_elements3_close e1

read_tuple4 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements4_close e1++ (read_close` (read_tuple4 se e1s))
		_
			-> read_tuple_elements4_close e1

read_tuple5 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements5_close e1++ (read_close` (read_tuple5 se e1s))
		_
			-> read_tuple_elements5_close e1

read_tuple6 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements6_close e1++ (read_close` (read_tuple6 se e1s))
		_
			-> read_tuple_elements6_close e1

read_tuple7 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements7_close e1++ (read_close` (read_tuple7 se e1s))
		_
			-> read_tuple_elements7_close e1

read_tuple8 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements8_close e1++ (read_close` (read_tuple8 se e1s))
		_
			-> read_tuple_elements8_close e1

read_tuple9 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements9_close e1++ (read_close` (read_tuple9 se e1s))
		_
			-> read_tuple_elements9_close e1

read_tuple10 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements10_close e1++ (read_close` (read_tuple10 se e1s))
		_
			-> read_tuple_elements10_close e1

read_tuple11 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements11_close e1++ (read_close` (read_tuple11 se e1s))
		_
			-> read_tuple_elements11_close e1

read_tuple12 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements12_close e1++ (read_close` (read_tuple12 se e1s))
		_
			-> read_tuple_elements12_close e1

read_tuple13 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements13_close e1++ (read_close` (read_tuple13 se e1s))
		_
			-> read_tuple_elements13_close e1

read_tuple14 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements14_close e1++ (read_close` (read_tuple14 se e1s))
		_
			-> read_tuple_elements14_close e1

read_tuple15 s r
	# (e1:e1s)=r
	= case skip_spaces` s of
		'(':se
			-> read_tuple_elements15_close e1++ (read_close` (read_tuple15 se e1s))
		_
			-> read_tuple_elements15_close e1

read_tuple_elements2_close r
	= read_close`
	   [((e0,e1), u) |
	 	(e0,',':t) <- skip_spaces`` r,
	 	(e1,u) <- readsPrec 0 t]

read_tuple_elements3_close r
	= read_close`
	   [((e0,e1,e2), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,s) <- readsPrec 0 s]

read_tuple_elements4_close r
	= read_close`
	   [((e0,e1,e2,e3), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,s) <- readsPrec 0 s]

read_tuple_elements5_close r
	= read_close`
	   [((e0,e1,e2,e3,e4), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,s) <- readsPrec 0 s]

read_tuple_elements6_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,s) <- readsPrec 0 s]

read_tuple_elements7_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5,e6), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e6,s) <- readsPrec 0 s]

read_tuple_elements8_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5,e6,e7), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e6,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e7,s) <- readsPrec 0 s]

read_tuple_elements9_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5,e6,e7,e8), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e6,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e7,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e8,s) <- readsPrec 0 s]

read_tuple_elements10_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e6,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e7,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e8,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e9,s) <- readsPrec 0 s]

read_tuple_elements11_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e6,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e7,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e8,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e9,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e10,s) <- readsPrec 0 s]

read_tuple_elements12_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e6,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e7,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e8,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e9,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e10,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e11,s) <- readsPrec 0 s]

read_tuple_elements13_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e6,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e7,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e8,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e9,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e10,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e11,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e12,s) <- readsPrec 0 s]

read_tuple_elements14_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e6,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e7,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e8,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e9,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e10,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e11,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e12,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e13,s) <- readsPrec 0 s]

read_tuple_elements15_close r
	= read_close`
	   [((e0,e1,e2,e3,e4,e5,e6,e7,e8,e9,e10,e11,e12,e13,e14), s) |
	 	(e0,',':s) <- skip_spaces`` r,
	 	(e1,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e2,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e3,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e4,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e5,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e6,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e7,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e8,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e9,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e10,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e11,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e12,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e13,',':s) <- skip_spaces`` (readsPrec 0 s),
	 	(e14,s) <- readsPrec 0 s]
