definition module PreludeTypes where

//import _SystemArray

from StdBool import not,&&,||
from StdList import ++,map

from PreludeInlineFunctions import even

type String = [Char]

:: Integer = {integer_s ::!Int, integer_a ::!.{#Int}};

data Maybe x = Nothing | Just x
data Either a b = Left a | Right b;
data Ratio a = {- (Integral a=>) -} !a :% !a
type Rational = Ratio Integer;

-- :: Ordering = LT | EQ | GT

newtype Ordering = Ordering Int

GT :== Ordering `1
EQ :== Ordering `0
LT :== Ordering `-1

newtype IO a = IO (*World -> *(a,!*World))

data () = ()

class
  Eq a where
	(==) infix 4 :: a -> a -> Bool
	(/=) infix 4 :: a -> a -> Bool
	(==) x y = not (x /= y)
	(/=) x y = not (x == y)

class (Eq a) =>
  Ord a where
	compare :: a -> a -> Ordering
	(<=) infix 4 :: a -> a -> Bool
	(<) infix  4 :: a -> a -> Bool
	(>=) infix 4 :: a -> a -> Bool
	(>) infix  4 :: a -> a -> Bool
	min :: a -> a -> a
	max :: a -> a -> a

	compare a b
		= if a==b
		  then EQ
		  else  if a<=b
				then LT
				else GT

--	(<=) x y = case compare x y of GT -> False; _ -> True
	(<=) x y = case compare x y of Ordering `1 -> False; _ -> True

--	(<) x y  = case compare x y of LT -> True; _ -> False
	(<) x y  = case compare x y of Ordering `-1 -> True; _ -> False

--	(>=) x y = case compare x y of LT -> False; _ -> True 
	(>=) x y = case compare x y of Ordering `-1 -> False; _ -> True 

--	(>) x y  = case compare x y of GT -> True; _ -> False 
	(>) x y  = case compare x y of Ordering `1 -> True; _ -> False 

	min x y = if (x<y) x y
	max x y = if (x>=y) x y

class Enum a where
	succ :: (a -> a)
	pred :: (a -> a)
	toEnum :: Int -> a
	fromEnum :: a -> Int
	enumFrom :: a -> [a] -- [n..]
	enumFromThen :: a -> a -> [a] -- [n,n..]
	enumFromTo :: a -> a -> [a] -- [n..m]
	enumFromThenTo :: a -> a -> a -> [a] -- [n,n..m]

	succ = \x -> toEnum (1+fromEnum x)
	pred = \x -> toEnum (fromEnum x-1)

--	enumFrom x = map toEnum [fromEnum x ..]
	enumFrom x = enumFrom (fromEnum x)
		where
			enumFrom n = toEnum n : enumFrom (n+1)

--	enumFromThen x y = map toEnum [fromEnum x, fromEnum y ..]
	enumFromThen x y
		# nx=fromEnum x
		  ny=fromEnum y
		= toEnum nx : enumFromBy ny (ny-nx)
			where
				enumFromBy n d = toEnum n : enumFromBy (n + d) d

--	enumFromTo x y = map toEnum [fromEnum x .. fromEnum y]
	enumFromTo x y = enumFromTo (fromEnum x) (fromEnum y)
		where
			enumFromTo n e
				| n <= e
					= toEnum n : enumFromTo (n+1) e
					= []

--	enumFromThenTo x y z = map toEnum [fromEnum x, fromEnum y .. fromEnum z]
	enumFromThenTo x y z
		# nx=fromEnum x
		  ny=fromEnum y
		  nz=fromEnum z
		| nx <= ny
			= enumFromByTo nx (ny - nx) nz
			= enumFromByDownTo nx (ny - nx) nz
	where
		enumFromByTo n d e
			| n<=e
				= toEnum n : enumFromByTo (n + d) d e
				= []
		enumFromByDownTo n d e
			| n>=e
				= toEnum n : enumFromByDownTo (n + d) d e
				= []

class
  Bounded a where
	minBound :: a
	maxBound :: a

class (Eq a,Show a) =>
  Num a where
	(+) infixl 6 :: a -> a -> a
	(-) infixl 6 prefix negate :: a -> a -> a
	(*) infixl 7 :: a -> a -> a
	abs, signum :: a -> a
	negate :: a -> a
	fromInteger :: !Integer -> a

	(-) x y = x + negate y
	negate x = 0 - x

class (Num a,Ord a) =>
  Real a where
	toRational :: a -> Rational

class (Real a,Enum a) =>
  Integral a where
	toInteger :: a -> Integer
	quot infixl 7 :: a -> a -> a
	rem infixl 7 :: a -> a -> a
	div infixl 7 :: a -> a -> a
	mod infixl 7 :: a -> a -> a
	quotRem, divMod :: a -> a -> (a,a)
	quot n d = q where (q,r) = quotRem n d
	rem n d = r where (q,r) = quotRem n d
	div n d = q where (q,r) = divMod d
	mod n d = r where (q,r) = divMod d
	divMod n d = if (signum r == - signum d) (q-1,r+d) qr
					where qr@(q,r) = quotRem n d

class (Num a) =>
  Fractional a where
	(/) infixl 7 :: a -> a -> a
	recip :: a -> a
	fromRational :: Rational -> a
	recip x = 1/x
	(/) x y = x * recip x

class (Fractional a) =>
  Floating a where
	sqrt, log, exp :: a -> a
	(**) infixr 8 :: a -> a -> a
	logBase :: a -> a -> a
	sin, cos, tan :: a -> a
	asin, acos, atan :: a -> a
	sinh, cosh, tanh :: a -> a
	asinh, acosh, atanh :: a -> a
	pi :: a
	(**) x y = exp (log x * y)
	logBase x y = log y / log x
	sqrt x = x ** 0.5
	tan x = sin x / cos x
	tanh x = sinh x / cosh x

class (Real a,Fractional a) =>
  RealFrac a where
	round, truncate :: (Integral b) => a -> b
	floor, ceiling :: (Integral b) => a -> b
	properFraction :: (Integral b) => a -> (b,a)
	truncate x = m where (m,_) = properFraction x

	round x
		= let (n,r) = properFraction x
			  m = if (r<0) (n-1) (n+1)
		  in case signum (abs r-0.5) of
				-1 -> n
				0 -> if (even n) n m
				1 -> m
	ceiling x = if r > 0 then n + 1 else n
				where (n,r) = properFraction x
	floor x = if r < 0 then n - 1 else n
				where (n,r) = properFraction x

class (RealFrac a,Floating a) =>
  RealFloat a where
	floatRadix :: a -> Integer
	floatDigits :: a -> Int
	floatRange :: a -> (Int,Int)
	decodeFloat :: a -> (Integer,Int)
	encodeFloat :: Integer -> Int -> a
	exponent :: a -> Int
	significand :: a -> a
	scaleFloat :: Int -> a -> a
	isNaN, isInfinite, isDenormalized, isNegativeZero, isIEEE :: a -> Bool
	atan2 :: a -> a -> a

	exponent x = if m == 0 then 0 else n + floatDigits
					where (m,n) = decodeFloat x
	significand x = encodeFloat m (- floatDigits x)
					where (m,_) = decodeFloat x
	scaleFloat k x = encodeFloat m (n+k)
					where (m,n) = decodeFloat x
	atan2 y x
		| x>0 = atan (y/x)
		| x==0 && y>0 = pi/2
		| x<0 && y>0 = pi + atan (y/x)
		| (x<=0 && y<0) ||
		  (x<0 && isNegativeZero y) ||
		  (isNegativeZero x && isNegativeZero y)
			= -atan2 (-y) x
		| y==0 && (x<0 || isNegativeZero x)
			= pi -- must be after the previous test on zero y
		| x==0 && y==0 = y -- must be after the other double zero tests
		| otherwise = x + y -- x or y is a NaN, return a NaN (via +)
	-- Minimal complete definition:
	-- All except exponent, significand,
	-- scaleFloat, atan2

class Functor f where
	fmap :: (a -> b) -> f a -> f b

type ShowS = String -> String

class
  Show a where
	show :: a -> [Char]
	showsPrec :: Int -> a -> ([Char]->[Char])
	showList :: [a] -> ([Char]->[Char])

	show x = showsPrec `0 x ""
	showsPrec _ x = (show x ++)
	showList l = case l of
					[] -> showString "[]"
					(x:xs) -> \t -> showChar '[' (shows x (showl xs t))
				where
					showl [] t = showChar ']' t
					showl (x:xs) t = showChar ',' (shows x (showl xs t))

showChar :: Char -> String -> String
showString :: !String -> String -> String
--shows :: (Show a) => a -> String -> String
shows s t :== showsPrec `0 s t

type ReadS a = String -> [(a,String)]

class
  Read a where
	readsPrec :: Int -> ([Char] -> [(a,[Char])])
	readList :: ([Char] -> [([a],[Char])])

	readsOpenParensPrec :: Int -> [Char] -> [[(a,[Char])]]
	readsOpenParensPrec p s
		= readsPrec p s : case skip_spaces` s of
							'(':s -> readsOpenParensPrec p s
							_ -> []

	readList
		= readpl
		where
			readpl r
				= case skip_spaces` r of
					'[':r -> readl r
					'(':r -> read_close (readpl r)
					_ -> []

			readl s
				= case skip_spaces` s of
					']':r -> ([],r):readl_ s
					_ -> readl_ s
			
			readl_ s
				= [(x:xs,u) | (x,t) <- reads s, (xs,u) <- readl' t]
			{-
				= readl_l1 (reads s)
				where
				readl_l1 ((x,t):xts) = readl_l2 (readl' t) x xts
				readl_l1 [] = []

				readl_l2 ((xs,u):xsus) x xts = (x:xs,u) : readl_l2 xsus x xts
				readl_l2 [] x xts = readl_l1 xts
			-}
			readl' s
				= case skip_spaces` s of
					']':t -> [([],t)]
					',':t -> readl_ t
					_ -> []

			read_close ((x,t):xts)
				= case skip_spaces` t of
					')':u -> (x,u) : read_close xts
					_     -> read_close xts
			read_close []
				= []

--reads :: (Read a) => ReadS a
reads :== readsPrec `0

class (Ord a) =>
  Ix a where
	range :: (a,a) -> [a]
	index :: (a,a) -> a -> Int
	inRange :: (a,a) -> a -> Bool
	rangeSize :: (a,a) -> Int
	rangeSize b@(l,h)
		| null (range b)
			= `0
		| otherwise
			= index b h + `1
		where
		null [] = True
		null (_:_) = False

	-- NB: replacing "null (range b)" by "not (l <= h)"
	-- fails if the bounds are tuples. For example,
	-- (1,2) <= (2,1)
	-- but the range is nevertheless empty
	-- range ((1,2),(2,1)) = []

null :: ![a] -> Bool

io_to_f (IO io) :== io;
//io_to_f io :== let (IO f) = io in f;

f_to_io f :== IO f;

--do_io' main w :== let (IO io_f) = main in let (_,world) = io_f w in world

/*
:: IO a (:== *World -> *(a,*World))

io_to_f :: !(IO a) -> (*World -> *(a,*World));
f_to_io :: !(*World -> *(a,*World)) -> IO a;
*/

skip_spaces` :: !String -> String
