definition module Random where

from PreludeTypes import qualified ::IO,class Show,class Read,class Bounded(..)
from PreludeTypes import ::(),::Float,::Integer

class RandomGen g where
	genRange :: g -> (Int, Int)
	next :: g -> (Int, g)
	split :: g -> (g, g)
	genRange g = (PreludeTypes.minBound,PreludeTypes.maxBound)

::	StdGen

instance RandomGen StdGen where
	next :: !StdGen -> (!Int,!StdGen)
	genRange :: !StdGen -> (!Int,!Int)
	split :: !StdGen -> (!StdGen,!StdGen)

instance PreludeTypes.Show StdGen
instance PreludeTypes.Read StdGen

mkStdGen :: !Int -> StdGen

class Random a where
	randomR :: (RandomGen g) => (a, a) -> g -> (a, g)
	random :: (RandomGen g) => g -> (a, g)
	randomRs :: (RandomGen g) => (a, a) -> g -> [a]
	randomRs r g = x : randomRs r g' where (x,g') = randomR r g
	randoms :: (RandomGen g) => g -> [a]
	randoms g = x : randoms g' where (x,g') = random g
	randomRIO :: (a,a) -> PreludeTypes.IO a
	randomRIO range = getStdRandom (randomR range)
	randomIO :: PreludeTypes.IO a
	randomIO = getStdRandom random

instance Random Int where
	randomR :: (RandomGen g) => !(!Int,!Int) -> !g -> (!Int,!g) special g=StdGen:==randomR_int`StdGen
	random :: (RandomGen g) => !g -> (!Int,!g) special g=StdGen:==random_int`StdGen
	randoms :: (RandomGen g) => g -> [Int] special g=StdGen
	randomRs :: (RandomGen g) => (Int,Int) -> g -> [Int] special g=StdGen:==randomRs_int`StdGen
	randomRIO :: (Int,Int) -> PreludeTypes.IO Int
	randomIO :: PreludeTypes.IO Int

instance Random Integer where
	randomR :: (RandomGen g) => !(!Integer,!Integer) -> !g -> (!Integer,!g) special g=StdGen
	random :: (RandomGen g) => !g -> (!Integer,!g) special g=StdGen
	randoms :: (RandomGen g) => g -> [Integer] special g=StdGen
	randomRs :: (RandomGen g) => (Integer,Integer) -> g -> [Integer] special g=StdGen
	randomRIO :: (Integer,Integer) -> PreludeTypes.IO Integer
	randomIO :: PreludeTypes.IO Integer

instance Random Double where
	random :: RandomGen g => !g -> (!Double,!g) special g=StdGen
	randomR :: (RandomGen g) => !(!Double,!Double) -> !g -> (!Double,!g) special g=StdGen
	randoms :: (RandomGen g) => g -> [Double] special g=StdGen
	randomRs :: (RandomGen g) => (Double, Double) -> g -> [Double] special g=StdGen
	randomRIO :: (Double,Double) -> PreludeTypes.IO Double
	randomIO :: PreludeTypes.IO Double

instance Random Float where
	random :: RandomGen g => !g -> (!Float,!g) special g=StdGen
	randomR :: (RandomGen g) => !(!Float,!Float) -> !g -> (!Float,!g) special g=StdGen
	randoms :: (RandomGen g) => g -> [Float] special g=StdGen
	randomRs :: (RandomGen g) => (Float, Float) -> g -> [Float] special g=StdGen
	randomRIO :: (Float,Float) -> PreludeTypes.IO Float
	randomIO :: PreludeTypes.IO Float

instance Random Char where
	random :: (RandomGen g) => !g -> (!Char,!g) special g=StdGen
	randomR :: (RandomGen g) => !(!Char,!Char) -> !g -> (!Char,!g) special g=StdGen
	randoms :: (RandomGen g) => g -> [Char] special g=StdGen
	randomRs :: (RandomGen g) => (Char, Char) -> g -> [Char] special g=StdGen
	randomRIO :: (Char,Char) -> PreludeTypes.IO Char
	randomIO :: PreludeTypes.IO Char

instance Random Bool where
	random :: (RandomGen g) => !g -> (!Bool,!g) special g=StdGen
	randomR :: (RandomGen g) => !(!Bool,!Bool) -> !g -> (!Bool,!g) special g=StdGen
	randoms :: (RandomGen g) => g -> [Bool] special g=StdGen
	randomRs :: (RandomGen g) => (Bool, Bool) -> g -> [Bool] special g=StdGen
	randomRIO :: (Bool,Bool) -> PreludeTypes.IO Bool
	randomIO :: PreludeTypes.IO Bool

random_int`StdGen :: !StdGen -> (!Int,!StdGen)
randomR_int`StdGen :: !(!Int,!Int) -> !StdGen -> (!Int,!StdGen)
randomRs_int`StdGen :: !(!Int,Int) -> StdGen -> [Int]

newStdGen :: PreludeTypes.IO StdGen
setStdGen :: StdGen -> PreludeTypes.IO ()
getStdGen :: PreludeTypes.IO StdGen
getStdRandom :: (StdGen -> (a,StdGen)) -> PreludeTypes.IO a
