implementation module Random where

from CPUTime import unsafe`cpu_time
from PreludeInternal import n_leading_zeros,n_leading_zeros_32,n_leading_zeros_64,int_array_to_unsigned_integer

from StdInt import bitand,<<,IF_INT_64_OR_32
from StdInt import qualified >>

class RandomGen g where
	genRange :: g -> (Int, Int)
	next :: g -> (Int, g)
	split :: g -> (g, g)

class Random a where
	randomR :: (RandomGen g) => (a, a) -> g -> (a, g)
	random :: (RandomGen g) => g -> (a, g)
	randomRs :: (RandomGen g) => (a, a) -> g -> [a]
	randomRs r g = x : randomRs r g' where (x,g') = randomR r g
	randoms :: (RandomGen g) => g -> [a]
	randoms g = x : randoms g' where (x,g') = random g
	randomRIO :: (a,a) -> IO a
	randomRIO range = getStdRandom (randomR range)
	randomIO :: IO a
	randomIO = getStdRandom random

mulud :: !Int -> !Int -> (!Int,!Int);
mulud a b = `code inline {
	mulUUL
}

addLU :: !Int -> !Int -> !Int -> (!Int,!Int);
addLU _ _ _ = `code inline {
	addLU
}

ushiftr :: !Int -> !Int -> Int;
ushiftr a b = `code inline {
	shiftrU
}

divdu :: !Int -> !Int -> !Int -> (!Int,!Int);
divdu a b c = `code inline {
	divLU
}

infix 4 <+
(<+) :: !Int -> !Int -> Bool;
(<+) a b = `code inline {
	ltU
}

(>+) infix 4
(>+) a b :== b <+ a

(>=+) infix 4
(>=+) a b :== not (a <+ b)

(<=+) infix 4
(<=+) a b :== not (b <+ a)

infix 7 >>+
(>>+) :: !Int -> !Int -> Int;
(>>+) a b
	= `code inline {
		shiftrU
	}

data StdGen = StdGen !Int !Int

instance Show StdGen where
  showsPrec p (StdGen s1 s2)
  	= showsPrec p s1 . 
	  showChar ' ' .
	  showsPrec p s2

instance Read StdGen where
	readsPrec _p
		= \ r ->
			case try_read r of
				r@[_] -> r
				_   -> [stdFromString r]
	  where
		try_read r
			= do
				(s1, r1) <- readsPrec 0 r
				(s2, r2) <- readsPrec 0 r1
				return (StdGen s1 s2, r2)

		stdFromString :: String -> (StdGen, String)
		stdFromString s = (mkStdGen num, rest)
		  where
			(cs, rest) = splitAt 6 s
			num        = foldl (\a x -> fromEnum x + 3 * a) 1 cs

instance RandomGen StdGen where
	next :: !StdGen -> (!Int,!StdGen)
	next (StdGen s1 s2)
	  # (h,l) = mulud 80028 s1
		s1 = (ushiftr l 1)+(85*h)
		s1 = s1 + (((s1+85) StdInt.>>31) bitand (-2147483563))
		(h,l) = mulud 81384 s2
		s2 = (ushiftr l 1)+(249*h)
		s2 = s2 + (((s2+249) StdInt.>>31) bitand (-2147483399))
		z = s1 - s2
		z = z + ((z StdInt.>>31) bitand 2147483562)      // >=0 && <=2147483561
	  =	(z, StdGen s1 s2)

	genRange :: !StdGen -> (!Int,!Int)
	genRange g = `code inline {
		pop_a 1
		pushI 2147483561
		pushI 0
	}

	split :: !StdGen -> (!StdGen,!StdGen)
	split std@(StdGen s1 s2)
		= (StdGen new_s1 t2, StdGen t1 new_s2)
	  where
		-- no statistical foundation for this!
		new_s1
			| s1 == 2147483562 = 1
			| otherwise        = s1 + 1

		new_s2
			| s2 == 1          = 2147483398
			| otherwise        = s2 - 1

 		StdGen t1 t2 = snd (next std)

randomRIO`World :: (Random a) => (a,a) -> !*World -> (!a,!*World)
randomRIO`World range w
	# g_rng = global_rng
	#! rng = g_rng~[0]
	# (v,new_rng) = randomR range rng
	= seq (unsafe_set_global_rng g_rng new_rng)
		(v,w)

randomIO`World :: (Random a) => !*World -> (!a,!*World)
randomIO`World w
	# g_rng = global_rng
	#! rng = g_rng~[0]
	# (v,new_rng) = random rng
	= seq (unsafe_set_global_rng g_rng new_rng)
	  (v,w)

instance Random Int where
	random :: (RandomGen g) => !g -> (!Int,!g)
	random g = random_int g

	randomR :: (RandomGen g) => !(!Int,!Int) -> !g -> (!Int,!g)
	randomR (a,b) g = random_int_r a b g

	randoms :: (RandomGen g) => g -> [Int]
	randoms g
		# (x,g) = random_int g
		#! x=x
		= x : randoms g

	randomRs :: (RandomGen g) => (Int, Int) -> g -> [Int]
	randomRs r g
		# (x,g) = randomR r g
		#! x=x
		= x : randomRs r g

	randomRIO :: (Int,Int) -> IO Int
	randomRIO range
		= IO (randomRIO`World range)

	randomIO :: IO Int
	randomIO
		= IO randomIO`World

randomRs_int`StdGen :: !(!Int,Int) -> StdGen -> [Int]
randomRs_int`StdGen (l,h) g
	| l==0
		= randomRh (h+1) g
		# m=h+1-l
		| m/=0
			= randomRs l m g
			= randoms g
  where
	randomRs l m g
		# (x,g) = random_int_n m g
		#! x=x+l
		= x : randomRs l m g

	randomRh ih g
		# (x,g) = random_int_n ih g
		#! x=x
		= x : randomRh ih g

	randoms g
		# (x,g) = random_int g
		#! x=x
		= x : randoms g

B:==`2147483562
BM1:==`2147483561
BM2:==`2147483560
BM3:==`2147483559
DB:==`4294967124

random_int :: RandomGen g => !g -> (!Int,!g)
random_int g
	# (r1,g) = next g
	| r1/=snd (genRange g)
		# (r2,g) = next g
		= (r1*B+r2,g)
	# (r2,g) = next g
	| r2<2147476166 // 2147483562-7396 2147483562^2 mod 2^32 = 7396
		= (r1*B+r2,g)
		= random_int g

random_int`StdGen :: !StdGen -> (!Int,!StdGen)
random_int`StdGen g
	# (r1,g) = next g
	| r1/=snd (genRange g)
		# (r2,g) = next g
		= (r1*B+r2,g)
	# (r2,g) = next g
	| r2<2147476166 // 2147483562-7396 2147483562^2 mod 2^32 = 7396
		= (r1*B+r2,g)
		= random_int`StdGen g

random_int_r :: RandomGen g => !Int -> !Int -> !g -> (!Int,!g)
random_int_r l h g
	| l==0
		= random_int_n (h+1) g
		# m=h+1-l
		| m/=0
			# (r,g) = random_int_n m g
			= (l+r,g)
			= random_int g

randomR_int`StdGen :: !(!Int,!Int) -> !StdGen -> (!Int,!StdGen)
randomR_int`StdGen (l,h) g
	| l==0
		= random_int_n (h+1) g
		# m=h+1-l
		| m/=0
			# (r,g) = random_int_n m g
			= (l+r,g)
			= random_int g

random_int_n :: RandomGen g => !Int -> !g -> (!Int,!g)
random_int_n n g
	| n <=+ B
		# (r,g) = next g
		  a = r `rem` n
		| r-a <= B-n
			= (a,g)
			= random_int_n1 a r n g
	| n <=+ DB
		= random_int_n2 n g
		= random_int_n3 n g
where
	random_int_n1 :: RandomGen g => !Int -> !Int -> !Int -> !g -> (!Int,!g)
	random_int_n1 a r n g
	    # (s,g) = next g
		  (h,l) = mulud a B
		  (h,l) = addLU h l s
		  (a,_) = divdu h l n
		| r/=BM1
			= (a,g)
		| n-a<=+B-s // s-a<=B-n
			= (a,g)
			= random_int_n1 a s n g

	random_int_n2 :: !Int -> !g -> (!Int,!g) | RandomGen g
	random_int_n2 n g
		# (r,g) = next g
	      (s,g) = next g
		  (h,l) = mulud r B
		  (h,l) = addLU h l s
		  (a,_) = divdu h l n
		| r<BM2
			= (a,g)
		| a <=+ B
			| r<BM1 || B-s<=+a
				= (a,g)
				= random_int_n2 n g
			| B-s<=+a-B
				= (a,g)
				= random_int_n2 n g

	random_int_n3 :: RandomGen g => !Int -> !g -> (!Int,!g)
	random_int_n3 n g
		# (r,g) = next g
	      (s,g) = next g
		  (h,l) = mulud r B
		  (h,l) = addLU h l s
		  (a,_) = divdu h l n
		| r<BM3
			= (a,g)
		| a<=+B
			| r<BM1 || B-s<=+a
				= (a,g)
				= random_int_n3 n g
		| a<=+DB
			| r<BM2 || B-s<=+a-B
				= (a,g)
				= random_int_n3 n g
			| B-s <=+ a-DB
				= (a,g)
				= random_int_n3 n g

instance Random Integer where
	random :: (RandomGen g) => !g -> (!Integer,!g)
	random g
		# (i,g ) = random_int g
		= (toInteger i,g)

	randomR :: (RandomGen g) => !(!Integer,!Integer) -> !g -> (!Integer,!g)
	randomR (a,b) g
		| size a~integer_a==0 && size b~integer_a==0
			# (i,g) = random_int_r a~integer_s b~integer_s g
			= (toInteger i,g)
		| a==0
			= random_integer_n b g
			# (n,g) = random_integer_n (b-a) g
			= (a+n,g)

	randoms :: (RandomGen g) => g -> [Integer]
	randoms g
		# (x,g) = random_int g
		#! x = toInteger x
		= x : randoms g

	randomRs :: (RandomGen g) => (Integer, Integer) -> g -> [Integer]
	randomRs r g
		# (x,g) = randomR r g
		#! x=x
		= x : randomRs r g

	randomRIO :: (Integer,Integer) -> IO Integer
	randomRIO range
		= IO (randomRIO`World range)

	randomIO :: IO Integer
	randomIO
		= IO randomIO`World

random_integer_n :: (RandomGen g) => !Integer -> !g -> (!Integer,!g)
random_integer_n n g
	# s=size n~integer_a
	| s==0
		# (i,g) = random_int_n (n~integer_s+1) g
		= (toInteger i,g)
	| s==1
		# (i,g) = random_int_h n~integer_a~[0] g
		= (unsigned_int_to_integer i,g)
	| s==2
		# nl = n~integer_a~[0]
		  nh = n~integer_a~[1]
		  (rh,rl,g) = random_int2 nh nl g
		| rh==0
			= (unsigned_int_to_integer rl,g)
			= ({integer_s=0,integer_a={rl,rh}},g)
		= random_integer_ln n~integer_a s g
  where
	random_integer_ln :: (RandomGen g) => !{#Int} -> !Int -> !g -> (!Integer,!g)
	random_integer_ln na s g
		# nl = na~[s-2]
		  nh = na~[s-1]
		  (rh,rl,g) = random_int2 nh nl g
		  a = createArray s 0
		  a = {a & [s-1]=rh,[s-2]=rl}
		| rh/=nh || rl/=nl
			= fill_random_ints (s-2) a g
			= fill_random_ints2 (s-2) a na g

	fill_random_ints :: (RandomGen g) => !Int -> !*{#Int} -> !g -> (!Integer,!g)
	fill_random_ints i a g
		| i>=0
			# (r,g) = random_int g
			  a = {a & [i]=r}
			= fill_random_ints (i-1) a g
			= (int_array_to_unsigned_integer a,g)

	fill_random_ints2 :: (RandomGen g) => !Int -> !*{#Int} -> !{#Int} -> !g -> (!Integer,!g)
	fill_random_ints2 i a na g
		| i>=0
			# (r,g) = random_int g
			| r<+na~[i]
				# a = {a & [i]=r}
				= fill_random_ints (i-1) a g
			| r==na~[i]
				# a = {a & [i]=r}
				= fill_random_ints2 (i-1) a na g
				= random_integer_ln na (size na) g
			= (int_array_to_unsigned_integer a,g)

random_int_h :: RandomGen g => !Int -> !g -> (!Int,!g)
random_int_h h g
	| h/= -1
		= random_int_n (h+1) g
		= random_int g

unsigned_int_to_integer i
	| i>=0
		= {integer_s=i,integer_a={}}
		= {integer_s=0,integer_a={i}}

random_int2 :: (RandomGen g) => !Int -> !Int -> g -> (!Int,!Int,!g)
random_int2 nh nl g
	| nl==`-1
  		# (rh,g) = random_int_h nh g
		# (rl,g) = random_int g
		= (rh,rl,g)
	| nh<0
		= random_int2h nh nl g
		# nlzh = n_leading_zeros nh
		  hs = (nh<<nlzh) + (nl>>+((IF_INT_64_OR_32 64 32)-nlzh))
		= random_int2s hs nl nlzh g
  where
  	random_int2h nh nl g
  		# (rh,g) = random_int_h nh g
		# (rl,g) = random_int g
  		| rh/=nh
			= (rh,rl,g)
  		| rl<=+nl
  			= (rh,rl,g)
  			= random_int2h nh nl g

	random_int2s hs nl nlzh g
		# (rh,g) = random_int_h hs g
		  (rl,g) = random_int_n (1<<((IF_INT_64_OR_32 64 32)-nlzh)) g
		  rl = rl+(rh<<((IF_INT_64_OR_32 64 32)-nlzh))
		| rh/=hs
			# rh = rh>>+nlzh
			= (rh,rl,g)
		| rl<=+nl
			# rh = rh>>+nlzh
			= (rh,rl,g)
			= random_int2s hs nl nlzh g

instance Random Double where
	random :: RandomGen g => !g -> (!Double,!g)
	random g
	 	# (i,g) = random_int g
		= (0.5+fromIntegral i*0.23283064365386962890625E-9,g)

	randomR :: (RandomGen g) => !(!Double,!Double) -> !g -> (!Double,!g)
	randomR (l,h) g
	 	# (i,g) = random_int g
		  r = (h-l) * `2.3283064365386962890625E-10 /* 2^-32 */
		  m = `0.5 * (l+h)
		= (m+fromIntegral i*r,g)

	randoms :: (RandomGen g) => g -> [Double]
	randoms g
		# (i,g) = random_int g
		#! x=0.5+fromIntegral i*0.23283064365386962890625E-9
		= x : randoms g

	randomRs :: (RandomGen g) => (Double, Double) -> g -> [Double]
	randomRs r g
		# (x,g) = randomR r g
		#! x=x
		= x : randomRs r g

	randomRIO :: (Double,Double) -> IO Double
	randomRIO range
		= IO (randomRIO`World range)

	randomIO :: IO Double
	randomIO
		= IO randomIO`World

instance Random Float where
	random :: RandomGen g => !g -> (!Float,!g)
	random g
	 	# (i,g) = random_int g
		= (`Float (0.5+fromIntegral i*0.23283064365386962890625E-9),g)

	randomR :: (RandomGen g) => !(!Float,!Float) -> !g -> (!Float,!g)
	randomR (`Float l,`Float h) g
	 	# (i,g) = random_int g
		  r = (h-l) * `2.3283064365386962890625E-10 /* 2^-32 */
		  m = `0.5 * (l+h)
		= (`Float (m+fromIntegral i*r),g)

	randoms :: (RandomGen g) => g -> [Float]
	randoms g
		# (i,g) = random_int g
		#! x=0.5+fromIntegral i*0.23283064365386962890625E-9
		= x : randoms g

	randomRs :: (RandomGen g) => (Float, Float) -> g -> [Float]
	randomRs r g
		# (x,g) = randomR r g
		#! x=x
		= x : randomRs r g

	randomRIO :: (Float,Float) -> IO Float
	randomRIO range
		= IO (randomRIO`World range)

	randomIO :: IO Float
	randomIO
		= IO randomIO`World

instance Random Char where
	random :: (RandomGen g) => !g -> (!Char,!g)
	random g = random_char g

	randomR :: (RandomGen g) => !(!Char,!Char) -> !g -> (!Char,!g)
	randomR (a,b) g = random_char_r a b g

	randoms :: (RandomGen g) => g -> [Char]
	randoms g
		# (x,g) = random_char g
		#! x=x
		= x : randoms g

	randomRs :: (RandomGen g) => (Char, Char) -> g -> [Char]
	randomRs r g
		# (x,g) = randomR r g
		#! x=x
		= x : randomRs r g

	randomRIO :: (Char,Char) -> IO Char
	randomRIO range
		= IO (randomRIO`World range)

	randomIO :: IO Char
	randomIO
		= IO randomIO`World

random_char :: RandomGen g => !g -> (!Char,!g)
random_char g
	# (r,g) = next g
	| r<0x7FFFFF00
		= (toEnum r,g)
		# a = r bitand 0xff
		= random_char_n a r g
where
	random_char_n :: RandomGen g => !Int -> !Int -> !g -> (!Char,!g)
	random_char_n a r g
	    # (s,g) = next g
		  l = a * B + s
		  a = l bitand 0xff
		| r/=BM1
			= (toEnum a,g)
		| 256-a<=+B-s // s-a<=B-n
			= (toEnum a,g)
			= random_char_n a s g

randomR_char`StdGen :: !(!Char,!Char) -> !StdGen -> (!Char,!StdGen)
randomR_char`StdGen (lc,hc) g
	# l=fromEnum lc
	  m=fromEnum hc+1-fromEnum l
	  (r,g) = random_small_int_n m g
	= (toEnum (l+r),g)

random_char_r :: RandomGen g => !Char -> !Char -> !g -> (!Char,!g)
random_char_r lc hc g
	# l=fromEnum lc
	  m=fromEnum hc+1-fromEnum l
	  (r,g) = random_small_int_n m g
	= (toEnum (l+r),g)

random_small_int_n :: RandomGen g => !Int -> !g -> (!Int,!g)
random_small_int_n n g
	# (r,g) = next g
	  a = r `rem` n
	| r-a <= B-n
		= (a,g)
		= random_small_int_n1 a r n g
where
	random_small_int_n1 :: RandomGen g => !Int -> !Int -> !Int -> !g -> (!Int,!g)
	random_small_int_n1 a r n g
	    # (s,g) = next g
		  (h,l) = mulud a B
		  (h,l) = addLU h l s
		  (a,_) = divdu h l n
		| r/=BM1
			= (a,g)
		| n-a<=+B-s // s-a<=B-n
			= (a,g)
			= random_small_int_n1 a s n g

instance Random Bool where
	random :: (RandomGen g) => !g -> (!Bool,!g)
	random g
		# (r,g) = next g
		= (r bitand 1==1,g)

	randomR :: (RandomGen g) => !(!Bool,!Bool) -> !g -> (!Bool,!g)
	randomR (b1,b2) g
		| b1/=b2
			# (r,g) = next g
			= (r bitand 1==1,g)
			= (b1,g)

	randoms :: (RandomGen g) => g -> [Bool]
	randoms g
		# (r,g) = next g
		#! b = r bitand 1==1
		= b : randoms g

	randomRs :: (RandomGen g) => (Bool, Bool) -> g -> [Bool]
	randomRs r g
		# (x,g) = randomR r g
		#! x=x
		= x : randomRs r g

	randomRIO :: (Bool,Bool) -> IO Bool
	randomRIO range
		= IO (randomRIO`World range)

	randomIO :: IO Bool
	randomIO
		= IO randomIO`World

mkStdGen :: !Int -> StdGen
mkStdGen s
	| s < 0
		= mkStdGen (-s)
		= StdGen (s1+1) (s2+1)
  where
	(q, s1) = s `divMod` 2147483562
	s2      = q `mod` 2147483398

makeStdGen :: StdGen
makeStdGen = createStdGen (getClockSeconds * 12345 + unsafe`cpu_time)

createStdGen :: Integer -> StdGen
createStdGen s
	| s < 0
		= createStdGen (-s)
		= StdGen (fromInteger (s1+1)) (fromInteger (s2+1))
  where
	(q, s1) = s `divMod` 2147483562
	s2      = q `mod` 2147483398

getClockSeconds :: Integer
getClockSeconds
	# file_time = GetSystemTimeAsFileTime {0,0}
	  file_time_l = file_time~[0]
	  file_time_h = file_time~[1]
	  (rh,qh) = divdu 0 file_time_h 10000000
	  (rl,ql) = divdu rh file_time_l 10000000
	= unsigned_int_2_to_integer qh ql

unsigned_int_2_to_integer :: !Int -> !Int -> Integer
unsigned_int_2_to_integer h l
	| h==0
		| l>=0
			= {integer_s=l,integer_a={}}
			= {integer_s=0,integer_a={l}}
		= {integer_s=0,integer_a={l,h}};

newStdGen :: IO StdGen
newStdGen = do
	rng <- getStdGen
	let (a,b) = split rng
	setStdGen a
	return b

getStdRandom :: (StdGen -> (a,StdGen)) -> IO a
getStdRandom f = do
	rng <- getStdGen
	let (v, new_rng) = f rng
	setStdGen new_rng
	return v

setStdGen :: StdGen -> IO ()
setStdGen sgen = IO (setStdGenWorldUnit sgen)
  where
	setStdGenWorldUnit :: !StdGen -> !*World -> (!(),!*World)
	setStdGenWorldUnit sgen w
		= seq (unsafe_set_global_rng global_rng sgen)
			((),w)

getStdGen :: IO StdGen
getStdGen = IO getStdGenWorld

getStdGenWorld :: !*World -> (!StdGen,!*World)
getStdGenWorld w
	#! rng = global_rng~[0]
	= (rng,w)

type FILETIME = {#Int} -- 2

global_rng :: {!StdGen}
global_rng =: {makeStdGen}

unsafe_set_global_rng :: !{!StdGen} -> !StdGen -> {!StdGen}
unsafe_set_global_rng a g
	= `code {
		pushI 0
		update _ 1 0
	}

GetSystemTimeAsFileTime :: !FILETIME -> FILETIME
GetSystemTimeAsFileTime a
	= `code {
		push_a 0
		ccall GetSystemTimeAsFileTime@4 "PA:V:A"
	}
