implementation module Ratio where

import Prelude
from StdMisc import abort

infixl 7 %
(%) :: (Integral a) => a -> a -> Ratio a
(%) x y = reduce (x * signum y) (abs y)

reduce _ 0 = abort `"Ratio.% : zero denominator"
reduce x y = (x `quot` d) :% (y `quot` d)
	where
		d = gcd x y
		x % y = reduce (x * signum y) (abs y)

numerator :: (Integral a) => !(Ratio a) -> a
numerator (a :% b) = a;

denominator :: (Integral a) => !(Ratio a) -> a
denominator (a :% b) = b;

approxRational :: (RealFrac a) => a -> a -> Rational
approxRational x eps
	= simplest (x-eps) (x+eps)
  where
  	simplest x y
  		| y < x = simplest y x
		| x == y = xr
		| x > 0 = simplest' n d n' d'
		| y < 0 = - simplest' (-n') d' (-n) d
		| otherwise = 0 :% 1
	  where
		xr@(n:%d) = toRational x
		(n':%d') = toRational y

	simplest' n d n' d' -- assumes 0 < n%d < n'%d'
		| r == 0 = q :% 1
		| q /= q'
			 = (q+1) :% 1
		| otherwise = (q*n''+d'') :% n''
	  where
	  	(q,r) = quotRem n d
		(q',r') = quotRem n' d'
		(n'':%d'') = simplest' d' r' d r
