module System (
    getArgs,getProgName,getEnv,ExitCode(..),exitWith,exitFailure
  ) where

import _SystemArray
import PreludeInternal (char_list_to_zstring)
import PreludeInternalIOError

from StdString import class +++(..),instance +++ {#Char}
from StdString import class toString(..),instance toString Int
from StdMisc import abort
from StdInt import IF_INT_64_OR_32,<<

data ExitCode = ExitSuccess | ExitFailure Int deriving (Eq, Ord, Read, Show)

exitWith :: !ExitCode -> IO a
exitWith ExitSuccess = IO (exitWithW 0)
exitWith (ExitFailure n) = IO (exitWithW n)

exitWithW :: !Int -> !*World -> (!a,!*World)
exitWithW r w
	#! a = seq (store_int return_code_p r) (abort `"")
	= (a,w)

exitFailure :: IO a
exitFailure = IO exitFailureW

exitFailureW :: !*World -> (!a,!*World)
exitFailureW w
	#! a = seq (store_int return_code_p 1) (abort `"")
	= (a,w)

getArgs :: IO [[Char]]
getArgs
	= return (args 1 global_argc global_argv)
	where
		args arg_n argc argv
			| arg_n<argc
				# arg = c_string_to_list_of_char (load_int (argv+(arg_n<<IF_INT_64_OR_32 3 2)))
				= arg : args (arg_n+1) argc argv
				= []

getProgName :: IO [Char]
getProgName
	= return (c_string_to_list_of_char (load_int global_argv))

getEnv :: String -> IO String
getEnv var_name = IO (getEnvW var_name)

getEnvW :: !String -> !*World -> (!String,!*World)
getEnvW var_name w
	# zc_var_name = char_list_to_zstring var_name
	  a = createArray 64 '\0'
	  r = GetEnvironmentVariable zc_var_name a 64
	| r>0
		| r<64
			= (begin_string_to_char_list r a,w)
			# a = createArray r '\0'
			  r = GetEnvironmentVariable zc_var_name a r
			| r>=0
				= (begin_string_to_char_list r a,w)
				= abort `"GetEnv"
	| r==0
		| GetLastError/=ERROR_ENVVAR_NOT_FOUND
			= ("",w)
			# (IO io) = ioError DoesNotExistError
			= io w
		= abort `"GetEnv"

begin_string_to_char_list :: !Int -> !{#Char} -> [Char]
begin_string_to_char_list n s
	= begin_string_to_char_list 0 n s
  where
	begin_string_to_char_list :: !Int -> !Int -> !{#Char} -> [Char]
	begin_string_to_char_list i n s
		| i<n
			#! c=s~[i]
			= c:begin_string_to_char_list (i+1) n s
			= []	

c_string_to_list_of_char p
	# c=load_char p
	| c=='\0'
		= []
		= c:c_string_to_list_of_char (p+1)

ERROR_ENVVAR_NOT_FOUND:==203

GetEnvironmentVariable :: !{#Char} -> !{#Char} -> !Int -> Int
GetEnvironmentVariable name buffer buffer_size = `code {
	ccall GetEnvironmentVariableA@12 "PssI:I"
}

GetLastError :: Int
GetLastError = `code {
	ccall GetLastError@0 "P:I"
}

global_argc :: Int
global_argc = `code {
	pushLc global_argc
	load_i 0
}

global_argv :: Int
global_argv = `code {
	pushLc global_argv
	load_i 0
}

return_code_p :: Int
return_code_p = `code {
	pushLc return_code
}

load_int :: !Int -> Int
load_int p = `code {
	load_i 0
}

load_char :: !Int -> Char
load_char p = `code {
	load_ui8 0
}

store_int :: !Int -> !Int -> Int
store_int p v = `code {
	:store_int_0
	|	movl	%eax,(%ebx)
	instruction 137
	instruction 3
	:store_int_1
	update_b 0 1
	pop_b 1
}

